#QUALTRICS DATA ANALYSIS

#run this section first

#list of required packages:
#no need to run install after first time
##np [non parametrics]
##dgof [for discrete ks tests]
##stats4 [for mle]
##strucchange [for structural change regressions]
##sandwich [for heteroskedasticity-consistent estimation, default is HC3]
##lmtest [for hypothesis tests with HC estimates]
##stargazer [for latex output]
##bbmle [for better mle]
##ggplot [for better plotting]
##locfit [for local likelihood estimation]
##sm [does local likelihood estimation and has nonparametric tests for monotonicity]
##nleqslv [for nonlinear equation solving]
##stringr [for string-related functions, esp. substrings]
##multiwayvcov [for clustered standard errors]
##diptest [for hartigans' dip test]

install.packages("np");
install.packages("dgof");
#install.packages("stats4");
install.packages("strucchange");
install.packages("sandwich");
install.packages("lmtest");
install.packages("stargazer");
install.packages("bbmle");
install.packages("ggplot2");
install.packages("locfit");
install.packages("sm");
install.packages("nleqslv");
install.packages("stringr");
install.packages("multiwayvcov");
install.packages("diptest");

library(np);
library(dgof);
library(stats4);
library(strucchange);
library(sandwich);
library(lmtest);
library(stargazer);
library(bbmle);
library(ggplot2);
library(locfit);
library(sm);
library(nleqslv);
library(stringr);
library(multiwayvcov);
library(diptest);

#set parameters
num_tasks <- 100;

dot_counts <- 38:42;
ang_counts <- c(35,40,45,50,55);

#set lpm weighting function
#0.001 and 0.999 are ad hoc choices
lpm_weights <- function(x) {
	if(x <= 0) {
		fit <- 0.001;
	} else if (x >= 1) {
		fit <- 0.999;
	} else {
		fit <- x;
	}
	return(sqrt(fit*(1-fit)));
}

#set vector of incentives
incentives <- c(2*(1:50)-1,2*(1:50));

#read and import data
#only use lines belonging to actual subjects
##$10
data_all_10 <- read.delim("dots_angle_10p.txt"); #change file name according to what it is
data_10 <- data_all_10[-1,];
subjects_10 <- dim(data_10)[1]; #number of subjects at prize 10

##$20
data_all_20 <- read.delim("dots_angle_20p.txt"); #change file name according to what it is
data_20 <- data_all_20[-1,];
subjects_20 <- dim(data_20)[1]; #number of subjects at prize 20

data <- rbind(data_10,data_20);
n_subj <- subjects_10+subjects_20;
is20 <- c(rep(0,subjects_10),rep(1,subjects_20));

#DOTS
#read dots data
true_dots <- data["trueDots"];
inc_dots <- data["incDots"];
chose_dots <- data["choseDots"];

true_dots <- as.vector(unlist(apply(true_dots,1,strsplit,split=".",fixed=TRUE)));
dim(true_dots) <- c(num_tasks+1,n_subj);
true_dots <- t(true_dots)[,-1];
class(true_dots) <- "numeric";

inc_dots <- as.vector(unlist(apply(inc_dots,1,strsplit,split=".",fixed=TRUE)));
dim(inc_dots) <- c(num_tasks+1,n_subj);
inc_dots <- t(inc_dots)[,-1];
class(inc_dots) <- "numeric";

chose_dots <- as.vector(unlist(apply(chose_dots,1,strsplit,split=".",fixed=TRUE)));
dim(chose_dots) <- c(num_tasks+1,n_subj);
chose_dots <- t(chose_dots)[,-1];
class(chose_dots) <- "numeric";

#check if correct
correct_dots <- (true_dots == chose_dots) + 0;

#get distances
dist_dots <- chose_dots - true_dots;
abs_dist_dots <- abs(dist_dots);


#ANGLES
#read angle data
true_ang <- data["trueAngle"];
inc_ang <- data["incAngle"];
chose_ang <- data["choseAngle"];

#split angle data strings
#n_subj <- length(true_ang[,1]);

true_ang <- as.vector(unlist(apply(true_ang,1,strsplit,split=".",fixed=TRUE)));
dim(true_ang) <- c(num_tasks+1,n_subj);
true_ang <- t(true_ang)[,-1];
class(true_ang) <- "numeric";

inc_ang <- as.vector(unlist(apply(inc_ang,1,strsplit,split=".",fixed=TRUE)));
dim(inc_ang) <- c(num_tasks+1,n_subj);
inc_ang <- t(inc_ang)[,-1];
class(inc_ang) <- "numeric";

chose_ang <- as.vector(unlist(apply(chose_ang,1,strsplit,split=".",fixed=TRUE)));
dim(chose_ang) <- c(num_tasks+1,n_subj);
chose_ang <- t(chose_ang)[,-1];
class(chose_ang) <- "numeric";

#check if correct
correct_ang <- (true_ang == chose_ang) + 0;

#get distances
dist_ang <- chose_ang - true_ang;
abs_dist_ang <- abs(dist_ang);


#DEMOGRAPHICS
#get relevant demographic variables, reshape them into matrices and aggregate vectors
age <- as.numeric(as.vector(unlist(data["QID9_TEXT"])));
dim(age) <- c(n_subj,1);
age_mat <- apply(age,1,matrix,1,num_tasks);
agg_age <- as.vector(age_mat);
mean(age,na.rm=TRUE);
sd(age,na.rm=TRUE)*sqrt(1-1/length(age[!is.na(age)])); #st dev, getting rid of sample correction

#gender
gender <- as.vector(unlist(data["QID39"]));
male <- (gender == "Male") + 0;
male[which(gender == "")] <- NA;
dim(male) <- c(n_subj,1);
male_mat <- apply(male,1,matrix,1,num_tasks); #make matrix for gender for aggregate regressions
agg_male <- as.vector(male_mat);
table(gender); #gender data

#level of education
lvl_edu <- as.vector(unlist(data["QID40"]));
dim(lvl_edu) <- c(n_subj,1);
edu_mat <- apply(lvl_edu,1,matrix,1,num_tasks);
agg_edu <- as.vector(edu_mat);
bac_mat <- (substr(edu_mat,1,1) %in% c("B","M","D")) + 0; #does subject at least have bachelors degree
agg_bac <- as.vector(bac_mat); 
table(lvl_edu);

#area of study - identify subjects who study econ, psych, or neuro
area_study <- as.vector(unlist(data["QID41_TEXT"]));
dim(area_study) <- c(n_subj,1);
study_mat <- apply(area_study,1,matrix,1,num_tasks);
epn_agg <- grepl("(econ|psych|neuro)",study_mat,ignore.case=TRUE)+0;
epn_mat <- t(matrix(epn_agg,nrow=num_tasks,ncol=n_subj));
epn <- epn_mat[,1];

#date dummies
#dates are 06/01 (baseline), 06/08, 06/10, 07/18, 10/19, 10/20
date <- as.vector(unlist(data["endDate"]));
dim(date) <- c(n_subj,1);
date_mat <- apply(date,1,matrix,1,num_tasks);

date0608_mat <- (substr(date_mat,6,10)=="06-08")+0;
date0608_agg <- as.vector(date0608_mat);
date0610_mat <- (substr(date_mat,6,10)=="06-10")+0;
date0610_agg <- as.vector(date0610_mat);
date0718_mat <- (substr(date_mat,6,10)=="07-18")+0;
date0718_agg <- as.vector(date0718_mat);
date1019_mat <- (substr(date_mat,6,10)=="10-19")+0;
date1019_agg <- as.vector(date1019_mat);
date1020_mat <- (substr(date_mat,6,10)=="10-20")+0;
date1020_agg <- as.vector(date1020_mat);

#$20 prize
is20_agg <- c(rep(0,subjects_10*num_tasks),rep(1,subjects_20*num_tasks));

#dots or angles first?
dots_first <- as.numeric(as.vector(unlist(data["dotsFirst"])));
dim(dots_first) <- c(n_subj,1);
dots_first_mat <- apply(dots_first,1,matrix,1,num_tasks);
agg_dots_first <- as.vector(dots_first_mat);

#task order
task_num <- rep(1:100,n_subj);

#subject id
subj_id_mat <- apply(as.matrix(1:n_subj),1,matrix,1,num_tasks);
subj_id_agg <- as.vector(subj_id_mat);


#REACTION TIMES
#react is total time spent on page; first is time until first click
react_odd_dots <- data[which(str_sub(names(data),-17,-1) == "QID23_PAGE_SUBMIT")];
first_odd_dots <- data[which(str_sub(names(data),-17,-1) == "QID23_FIRST_CLICK")];
react_even_dots <- data[which(str_sub(names(data),-17,-1) == "QID25_PAGE_SUBMIT")];
first_even_dots <- data[which(str_sub(names(data),-17,-1) == "QID25_FIRST_CLICK")];
react_odd_ang <- data[which(str_sub(names(data),-17,-1) == "QID27_PAGE_SUBMIT")];
first_odd_ang <- data[which(str_sub(names(data),-17,-1) == "QID27_FIRST_CLICK")];
react_even_ang <- data[which(str_sub(names(data),-17,-1) == "QID29_PAGE_SUBMIT")];
first_even_ang <- data[which(str_sub(names(data),-17,-1) == "QID29_FIRST_CLICK")];

#put reaction time data into matrices
react_dots <- matrix(as.numeric(cbind(as.matrix(react_odd_dots),as.matrix(react_even_dots))),n_subj,num_tasks);
first_dots <- matrix(as.numeric(cbind(as.matrix(first_odd_dots),as.matrix(first_even_dots))),n_subj,num_tasks);
react_ang <- matrix(as.numeric(cbind(as.matrix(react_odd_ang),as.matrix(react_even_ang))),n_subj,num_tasks);
first_ang <- matrix(as.numeric(cbind(as.matrix(first_odd_ang),as.matrix(first_even_ang))),n_subj,num_tasks);

#reorder reaction times by order of task completion
react_reorder_dots <- matrix(nrow=n_subj,ncol=num_tasks);
first_reorder_dots <- matrix(nrow=n_subj,ncol=num_tasks);
react_reorder_ang <- matrix(nrow=n_subj,ncol=num_tasks);
first_reorder_ang <- matrix(nrow=n_subj,ncol=num_tasks);

for(i in 1:n_subj) {
	for(j in 1:num_tasks) {
		inc <- inc_dots[i,j];
		if(inc %% 2 == 0) {
			react_reorder_dots[i,j] <- react_dots[i,inc/2+50];
			first_reorder_dots[i,j] <- first_dots[i,inc/2+50];
			react_reorder_ang[i,j] <- react_ang[i,inc/2+50];
			first_reorder_ang[i,j] <- first_ang[i,inc/2+50];
		} else {
			react_reorder_dots[i,j] <- react_dots[i,(inc+1)/2];
			first_reorder_dots[i,j] <- first_dots[i,(inc+1)/2];
			react_reorder_ang[i,j] <- react_ang[i,(inc+1)/2];
			first_reorder_ang[i,j] <- first_ang[i,(inc+1)/2];
		}
	}
}

#put into aggregate vector for easy calculations
react_all_dots <- as.vector(t(react_reorder_dots));
react_all_ang <- as.vector(t(first_reorder_dots));
first_all_dots <- as.vector(t(react_reorder_ang));
first_all_ang <- as.vector(t(first_reorder_ang));

#####END OF PREAMBLE CODE
#In what follows, blocks separated by a row of pound signs run independently.

######

######
######
#SUMMARY STATISTICS
######
#REACTION TIMES
mean(react_all_dots);
mean(react_all_ang);
mean(first_all_dots);
mean(first_all_ang);
#histograms
hist(react_all_dots,xlab="Reaction time (s)",ylab="Percentage of responses",main="",freq=FALSE,breaks=3*(0:120));
hist(react_all_ang,xlab="Reaction time (s)",ylab="Percentage of responses",main="",freq=FALSE,breaks=3*(0:60));
hist(first_all_dots,xlab="Time to first click (s)",ylab="Percentage of responses",main="",freq=FALSE,breaks=3*(0:120));
hist(first_all_ang,xlab="Time to first click (s)",ylab="Percentage of responses",main="",freq=FALSE,breaks=3*(0:45));

#better histogram
hist(react_all_dots[which(react_all_dots<=150)],xlab="Reaction time (s)",ylab="Percentage of responses",main="",freq=FALSE,breaks=3*(0:60));


######
######
#AGGREGATE ANALYSIS
#standard errors with HC3 heteroskedasticity correction
######
#dots
agg_correct_dots <- as.vector(t(correct_dots));
agg_inc_dots <- as.vector(t(inc_dots));

dots_data_agg <- data.frame(agg_correct_dots,agg_inc_dots,agg_age,agg_male,agg_bac,epn_agg,is20_agg,agg_dots_first,date0608_agg,date0610_agg,date0718_agg,date1019_agg,date1020_agg,task_num,subj_id_agg,react_all_dots);

#mean performance
mean_agg_dots <- mean(agg_correct_dots);
mean_agg_dots;
#linear regression
agg_reg_dots <- lm(agg_correct_dots ~ agg_inc_dots, na.action = na.exclude, data=dots_data_agg);
agg_reg_dem_dots <- lm(agg_correct_dots ~ agg_inc_dots + agg_age + agg_male + agg_bac + epn_agg + is20_agg + agg_dots_first + task_num, na.action = na.exclude, data=dots_data_agg);

#distance regression
agg_dist_reg_dots <- lm(as.vector(t(abs_dist_dots)) ~ as.vector(t(inc_dots)));
agg_dist_reg_dots_test <- coeftest(agg_dist_reg_dots,vcovHC(agg_dist_reg_dots));
agg_dist_reg_dots_test;
#plot of distance
barplot(table(abs_dist_dots)/(num_tasks*n_subj),xlab="Distance from True State",ylab="Percentage of Responses");

#weighted least squares
weights_dots <- apply(as.matrix(fitted(agg_reg_dots)),1,lpm_weights);
agg_wls_dots <- lm(agg_correct_dots ~ agg_inc_dots, weights=weights_dots);
#weights_dem_dots <- apply(as.matrix(fitted(agg_reg_dem_dots)),1,lpm_weights); #redo this line when the na.exclude issue is fixed
weights_dem_dots <- sqrt(fitted(agg_reg_dem_dots)*(1-fitted(agg_reg_dem_dots)));
agg_wls_dem_dots <- lm(agg_correct_dots ~ agg_inc_dots + agg_age + agg_male + agg_bac + epn_agg + is20_agg + agg_dots_first + task_num, weights=weights_dem_dots, data=dots_data_agg);

#cluster standard errors
agg_cluster_dots <- coeftest(agg_reg_dots,cluster.vcov(agg_reg_dots,subj_id_agg));
agg_cluster_dem_dots <- coeftest(agg_reg_dem_dots,cluster.vcov(agg_reg_dem_dots,subj_id_agg));

######
#angle
agg_correct_ang <- as.vector(t(correct_ang));
agg_inc_ang <- as.vector(t(inc_ang));

ang_data_agg <- data.frame(agg_correct_ang,agg_inc_ang,agg_age,agg_male,agg_bac,epn_agg,is20_agg,agg_dots_first,date0608_agg,date0610_agg,date0718_agg,date1019_agg,date1020_agg,task_num,subj_id_agg,react_all_ang);

#mean performance
mean_agg_ang <- mean(agg_correct_ang);
mean_agg_ang;
#linear regression
agg_reg_ang <- lm(agg_correct_ang ~ agg_inc_ang);
agg_reg_dem_ang <- lm(agg_correct_ang ~ agg_inc_ang + agg_age + agg_male + agg_bac + epn_agg + is20_agg + agg_dots_first + date0608_agg + date0610_agg + date0718_agg + date1019_agg + date1020_agg + task_num, na.action = na.exclude, data=ang_data_agg);

#distance regression
agg_dist_reg_ang <- lm(as.vector(abs_dist_ang) ~ as.vector(inc_ang));
agg_dist_reg_ang_test <- coeftest(agg_dist_reg_ang,vcovHC(agg_dist_reg_ang));
agg_dist_reg_ang_test;
#plot of distance
barplot(table(abs_dist_ang)/(num_tasks*n_subj),xlab="Distance from True State",ylab="Percentage of Responses");

#weighted least squares
weights_ang <- apply(as.matrix(agg_reg_ang$fitted.values),1,lpm_weights);
agg_wls_ang <- lm(agg_correct_ang ~ agg_inc_ang, weights=weights_ang);
#weights_dem_ang <- apply(as.matrix(fitted(agg_reg_dem_ang)),1,lpm_weights); #redo this line when the na.exclude issue is fixed
weights_dem_ang <- sqrt(fitted(agg_reg_dem_ang)*(1-fitted(agg_reg_dem_ang)));
agg_wls_dem_ang <- lm(agg_correct_ang ~ agg_inc_ang + agg_age + agg_male + agg_bac + epn_agg + is20_agg + agg_dots_first + date0608_agg + date0610_agg + date0718_agg + date1019_agg + date1020_agg + task_num, weights=weights_dem_dots, data=ang_data_agg);

#cluster standard errors
agg_cluster_ang <- coeftest(agg_reg_dem_ang,cluster.vcov(agg_reg_dem_ang,subj_id_agg));


####################
####################

#INDIVIDUAL ANALYSIS

######
######
#Constant regressions (means with associated likelihoods)
######
#dots
mean_reg_dots <- as.list(mat.or.vec(n_subj,1));
mean_reg_sum_dots <- as.list(mat.or.vec(n_subj,1));
for(i in 1:n_subj) {
	mean_reg_dots[[i]] <- lm(correct_dots[i,] ~ 1);
	mean_reg_sum_dots[[i]] <- summary(mean_reg_dots[[i]]);
}

######
#angle
mean_reg_ang <- as.list(mat.or.vec(n_subj,1));
mean_reg_sum_ang <- as.list(mat.or.vec(n_subj,1));
for(i in 1:n_subj) {
	mean_reg_ang[[i]] <- lm(correct_ang[i,] ~ 1);
	mean_reg_sum_ang[[i]] <- summary(mean_reg_ang[[i]]);
}



######
######
#LPMs
#standard errors with and without heteroskedasticity correction
######
#dots
mean_dots <- mat.or.vec(n_subj,1);
reg_dots <- as.list(mat.or.vec(n_subj,1)); #no het correction
reg_sum_dots <- as.list(mat.or.vec(n_subj,1));
lpm_weights_dots <-  mat.or.vec(n_subj,num_tasks);
lpm_reg_dots <- as.list(mat.or.vec(n_subj,1)); #het correction
lpm_reg_sum_dots <- as.list(mat.or.vec(n_subj,1));
for(i in 1:n_subj) {
	#x11();
	#plot(correct_dots[i,] ~ inc_dots[i,], main=paste("Dots: subject ",i),xlab="Incentive Level",ylab="Correct",pch=20);
	mean_dots[i] <- mean(correct_dots[i,]);
	reg_dots[[i]] <- lm(correct_dots[i,] ~ inc_dots[i,]);
	reg_sum_dots[[i]] <- summary(reg_dots[[i]]);
	lpm_weights_dots[i,] <- apply(as.matrix(fitted(reg_dots[[i]])),1,lpm_weights);
	lpm_reg_dots[[i]] <- lm(correct_dots[i,] ~ inc_dots[i,], weights=lpm_weights_dots[i,]);
	lpm_reg_sum_dots[[i]] <- summary(lpm_reg_dots[[i]]);
}

######
#angle
mean_ang <- mat.or.vec(n_subj,1);
reg_ang <- as.list(mat.or.vec(n_subj,1)); #no het correction
reg_sum_ang <- as.list(mat.or.vec(n_subj,1));
lpm_weights_ang <-  mat.or.vec(n_subj,num_tasks);
lpm_reg_ang <- as.list(mat.or.vec(n_subj,1)); #het correction
lpm_reg_sum_ang <- as.list(mat.or.vec(n_subj,1));
for(i in 1:n_subj) {
	#x11();
	#plot(correct_ang[i,] ~ inc_ang[i,], main=paste("Angle: subject ",i),xlab="Incentive Level",ylab="Correct",pch=20);
	mean_ang[i] <- mean(correct_ang[i,]);
	reg_ang[[i]] <- lm(correct_ang[i,] ~ inc_ang[i,]);
	reg_sum_ang[[i]] <- summary(reg_ang[[i]]);
	lpm_weights_ang[i,] <- apply(as.matrix(fitted(reg_ang[[i]])),1,lpm_weights);
	lpm_reg_ang[[i]] <- lm(correct_ang[i,] ~ inc_ang[i,], weights=lpm_weights_ang[i,]);
	lpm_reg_sum_ang[[i]] <- summary(lpm_reg_ang[[i]]);
}


######
######
#POLYNOMIAL LPMs (2nd and 3rd)
#standard errors with and without heteroskedasticity correction
######
#dots
reg2_dots <- as.list(mat.or.vec(n_subj,1));
lpm2_weights_dots <-  mat.or.vec(n_subj,num_tasks);
lpm2_reg_dots <- as.list(mat.or.vec(n_subj,1)); #het correction
lpm2_reg_sum_dots <- as.list(mat.or.vec(n_subj,1));
reg3_dots <- as.list(mat.or.vec(n_subj,1));
lpm3_weights_dots <-  mat.or.vec(n_subj,num_tasks);
lpm3_reg_dots <- as.list(mat.or.vec(n_subj,1)); #het correction
lpm3_reg_sum_dots <- as.list(mat.or.vec(n_subj,1));
for(i in 1:n_subj) {
	reg2_dots[[i]] <- lm(correct_dots[i,] ~ inc_dots[i,] + I(inc_dots[i,]^2));
	lpm2_weights_dots[i,] <- apply(as.matrix(fitted(reg2_dots[[i]])),1,lpm_weights);
	lpm2_reg_dots[[i]] <- lm(correct_dots[i,] ~ inc_dots[i,] + I(inc_dots[i,]^2), weights=lpm2_weights_dots[i,]);
	lpm2_reg_sum_dots[[i]] <- summary(lpm2_reg_dots[[i]]);

	reg3_dots[[i]] <- lm(correct_dots[i,] ~ inc_dots[i,] + I(inc_dots[i,]^2) + I(inc_dots[i,]^3));
	lpm3_weights_dots[i,] <- apply(as.matrix(fitted(reg3_dots[[i]])),1,lpm_weights);
	lpm3_reg_dots[[i]] <- lm(correct_dots[i,] ~ inc_dots[i,] + I(inc_dots[i,]^2) + I(inc_dots[i,]^3), weights=lpm3_weights_dots[i,]);
	lpm3_reg_sum_dots[[i]] <- summary(lpm3_reg_dots[[i]]);
}

######
#angle
reg2_ang <- as.list(mat.or.vec(n_subj,1));
lpm2_weights_ang <-  mat.or.vec(n_subj,num_tasks);
lpm2_reg_ang <- as.list(mat.or.vec(n_subj,1)); #het correction
lpm2_reg_sum_ang <- as.list(mat.or.vec(n_subj,1));
reg3_ang <- as.list(mat.or.vec(n_subj,1));
lpm3_weights_ang <-  mat.or.vec(n_subj,num_tasks);
lpm3_reg_ang <- as.list(mat.or.vec(n_subj,1)); #het correction
lpm3_reg_sum_ang <- as.list(mat.or.vec(n_subj,1));
for(i in 1:n_subj) {
	reg2_ang[[i]] <- lm(correct_ang[i,] ~ inc_ang[i,] + I(inc_ang[i,]^2));
	lpm2_weights_ang[i,] <- apply(as.matrix(fitted(reg2_ang[[i]])),1,lpm_weights);
	lpm2_reg_ang[[i]] <- lm(correct_ang[i,] ~ inc_ang[i,] + I(inc_ang[i,]^2), weights=lpm2_weights_ang[i,]);
	lpm2_reg_sum_ang[[i]] <- summary(lpm2_reg_ang[[i]]);

	reg3_ang[[i]] <- lm(correct_ang[i,] ~ inc_ang[i,] + I(inc_ang[i,]^2) + I(inc_ang[i,]^3));
	lpm3_weights_ang[i,] <- apply(as.matrix(fitted(reg3_ang[[i]])),1,lpm_weights);
	lpm3_reg_ang[[i]] <- lm(correct_ang[i,] ~ inc_ang[i,] + I(inc_ang[i,]^2) + I(inc_ang[i,]^3), weights=lpm3_weights_ang[i,]);
	lpm3_reg_sum_ang[[i]] <- summary(lpm3_reg_ang[[i]]);
}



######
######
#BREAKPOINT ANALYSIS (allowing for slope)
######
#dots
breaks_dots <- as.list(mat.or.vec(n_subj,1)); #has breakpoints in the list element $breakpoints
exceeds_break_dots <- matrix(nrow=n_subj,ncol=num_tasks); #does incentive exceed break
exceeds_break_inc_dots <- matrix(nrow=n_subj,ncol=num_tasks); #incentive if exceeds break, else 0
breaks_reg_dots <- as.list(mat.or.vec(n_subj,1)); #linear regression results with breaks, if they exist
breaks_reg_dots_sum <- as.list(mat.or.vec(n_subj,1)); #linear regression summary with breaks, if they exist
for(i in 1:n_subj) {
	breaks_dots[[i]] <- breakpoints(correct_dots[i,] ~ inc_dots[i,],breaks=1);
	if(is.numeric(breaks_dots[[i]]$breakpoints)) {
		exceeds_break_dots[i,] <- (inc_dots[i,] >= breaks_dots[[i]]$breakpoints)+0;
		exceeds_break_inc_dots[i,] <- exceeds_break_dots[i,]*inc_dots[i,];
		breaks_reg_dots[[i]] <- lm(correct_dots[i,] ~ inc_dots[i,] + exceeds_break_dots[i,] + exceeds_break_inc_dots[i,]);
	}
	else {
		breaks_reg_dots[[i]] <- lm(correct_dots[i,] ~ inc_dots[i,]);
	}
	breaks_reg_dots_sum[[i]] <- summary(breaks_reg_dots[[i]]);
}

######
#angle
breaks_ang <- as.list(mat.or.vec(n_subj,1)); #has breakpoints in the list element $breakpoints
exceeds_break_ang <- matrix(nrow=n_subj,ncol=num_tasks); #does incentive exceed break
exceeds_break_inc_ang <- matrix(nrow=n_subj,ncol=num_tasks); #incentive if exceeds break, else 0
breaks_reg_ang <- as.list(mat.or.vec(n_subj,1)); #linear regression results with breaks, if they exist
breaks_reg_ang_sum <- as.list(mat.or.vec(n_subj,1)); #linear regression summary with breaks, if they exist
for(i in 1:n_subj) {
	breaks_ang[[i]] <- breakpoints(correct_ang[i,] ~ inc_ang[i,],breaks=1);
	if(is.numeric(breaks_ang[[i]]$breakpoints)) {
		exceeds_break_ang[i,] <- (inc_ang[i,] >= breaks_ang[[i]]$breakpoints)+0;
		exceeds_break_inc_ang[i,] <- exceeds_break_ang[i,]*inc_ang[i,];
		breaks_reg_ang[[i]] <- lm(correct_ang[i,] ~ inc_ang[i,] + exceeds_break_ang[i,] + exceeds_break_inc_ang[i,]);
	}
	else {
		breaks_reg_ang[[i]] <- lm(correct_ang[i,] ~ inc_ang[i,]);
	}
	breaks_reg_ang_sum[[i]] <- summary(breaks_reg_ang[[i]]);
}


######
######
#BREAKPOINT ANALYSIS (no slope)
######
#dots
inc_dummies_dots <- array(dim=c(n_subj,num_tasks,num_tasks)); #creates vectors of dummies for whether or not the incentive level is at least the dummy level
breaks_reg_noslope_dots <- as.list(mat.or.vec(n_subj,1)); #stores regression results for each dummy for each person
breaks_reg_noslope_r2_dots <- matrix(nrow=n_subj,ncol=num_tasks); #store r squareds for each regression
breaks_reg_noslope_optinc_dots <- vector(length=n_subj); #stores optimal incentive level for break
breaks_reg_noslope_opt_dots <- as.list(mat.or.vec(n_subj,1)); #stores regression results with the highest r2 for each person
breaks_reg_noslope_opt_dots_sum <- as.list(mat.or.vec(n_subj,1)); #stores regression summary with the highest r2 for each person
for(i in 1:n_subj) {
	breaks_reg_noslope_dots[[i]] <- as.list(mat.or.vec(num_tasks,1));
	for(j in 1:num_tasks) {
		#populate inc_dummies_dots for subject i
		inc_dummies_dots[i,,j] <- (inc_dots[i,] >= j)+0;
		#run regressions and store r squareds
		breaks_reg_noslope_dots[[i]][[j]] <- lm(correct_dots[i,] ~ inc_dummies_dots[i,,j]);
		breaks_reg_noslope_r2_dots[i,j] <- summary(breaks_reg_noslope_dots[[i]][[j]])$r.squared;
	}
	#find optimal r squared
	breaks_reg_noslope_optinc_dots[i] <- which.max(breaks_reg_noslope_r2_dots[i,]);
	#store this regression
	breaks_reg_noslope_opt_dots[[i]] <- breaks_reg_noslope_dots[[i]][[breaks_reg_noslope_optinc_dots[i]]];
	breaks_reg_noslope_opt_dots_sum[[i]] <- summary(breaks_reg_noslope_opt_dots[[i]]);
}

#angle
inc_dummies_ang <- array(dim=c(n_subj,num_tasks,num_tasks)); #creates vectors of dummies for whether or not the incentive level is at least the dummy level
breaks_reg_noslope_ang <- as.list(mat.or.vec(n_subj,1)); #stores regression results for each dummy for each person
breaks_reg_noslope_r2_ang <- matrix(nrow=n_subj,ncol=num_tasks); #store r squareds for each regression
breaks_reg_noslope_optinc_ang <- vector(length=n_subj); #stores optimal incentive level for break
breaks_reg_noslope_opt_ang <- as.list(mat.or.vec(n_subj,1)); #stores regression results with the highest r2 for each person
breaks_reg_noslope_opt_ang_sum <- as.list(mat.or.vec(n_subj,1)); #stores regression summary with the highest r2 for each person
for(i in 1:n_subj) {
	breaks_reg_noslope_ang[[i]] <- as.list(mat.or.vec(num_tasks,1));
	for(j in 1:num_tasks) {
		#populate inc_dummies_ang for subject i
		inc_dummies_ang[i,,j] <- (inc_ang[i,] >= j)+0;
		#run regressions and store r squareds
		breaks_reg_noslope_ang[[i]][[j]] <- lm(correct_ang[i,] ~ inc_dummies_ang[i,,j]);
		breaks_reg_noslope_r2_ang[i,j] <- summary(breaks_reg_noslope_ang[[i]][[j]])$r.squared;
	}
	#find optimal r squared
	breaks_reg_noslope_optinc_ang[i] <- which.max(breaks_reg_noslope_r2_ang[i,]);
	#store this regression
	breaks_reg_noslope_opt_ang[[i]] <- breaks_reg_noslope_ang[[i]][[breaks_reg_noslope_optinc_ang[i]]];
	breaks_reg_noslope_opt_ang_sum[[i]] <- summary(breaks_reg_noslope_opt_ang[[i]]);
}


######
######
#BREAKPOINT ANALYSIS (constant slope)
######
#dots
inc_dummies_dots <- array(dim=c(n_subj,num_tasks,num_tasks)); #creates vectors of dummies for whether or not the incentive level is at least the dummy level
breaks_reg_cslope_dots <- as.list(mat.or.vec(n_subj,1)); #stores regression results for each dummy for each person
breaks_reg_cslope_r2_dots <- matrix(nrow=n_subj,ncol=num_tasks); #store r squareds for each regression
breaks_reg_cslope_optinc_dots <- vector(length=n_subj); #stores optimal incentive level for break
breaks_reg_cslope_opt_dots <- as.list(mat.or.vec(n_subj,1)); #stores regression results with the highest r2 for each person
breaks_reg_cslope_opt_dots_sum <- as.list(mat.or.vec(n_subj,1)); #stores regression summary with the highest r2 for each person
for(i in 1:n_subj) {
	breaks_reg_cslope_dots[[i]] <- as.list(mat.or.vec(num_tasks,1));
	for(j in 1:num_tasks) {
		#populate inc_dummies_dots for subject i
		inc_dummies_dots[i,,j] <- (inc_dots[i,] >= j)+0;
		#run regressions and store r squareds
		breaks_reg_cslope_dots[[i]][[j]] <- lm(correct_dots[i,] ~ inc_dots[i,] + inc_dummies_dots[i,,j]);
		breaks_reg_cslope_r2_dots[i,j] <- summary(breaks_reg_cslope_dots[[i]][[j]])$r.squared;
	}
	#find optimal r squared
	breaks_reg_cslope_optinc_dots[i] <- which.max(breaks_reg_cslope_r2_dots[i,]);
	#store this regression
	breaks_reg_cslope_opt_dots[[i]] <- breaks_reg_cslope_dots[[i]][[breaks_reg_cslope_optinc_dots[i]]];
	breaks_reg_cslope_opt_dots_sum[[i]] <- summary(breaks_reg_cslope_opt_dots[[i]])
}

######
#MLE (Shannon)
#starting parameter is set arbitrarily
######
######
#dots
mle_dots <- as.list(mat.or.vec(n_subj,1));
for(i in 1:n_subj) {
	ll_shannon_dots <- function(a) {
		-sum(
			correct_dots[i,]*(
				inc_dots[i,]/a
				-log(
					length(dot_counts)-1+exp(inc_dots[i,]/a)
				)
			)+(1-correct_dots[i,])*(
				log(length(dot_counts)-1)
				-log(
					length(dot_counts)-1+exp(inc_dots[i,]/a)
				)
			)
		);
	}
	mle_dots[[i]] <- mle2(ll_shannon_dots,method="L-BFGS-B",lower=c(a=0),start=c(a=5));
}

######
#angle
mle_ang <- as.list(mat.or.vec(n_subj,1));
for(i in 1:n_subj) {
	ll_shannon_ang <- function(a) {
		-sum(
			correct_ang[i,]*(
				inc_ang[i,]/(a*length(ang_counts))
				-log(
					length(ang_counts)-1+exp(inc_ang[i,]/(a*length(ang_counts)))
				)
			)+(1-correct_ang[i,])*(
				log(length(ang_counts)-1)
				-log(
					length(ang_counts)-1+exp(inc_ang[i,]/(a*length(ang_counts)))
				)
			)
		);
	}
	mle_ang[[i]] <- mle2(ll_shannon_ang,method="L-BFGS-B",lower=c(a=0),start=c(a=5));
}



######
#PROBIT
######
######
#dots
probit_dots <- as.list(mat.or.vec(n_subj,1));
probit_sum_dots <- as.list(mat.or.vec(n_subj,1));
for(i in 1:n_subj) {
	probit_dots[[i]] <- glm(correct_dots[i,] ~ inc_dots[i,], family=binomial(link="probit"));
	probit_sum_dots[[i]] <- summary(probit_dots[[i]]);
}

######
#angle
probit_ang <- as.list(mat.or.vec(n_subj,1));
probit_sum_ang <- as.list(mat.or.vec(n_subj,1));
for(i in 1:n_subj) {
	probit_ang[[i]] <- glm(correct_ang[i,] ~ inc_ang[i,], family=binomial(link="probit"));
	probit_sum_ang[[i]] <- summary(probit_ang[[i]]);
}



######
#NORMAL SIGNALS (assuming costs linear in precision w/ parameter a)
######
######
#dots
normsig_dots <- as.list(mat.or.vec(n_subj,1));

for (i in 1:n_subj) {
	ll_norm_dots <- function(a) {
		like <- 0; #negative log likelihood

		for(j in 1:num_tasks) {
			#solve for sqrt of precision
			solve_sqrt_prec <- function(sqrt_prec) {
				return(a*sqrt_prec - 2/5*dnorm(0.5*sqrt_prec)*inc_dots[i,j]);
			}
			sqrt_prec_soln <- nleqslv(2,solve_sqrt_prec)$x;

			#if true state is 38 or 42, then there's one likelihood (because on edges)
			#if true state is 39, 40, or 41, then there's another likelihood (because in interior)

			if(true_dots[i,j] %in% c(38,42)) {
				if(correct_dots[i,j]==1) {
					like <- like - log(pnorm(0.5,sd=1/sqrt_prec_soln));
				} else {
					like <- like - log(pnorm(0.5,sd=1/sqrt_prec_soln,lower.tail=FALSE));
				}
			} else {
				if(correct_dots[i,j]==1) {
					like <- like - log(pnorm(0.5,sd=1/sqrt_prec_soln)-pnorm(-0.5,sd=1/sqrt_prec_soln));
				} else {
					like <- like - log(2*pnorm(-0.5,sd=1/sqrt_prec_soln));
				}
			}
		}
		return(like);		
	}

	#mle step
	normsig_dots[[i]] <- mle2(ll_norm_dots,start=list(a=20),lower=(a=0),method="L-BFGS-B");
}



#######
#angle
normsig_ang <- as.list(mat.or.vec(n_subj,1));

for (i in 1:n_subj) {
	ll_norm_ang <- function(a) {
		like <- 0; #negative log likelihood

		for(j in 1:num_tasks) {
			#solve for sqrt of precision
			solve_sqrt_prec <- function(sqrt_prec) {
				return(a*sqrt_prec - 2/5*dnorm(0.5*sqrt_prec)*inc_ang[i,j]);
			}
			sqrt_prec_soln <- nleqslv(2,solve_sqrt_prec)$x;

			#if true state is 38 or 42, then there's one likelihood (because on edges)
			#if true state is 39, 40, or 41, then there's another likelihood (because in interior)

			if(true_ang[i,j] %in% c(38,42)) {
				if(correct_ang[i,j]==1) {
					like <- like - log(pnorm(0.5,sd=1/sqrt_prec_soln));
				} else {
					like <- like - log(pnorm(0.5,sd=1/sqrt_prec_soln,lower.tail=FALSE));
				}
			} else {
				if(correct_ang[i,j]==1) {
					like <- like - log(pnorm(0.5,sd=1/sqrt_prec_soln)-pnorm(-0.5,sd=1/sqrt_prec_soln));
				} else {
					like <- like - log(2*pnorm(-0.5,sd=1/sqrt_prec_soln));
				}
			}
		}
		return(like);		
	}

	#mle step
	normsig_ang[[i]] <- mle2(ll_norm_ang,start=list(a=20),lower=(a=0),method="L-BFGS-B");
}




######
#TSALLIS ESTIMATION
######
######
tsallis_dots <- as.list(mat.or.vec(n_subj,1));
tolerance_perf <- 0.00001; #sets distance performance can be from 0 or 1
tolerance_sigma <- 0.00001; #ensures there's no "blow-up" in the vicinity of sigma=1
#set sigma initialization values according to excel doc
sigma_init <- rep(0.75,n_subj);
sigma_init[c(1,6,11,18,20,21,26,27,32,44,48,54,59,60,61,71)] <- 1;
sigma_init[c(51,58,73)] <- 5;
sigma_init[57] <- 20;

for (i in 1:n_subj) {
	ll_tsal_dots <- function(alpha, sigma) {
		like <- 0; #negative log likelihood

		for(j in 1:num_tasks) {
			if(abs(sigma-1)<tolerance_sigma) {
				perf_soln <- exp(inc_dots[i,j]/alpha)/(4+exp(inc_dots[i,j]/alpha));
			} else {
			
				#solve for performance
				solve_perf <- function(perf) {
					return(
						alpha*sigma/(sigma-1)*(((1-perf)/4)^(sigma-1)-perf^(sigma-1))+inc_dots[i,j]
					);
				}
				
				perf_soln <- try(as.numeric(uniroot(solve_perf,c(0.2,1),extendInt="downX")$root));
				#Looking at the graph, the only way for rootfinding to fail is if the root is greater than 1 (since "roots" smaller than 0.2 will never be chosen). If performance larger than 1 is computed, then this resets it to just below 1 so that a likelihood can actually be computed
				if("try-error" %in% class(perf_soln)) {
					perf_soln <- 1-tolerance_perf; 
				}
			}
			
			like <- like - correct_dots[i,j]*log(perf_soln) - (1-correct_dots[i,j])*log(1-perf_soln);
		}
			
		return(like);		
	}

	#mle step
	try(tsallis_dots[[i]] <- mle2(ll_tsal_dots,start=list(alpha=100,sigma=sigma_init[i]),lower=list(alpha=0,sigma=0),upper=list(alpha=Inf,sigma=Inf),method="L-BFGS-B"));
	#debug, comment out
	#print(i);
}



######
######
######
#This section requires the running of previous sections as listed below.

#1. Constant performance
#2. Binary performance
#3. Linear model
#4. Linear with break
#5. Polynomial (deg 2)
#6. Polynomial (deg 3)
#7. Shannon MLE
#8. Tsallis entropy costs
#9. Normal Signals, cost linear in precision

num_models <- 9;

#"noprobit" variables are actually no normal

#dots
aic_matrix_dots <- mat.or.vec(n_subj,num_models);
best_model_dots <- mat.or.vec(n_subj,1);
best_model_nopoly_dots <- mat.or.vec(n_subj,1);
best_model_noprobit_dots <- mat.or.vec(n_subj,1);
rank_models_noprobit_dots <- mat.or.vec(n_subj,num_models-1);
rank_models_dots <- mat.or.vec(n_subj,num_models);

#add bic calculations
bic_matrix_dots <- mat.or.vec(n_subj,num_models);
best_model_bic_dots <- mat.or.vec(n_subj,1);
rank_models_bic_dots <- mat.or.vec(n_subj,num_models);

#add AICc calculations
aicc_matrix_dots <- mat.or.vec(n_subj,num_models);
best_model_aicc_dots <- mat.or.vec(n_subj,1);
rank_models_aicc_dots <- mat.or.vec(n_subj,num_models);

aicc_correction <- function(k,n) {
	return(2*k*(k+1)/(n-k-1));
}

aicc_penalty <- c(aicc_correction(1,num_tasks),aicc_correction(3,num_tasks),aicc_correction(2,num_tasks),aicc_correction(5,num_tasks),aicc_correction(3,num_tasks),aicc_correction(4,num_tasks),aicc_correction(1,num_tasks),aicc_correction(1,num_tasks),aicc_correction(2,num_tasks)); #calculates AICc penalty for the models we use


for(i in 1:n_subj) {
	aic_matrix_dots[i,1] <- AIC(mean_reg_dots[[i]]);
	aic_matrix_dots[i,2] <- AIC(breaks_reg_noslope_opt_dots[[i]])+2; #need extra penalty for estimating break location
	aic_matrix_dots[i,3] <- AIC(reg_dots[[i]]);
	aic_matrix_dots[i,4] <- AIC(breaks_reg_dots[[i]])+2; #need extra penalty for estimating break location
	aic_matrix_dots[i,5] <- AIC(lpm2_reg_dots[[i]]);
	aic_matrix_dots[i,6] <- AIC(lpm3_reg_dots[[i]]);
	aic_matrix_dots[i,7] <- AIC(mle_dots[[i]]);
	aic_matrix_dots[i,8] <- AIC(tsallis_dots[[i]]);
	aic_matrix_dots[i,9] <- AIC(normsig_dots[[i]]);
	
	best_model_dots[i] <- which.min(aic_matrix_dots[i,]);
	bnp <- which.min(aic_matrix_dots[i,c(1:4,7:9)]);
	best_model_nopoly_dots[i] <- bnp + 2*round(bnp/9); #turns 5s and 6s into 7s and 8s
	
	best_model_noprobit_dots[i] <- which.min(aic_matrix_dots[i,1:8]);
	rank_models_noprobit_dots[i,] <- rank(aic_matrix_dots[i,1:8]);
	rank_models_dots[i,] <- rank(aic_matrix_dots[i,]);
	
	bic_matrix_dots[i,1] <- BIC(mean_reg_dots[[i]]);
	bic_matrix_dots[i,2] <- BIC(breaks_reg_noslope_opt_dots[[i]])+log(num_tasks); #need extra penalty for estimating break location
	bic_matrix_dots[i,3] <- BIC(reg_dots[[i]]);
	bic_matrix_dots[i,4] <- BIC(breaks_reg_dots[[i]])+log(num_tasks); #need extra penalty for estimating break location
	bic_matrix_dots[i,5] <- BIC(lpm2_reg_dots[[i]]);
	bic_matrix_dots[i,6] <- BIC(lpm3_reg_dots[[i]]);
	bic_matrix_dots[i,7] <- 2*mle_dots[[i]]@min+log(num_tasks);
	bic_matrix_dots[i,9] <- 2*normsig_dots[[i]]@min+log(num_tasks);
	bic_matrix_dots[i,8] <- 2*tsallis_dots[[i]]@min + 2*log(num_tasks);

	best_model_bic_dots[i] <- which.min(bic_matrix_dots[i,]);
	rank_models_bic_dots[i,] <- rank(bic_matrix_dots[i,]);
	
	aicc_matrix_dots[i,] <- aic_matrix_dots[i,]+aicc_penalty;
	best_model_aicc_dots[i] <- which.min(aicc_matrix_dots[i,]);
	rank_models_aicc_dots[i,] <- rank(aicc_matrix_dots[i,]);
}


######
#angle
aic_matrix_ang <- mat.or.vec(n_subj,num_models);
best_model_ang <- mat.or.vec(n_subj,1);
best_model_nopoly_ang <- mat.or.vec(n_subj,1);
best_model_noprobit_ang <- mat.or.vec(n_subj,1);
for(i in 1:n_subj) {
	aic_matrix_ang[i,1] <- AIC(mean_reg_ang[[i]]);
	aic_matrix_ang[i,2] <- AIC(breaks_reg_noslope_opt_ang[[i]])+2; #need extra penalty for estimating break location
	aic_matrix_ang[i,3] <- AIC(reg_ang[[i]]);
	aic_matrix_ang[i,4] <- AIC(breaks_reg_ang[[i]])+2; #need extra penalty for estimating break location
	aic_matrix_ang[i,5] <- AIC(lpm2_reg_ang[[i]]);
	aic_matrix_ang[i,6] <- AIC(lpm3_reg_ang[[i]]);
	aic_matrix_ang[i,7] <- AIC(mle_ang[[i]]);
	aic_matrix_ang[i,8] <- AIC(normsig_ang[[i]]);
	
	best_model_ang[i] <- which.min(aic_matrix_ang[i,]);
	bnp <- which.min(aic_matrix_ang[i,c(1:4,7,8)]);
	best_model_nopoly_ang[i] <- bnp + 2*round(bnp/9); #turns 5s and 6s into 7s and 8s
	best_model_noprobit_ang[i] <- which.min(aic_matrix_ang[i,1:7]);
}


######
#Graph binary, Shannon, normal, Tsallis (because these are the only ones that minimized AIC)
#Highlight the best fit by AIC
#scatterplots have binned incentives

######
#dots

#set up incentive bins
#number of incentives to group in each bin
scatter_bin_width <- 10; #set to change bin width, preferably factor of 100
scatter_bin_num <- ceiling(100/scatter_bin_width);

#create vector of bin midpoints for graphing
bin_midpts_notrunc <- (1:scatter_bin_num)*scatter_bin_width - scatter_bin_width/2+0.5; #don't account for truncation at top
bin_midpts <- apply(as.matrix(bin_midpts_notrunc),c(1),min,mean(c(100,scatter_bin_width*(scatter_bin_num-1)+1))); #shift midpoint in top bin as necessary

#create empty matrix for success rate in each bin
bin_success <- mat.or.vec(n_subj,scatter_bin_num);

for(i in 1:n_subj) {
  for(k in 1:scatter_bin_num) {
    bin_success[i,k] <- mean(correct_dots[i,intersect(which(inc_dots[i,]>(k-1)*scatter_bin_width),which(inc_dots[i,]<=k*scatter_bin_width))]);
  }
  
	lty_bin <- 6;
	lty_shan <- 4;
	lty_tsal <- 5;
	lty_norm <- 2;

	if(best_model_dots[i] == 2) {
		lty_bin <- 1;
		best_model_text_dots <- "Binary (Fixed)";
	} else if(best_model_dots[i] == 7) {
		lty_shan <- 1;
		best_model_text_dots <- "Logistic (Shannon)";
	} else if(best_model_dots[i] == 8) {
		lty_tsal <- 1;
		best_model_text_dots <- "SIC (Tsallis)"
	} else if(best_model_dots[i] == 9) {
		lty_norm <- 1;
		best_model_text_dots <- "Concave (Normal)"
	}
	
	plot(bin_success[i,] ~ bin_midpts, main=paste("Dots: subject ",i," --- ",best_model_text_dots," as solid line",sep=""),xlab="Incentive Level",ylab="Correct",xlim=c(0,100),ylim=c(0,1));
	
	points(correct_dots[i,]~inc_dots[i,],pch=20,add=TRUE);
	
	#binary
	#lower line
	curve(0*x+breaks_reg_noslope_opt_dots[[i]]$coefficients[[1]],1,breaks_reg_noslope_optinc_dots[i],add=TRUE,lty=lty_bin);
	#upper line
	curve(0*x+breaks_reg_noslope_opt_dots[[i]]$coefficients[[1]]+breaks_reg_noslope_opt_dots[[i]]$coefficients[[2]],breaks_reg_noslope_optinc_dots[i],100,add=TRUE,lty=lty_bin);

	#shannon
	curve(exp(x/coef(mle_dots[[i]]))/
		(length(dot_counts)-1+exp(x/coef(mle_dots[[i]]))),1,100,add=TRUE,lty=lty_shan);
		
		
	#tsallis
	tsal_graph_coef <- coef(tsallis_dots[[i]]);
	
	graph_tsal <- function(inc) {
	  perf_tsal <- rep(NA,length(inc));
	  
	  for(j in 1:length(inc)) {
	    perf_foc <- function(y) {
	      inc[j] + tsal_graph_coef[1]*tsal_graph_coef[2]/(tsal_graph_coef[2]-1)*(((1-y)/4)^(tsal_graph_coef[2]-1)-y^(tsal_graph_coef[2]-1));
	    }
	    
	    perf_tsal_temp <- nleqslv(0.5,perf_foc)$x;
	    
	    if(perf_tsal_temp > 1) {
	      perf_tsal[j] <- 1;
	    } else {
	      perf_tsal[j] <- perf_tsal_temp;
	    }
	  }
	  return(perf_tsal);
	}
	
	curve(graph_tsal(x),1,100,add=TRUE,lty=lty_tsal);
	

	#pnorm
	cost_coef <- coef(normsig_dots[[i]]);

	sqrt_prec_graph <- function(inc) {
		zeta_soln <- rep(NA,length(inc));
		for(j in 1:length(inc)) {
			solve_sqrt_prec_graph <- function(sqrt_prec) {
				return(cost_coef*sqrt_prec - 2/5*dnorm(0.5*sqrt_prec)*inc[j]);
			}
			zeta_soln[j] <- nleqslv(2,solve_sqrt_prec_graph)$x;
		}
		return(zeta_soln);
	}

	#plot(correct_dots[i,]~inc_dots[i,]);

	curve(8/5*pnorm(0.5*sqrt_prec_graph(x))-3/5,1,100,add=TRUE,lty=lty_norm);
	
	
	##add legend
	legend(x = "bottomright", legend=c("Binary","Logistic","SIC","Concave"), lty=c(lty_bin,lty_shan,lty_tsal,lty_norm), inset=0.07);
	

	#save graphs - comment out next two lines to avoid saving
	dev.copy(pdf,paste("fit_graph_dots_",i,".pdf",sep=""),height=7,width=8.5);
	dev.off();
}

######
#angle
for(i in 1:n_subj) {
	lty_bin <- 2;
	lty_shan <- 2;
	lty_norm <- 2;

	if(best_model_ang[i] == 2) {
		lty_bin <- 1;
		best_model_text_ang <- "Binary";
	} else if(best_model_ang[i] == 7) {
		lty_shan <- 1;
		best_model_text_ang <- "Shannon";
	} else if(best_model_ang[i] == 8) {
		lty_prob <- 1;
		best_model_text_ang <- "Probit";
	}

	plot(correct_ang[i,] ~ inc_ang[i,], main=paste("Angles: subject ",i," --- ",best_model_text_ang," as solid line"),xlab="Incentive Level",ylab="Correct",pch=20);

	#binary
	#lower line
	curve(0*x+breaks_reg_noslope_opt_ang[[i]]$coefficients[[1]],1,breaks_reg_noslope_optinc_ang[i],add=TRUE,lty=lty_bin);
	#upper line
	curve(0*x+breaks_reg_noslope_opt_ang[[i]]$coefficients[[1]]+breaks_reg_noslope_opt_ang[[i]]$coefficients[[2]],breaks_reg_noslope_optinc_ang[i],100,add=TRUE,lty=lty_bin);

	#shannon
	curve(exp(x/(coef(mle_ang[[i]])*length(ang_counts)))/
		(length(ang_counts)-1+exp(x/(coef(mle_ang[[i]])*length(ang_counts)))),1,100,add=TRUE,lty=lty_shan);

	#pnorm
	cost_coef <- coef(normsig_ang[[i]]);

	sqrt_prec_graph <- function(inc) {
		zeta_soln <- rep(NA,length(inc));
		for(j in 1:length(inc)) {
			solve_sqrt_prec_graph <- function(sqrt_prec) {
				return(cost_coef*sqrt_prec - 2/5*dnorm(0.5*sqrt_prec)*inc[j]);
			}
			zeta_soln[j] <- nleqslv(2,solve_sqrt_prec_graph)$x;
		}
		return(zeta_soln);
	}

	#plot(correct_dots[i,]~inc_dots[i,]);

	curve(8/5*pnorm(0.5*sqrt_prec_graph(x))-3/5,1,100,add=TRUE,lty=lty_norm);

	#save graphs - comment out next two lines to avoid saving
	dev.copy(pdf,paste("fit_graph_ang_",i,".pdf",sep=""));
	dev.off();
}



##########################
##########################
#dots
#average aics, ranks
aic_means_dots <- apply(aic_matrix_dots[str_mon_subj,],2,mean);
avgranks_means_dots <- apply(rank_models_dots[str_mon_subj,],2,mean);

#ranks for subjects with binary best fit; logistic best fit
avgrank_means_bestbin_dots <- apply(rank_models_dots[intersect(str_mon_subj,which(best_model_dots==2)),],2,mean);
avgrank_means_bestlog_dots <- apply(rank_models_dots[intersect(str_mon_subj,which(best_model_dots==7)),],2,mean);


##########################
##########################

######
#DISTANCE ANALYSIS
######
######
#dots
mean_dist_dots <- mat.or.vec(n_subj,1);
reg_dist_dots <- as.list(mat.or.vec(n_subj,1));
for(i in 1:n_subj) {
	x11();
	plot(abs_dist_dots[i,] ~ inc_dots[i,], main=paste("Dots: subject ",i));
	mean_dist_dots[i] <- mean(dist_dots[i,]);
	reg_dist_dots[[i]] <- summary(lm(abs_dist_dots[i,] ~ inc_dots[i,]));
}

######
#angle
mean_dist_ang <- mat.or.vec(n_subj,1);
reg_dist_ang <- as.list(mat.or.vec(n_subj,1));
for(i in 1:n_subj) {
	x11();
	plot(abs_dist_ang[i,] ~ inc_ang[i,], main=paste("Angle: subject ",i));
	mean_dist_ang[i] <- mean(dist_ang[i,]);
	reg_dist_ang[[i]] <- summary(lm(abs_dist_ang[i,] ~ inc_ang[i,]));
}


######
######
#KOLMOGOROV-SMIRNOV TESTS (CONOVER CORRECTION)
######
#dots
dist_totals_dots <- matrix(nrow=n_subj,ncol=2*length(dot_counts)-1); #actual counts of (directed) distances
truth_totals_dots <- matrix(nrow=n_subj,ncol=length(dot_counts)); #totals in each state
error_shannon_dots <- matrix(nrow=n_subj,ncol=2*length(dot_counts)-1); #error distance frequencies for each subject under Shannon assumptions, empirical distribution of states
cum_error_shannon_dots <- matrix(nrow=n_subj,ncol=2*length(dot_counts)-1); #cumulative of previous line
for(i in 1:n_subj) {
	for(j in -(length(dot_counts)-1):(length(dot_counts)-1)) {
		dist_totals_dots[i,j+length(dot_counts)] <- sum(dist_dots[i,]==j);
	}	
 
	for(k in 1:length(dot_counts)) {
		truth_totals_dots[i,k] <- sum(true_dots[i,]==dot_counts[k]);
	}
	
	for(j in -(length(dot_counts)-1):(length(dot_counts)-1)) {
		if(j == 0) {
			error_shannon_dots[i,j+length(dot_counts)] <- mean(correct_dots[i,]);
		}
		else {
			error_shannon_dots[i,j+length(dot_counts)] <- 1/num_tasks*1/(length(dot_counts)-1)*(1-mean(correct_dots[i,]))*sum(truth_totals_dots[i,max(1,1-j):min(5,5-j)]);
		}

		cum_error_shannon_dots[i,j+length(dot_counts)] <- sum(error_shannon_dots[i,1:(j+length(dot_counts))]);
	}
}
#add 0 to start of cumulative error distribution
cum_error_shannon_dots <- cbind(0,cum_error_shannon_dots);

#create cdfs for comparison and test
shannon_cdf_dots <- as.list(mat.or.vec(n_subj,1));
ks_test_dots <- as.list(mat.or.vec(n_subj,1));
ks_stat_dots <- mat.or.vec(n_subj,1);
ks_pval_dots <- mat.or.vec(n_subj,1);
for(i in 1:n_subj) {
	shannon_cdf_dots[[i]] <- stepfun(-4:4, cum_error_shannon_dots[i,]);
	ks_test_dots[[i]] <- ks.test(dist_dots[i,],shannon_cdf_dots[[i]],simulate.p.value=FALSE);	
	ks_stat_dots[i] <- ks_test_dots[[i]]$statistic;
	ks_pval_dots[i] <- ks_test_dots[[i]]$p.value;
}

#check for significance
ks_pval_signif_dots <- (ks_pval_dots<=0.05)+0;

######
#angle
dist_totals_ang <- matrix(nrow=n_subj,ncol=2*length(ang_counts)-1); #actual counts of (directed) distances
truth_totals_ang <- matrix(nrow=n_subj,ncol=length(ang_counts)); #totals in each state
error_shannon_ang <- matrix(nrow=n_subj,ncol=2*length(ang_counts)-1); #error distance frequencies for each subject under Shannon assumptions, empirical distribution of states
cum_error_shannon_ang <- matrix(nrow=n_subj,ncol=2*length(ang_counts)-1); #cumulative of previous line
for(i in 1:n_subj) {
	for(j in -(length(ang_counts)-1):(length(ang_counts)-1)) {
		dist_totals_ang[i,j+length(ang_counts)] <- sum(dist_ang[i,]==5*j);
	}	
 
	for(k in 1:length(ang_counts)) {
		truth_totals_ang[i,k] <- sum(true_ang[i,]==ang_counts[k]);
	}
	
	for(j in -(length(ang_counts)-1):(length(ang_counts)-1)) {
		if(j == 0) {
			error_shannon_ang[i,j+length(ang_counts)] <- mean(correct_ang[i,]);
		}
		else {
			error_shannon_ang[i,j+length(ang_counts)] <- 1/num_tasks*1/(length(ang_counts)-1)*(1-mean(correct_ang[i,]))*sum(truth_totals_ang[i,max(1,1-j):min(5,5-j)]);
		}

		cum_error_shannon_ang[i,j+length(ang_counts)] <- sum(error_shannon_ang[i,1:(j+length(ang_counts))]);
	}
}
#add 0 to start of cumulative error distribution
cum_error_shannon_ang <- cbind(0,cum_error_shannon_ang);

#create cdfs for comparison and test
shannon_cdf_ang <- as.list(mat.or.vec(n_subj,1));
ks_test_ang <- as.list(mat.or.vec(n_subj,1));
ks_stat_ang <- mat.or.vec(n_subj,1);
ks_pval_ang <- mat.or.vec(n_subj,1);
for(i in 1:n_subj) {
	shannon_cdf_ang[[i]] <- stepfun(5*(-4:4), cum_error_shannon_ang[i,]);
	ks_test_ang[[i]] <- ks.test(dist_ang[i,],shannon_cdf_ang[[i]],simulate.p.value=FALSE);	
	ks_stat_ang[i] <- ks_test_ang[[i]]$statistic;
	ks_pval_ang[i] <- ks_test_ang[[i]]$p.value;
}

#check for significance
ks_pval_signif_ang <- (ks_pval_ang<=0.05)+0;



##########################
##########################

##REACTION TIME ANALYSIS

##aggregate regressions
#time on incentive
time_inc_dots_reg <- lm(react_all_dots ~ agg_inc_dots,data=dots_data_agg);
time_inc_cluster_dots <- coeftest(time_inc_dots_reg,cluster.vcov(time_inc_dots_reg,subj_id_agg));

#correctness on time
cor_time_dots_reg <- lm(agg_correct_dots ~ react_all_dots,data=dots_data_agg);
cor_time_cluster_dots <- coeftest(cor_time_dots_reg,cluster.vcov(cor_time_dots_reg,subj_id_agg));


##dip test for unimodality
dip_pval_dots <- rep(NA,n_subj);
for(i in 1:n_subj) {
	dip_pval_dots[i] <- dip.test(react_reorder_dots[i,])$p.value
}
bimodal_dots <- (dip_pval_dots<=0.05)+0;
bimodal_dots_subj <- which(bimodal_dots==1);
