##CHEREMUKHIN ET AL ESTIMATION

##get practical math functions for inverse reciprocal trig functions
install.packages("pracma");
library(pracma);

##N.B. R's implementation of acot uses the WRONG convention for negative arguments; have to correct by adding pi to result if argument is negative

#######
##set initialization values according to cher_init.xlsx doc

theta_fixed_init <- rep(3,n_subj);
kappa_fixed_init <- rep(1,n_subj);

theta_fixed_init[c(12,47,49)] <- 9;
theta_fixed_init[74] <- 19;
theta_fixed_init[57] <- 1.5;
theta_fixed_init[c(17,27,31,37,39,48,55,66,81)] <- 15;
theta_fixed_init[c(13,14,15,45)] <- 30;

kappa_fixed_init[57] <- 0.01;



cher_dots <- as.list(mat.or.vec(n_subj,1));
#change starting point back to 1 after running
for (i in str_mon_subj) {
	#use reciprocal of rho to avoid numerical issues
	ll_cher_dots <- function(theta, kappa) {
		like <- 0; #negative log likelihood

		for(j in 1:num_tasks) {
			#solve for performance
			solve_perf <- function(perf) {
				
				#get mutual information, minus capacity constraint
				info_minus_cap <- log(5)+perf*log(perf)+(1-perf)*log((1-perf)/4)-kappa;
				
				return(
theta*pi*(acot(info_minus_cap*600)+pi*(info_minus_cap<0))^(-1)*(log(perf)-log(1-perf)+log(4))-inc_dots[i,j]
				);
			}
			
			perf_soln <- try(as.numeric(uniroot(solve_perf,c(0.2,0.99999))$root));
						
			like <- like - correct_dots[i,j]*log(perf_soln) - (1-correct_dots[i,j])*log(1-perf_soln);

		}
		return(like);		
	}

	#mle step
	try(cher_dots[[i]] <- mle2(ll_cher_dots,start=list(theta=theta_fixed_init[i],kappa=kappa_fixed_init[i]),lower=list(theta=0,kappa=0),upper=list(theta=Inf,kappa=Inf),method="L-BFGS-B"));

#debug, comment out
print(i);
}

AIC_cher <- mat.or.vec(length(str_mon_subj),1);
for(i in 1:length(str_mon_subj)) {
	if(typeof(cher_dots[[str_mon_subj[i]]])=="S4") {
		AIC_cher[i] <- AIC(cher_dots[[str_mon_subj[i]]]); 
	}
}



#####
###Let rho vary flexibly and see what gets estimated

#######
##set initialization values according to cher_init.xlsx doc

theta_flex_init <- rep(10,n_subj);
kappa_flex_init <- rep(1,n_subj);
rho_flex_init <- rep(5,n_subj)

theta_flex_init[c(10,37,47,48,73,75)] <- 30;
theta_flex_init[c(2,17,22,49,57,66)] <- 1.5;

kappa_flex_init[c(22,49,57,66)] <- 0.01;
kappa_flex_init[c(2,17)] <- 0.0001;

rho_flex_init[c(2,17,73)] <- 10;


cher_rho_dots <- as.list(mat.or.vec(n_subj,1));
#change starting point back to 1 after running
for (i in str_mon_subj) {
	#use reciprocal of rho to avoid numerical issues
	ll_cher_rho_dots <- function(theta, kappa, rho) {
		like <- 0; #negative log likelihood

		for(j in 1:num_tasks) {
			#solve for performance
			solve_perf <- function(perf) {
				#get mutual information, minus capacity constraint
				info_minus_cap <- log(5)+perf*log(perf)+(1-perf)*log((1-perf)/4)-kappa;
				
				return(
theta*pi*(acot(info_minus_cap*rho)+pi*(info_minus_cap<0))^(-1)*(log(perf)-log(1-perf)+log(4))-inc_dots[i,j]
				);
			}
			
			perf_soln <- try(as.numeric(uniroot(solve_perf,c(0.2,0.99999))$root));
			
			like <- like - correct_dots[i,j]*log(perf_soln) - (1-correct_dots[i,j])*log(1-perf_soln);

		}
		return(like);		
	}

	#mle step
	try(cher_rho_dots[[i]] <- mle2(ll_cher_rho_dots,start=list(theta=theta_flex_init[i],kappa=kappa_flex_init[i],rho=rho_flex_init[i]),lower=list(theta=0,kappa=0,rho=0),upper=list(theta=Inf,kappa=Inf,rho=Inf),method="L-BFGS-B"));

#debug, comment out
print(i);
}

AIC_cher_rho <- mat.or.vec(length(str_mon_subj),1);
for(i in 1:length(str_mon_subj)) {
	if(typeof(cher_rho_dots[[str_mon_subj[i]]])=="S4") {
		AIC_cher_rho[i] <- AIC(cher_rho_dots[[str_mon_subj[i]]]); 
	}
}


#get best fits by AIC
best_model_cher_dots <- apply(cbind(aic_matrix_dots[str_mon_subj,],AIC_cher,AIC_cher_rho),1,which.min);

table(best_model_cher_dots);

#get rho values for histogram
rho_table <- rep(0,length(str_mon_subj));
index <- 1;
for(i in str_mon_subj) {
	rho_table[index] <- coef(cher_rho_dots[[i]])[3];
	index <- index+1;
}
log_rho_table <- log10(1+rho_table);
#graph histogram so that leftmost column is all the true zeroes
hist(log_rho_table,breaks=c(-1,0,1,2,3,4,5),ylim=c(0,20),main="",xlab=expression(paste("Base 10 logarithm of estimated 1+",rho,sep="")));
