###figure out parameters for the tests
###this section requires analysis_qualtrics to be run first

###shannon coefficients
shannon_coeffs <- sapply(mle_dots,coef);

###normal signal coefficients
normsig_coeffs <- sapply(normsig_dots,coef);

###binary performance coefficients
binary_intjump <- sapply(breaks_reg_noslope_opt_dots,coef);
binary_coeffs <- cbind(t(binary_intjump),breaks_reg_noslope_optinc_dots);

#tsallis coefficients
tsallis_coeffs <-sapply(tsallis_dots,coef);


###mean values
shannon_mean <- mean(shannon_coeffs[str_mon_subj]);
normsig_mean <- mean(normsig_coeffs[str_mon_subj]);
binary_mean <- apply(binary_coeffs[str_mon_subj,],2,mean);
tsallis_mean <- apply(tsallis_coeffs[,str_mon_subj],1,mean);

##standard devs <-
shannon_sd <- sd(shannon_coeffs[str_mon_subj]);
normsig_sd <- sd(normsig_coeffs[str_mon_subj]);
binary_sd <- apply(binary_coeffs[str_mon_subj,],2,sd);
tsallis_sd <- apply(tsallis_coeffs[,str_mon_subj],1,sd);

#################################

#################################

#################################

#################################

#################################

#################################

#################################

###SIMULATIONS START HERE
###no need to run analysis_qualtrics before running this
###use parameters from the first part, rounded, plus/minus 1/2 std. devs.

#necessary packages
install.packages("bbmle"); 
install.packages("nleqslv");

library(bbmle); #better mle
library(nleqslv); #solve nonlinear equations

##set seed, from as.numeric(Sys.time())
set.seed(1531268789); 
#seed is reset before each code block so that code blocks can run separately and results stay consistent

#use 100 incentives and number of subjects
n_subj <- 100;
inc_matrix <- t(matrix(1:100,nrow=100,ncol=n_subj));
inc_matrix_coarse <- t(matrix(10*rep(1:10,10),nrow=100,ncol=n_subj));

#generate true numbers
random_dots <- matrix(runif(100*n_subj),nrow=n_subj,ncol=100);
true <- 37+ceiling(5*random_dots);

#generate set of potential mistakes for each subject-incentive pair
reverse_setdiff <- function(a,b) {
	setdiff(b,a);
}
potential_mistakes <- apply(true,c(1,2),reverse_setdiff,38:42);

#make vectors that allow for indexing mistakes properly
task_col <- rep(1:100,n_subj);
subjects_col <- as.vector(t(matrix(rep(1:n_subj,100),nrow=n_subj,ncol=100)));

#make dummies for binary regression
inc_dummies <- matrix(nrow=100,ncol=100); #fine
for (k in 1:100) {
	inc_dummies[k,] <- c(rep(0,k-1),rep(1,100-k+1));
}
inc_dummies_coarse <- matrix(nrow=100,ncol=100); #coarse
for (k in 1:10) {
  inc_dummies_coarse[k,] <- rep(c(rep(0,k-1),rep(1,10-k+1)),10);
}

#constants for tsallis estimation
tolerance_perf <- 0.0001; #sets distance performance can be from 0 or 1
tolerance_sigma <- 0.0001; #ensures there's no "blow-up" in the vicinity of sigma=1
sigma_starts <- c(0.75,1,5,20,0.1); #same starting points used in the estimation on the full sample, plus 0.1 just in case


##########################
#run each "AS NULL" section separately
##########################


###SHANNON AS NULL
##set seed, from as.numeric(Sys.time())
set.seed(1566109963);

shan_vals <- c(10,15,30,45,60); #parameter values for model

#aic matrices
#fine
aic_mat_shan_shan <- matrix(nrow=n_subj,ncol=length(shan_vals));
aic_mat_shan_norm <- matrix(nrow=n_subj,ncol=length(shan_vals));
aic_mat_shan_bin <- matrix(nrow=n_subj,ncol=length(shan_vals));
aic_mat_shan_tsal <- matrix(nrow=n_subj,ncol=length(shan_vals));
best_model_shan <- matrix(nrow=n_subj,ncol=length(shan_vals));
#coarse
aic_mat_shan_shan_coarse <- matrix(nrow=n_subj,ncol=length(shan_vals));
aic_mat_shan_norm_coarse <- matrix(nrow=n_subj,ncol=length(shan_vals));
aic_mat_shan_bin_coarse <- matrix(nrow=n_subj,ncol=length(shan_vals));
aic_mat_shan_tsal_coarse <- matrix(nrow=n_subj,ncol=length(shan_vals));
best_model_shan_coarse <- matrix(nrow=n_subj,ncol=length(shan_vals));


shannon_base_time <- as.numeric(Sys.time()); #for debugging to calculate time elapsed

for(j in 1:length(shan_vals)) {
  
  print(paste("SHANNON TRUE MODEL -- iteration: j",j));
  print(paste("SHANNON TRUE MODEL -- time elapsed:",as.numeric(Sys.time())-shannon_base_time));
  
  ###########################
  ##########FINE#############
  ###########################
  
	##generate correctness data
	draw_thresholds <- exp(inc_matrix/shan_vals[j])/(4+exp(inc_matrix/shan_vals[j]));
	random_draws <- matrix(runif(100*n_subj),nrow=n_subj,ncol=100);
	correct <- (random_draws<=draw_thresholds)+0; #answered correctly?
	
	##generate mistake data
	random_mistake <- runif(100*n_subj);
	mistake_vector <- potential_mistakes[cbind(ceiling(4*random_mistake),subjects_col,task_col)];
	mistake_matrix <- t(matrix(mistake_vector,nrow=100,ncol=n_subj));
	
	##generate response data
	answer <- correct*true + (1-correct)*mistake_matrix;
		
	##conduct estimations##
	##generate required variables
	shan_est <- as.list(mat.or.vec(n_subj,1));
	norm_est <- as.list(mat.or.vec(n_subj,1));
	
	breaks_reg_noslope <- as.list(mat.or.vec(n_subj,1)); #stores regression results for each dummy for each person
	breaks_reg_noslope_r2 <- matrix(nrow=n_subj,ncol=100); #store r squareds for each regression
	breaks_reg_noslope_optinc <- vector(length=n_subj); #stores optimal incentive level for break
	breaks_reg_noslope_opt <- as.list(mat.or.vec(n_subj,1)); #stores regression results with the highest r2 for each person
	breaks_reg_noslope_opt_sum <- as.list(mat.or.vec(n_subj,1)); #stores regression summary with the highest r2 for each person
	
	tsal_est <- as.list(mat.or.vec(n_subj,1));
	
	for (i in 1:n_subj) {
		###
		##shannon
		ll_shannon_dots <- function(a) {
			-sum(
				correct[i,]*(
					inc_matrix[i,]/a
					-log(
						4+exp(inc_matrix[i,]/a)
					)
				)+(1-correct[i,])*(
					log(4)
					-log(
						4+exp(inc_matrix[i,]/a)
					)
				)
			);
		}
		shan_est[[i]] <- mle2(ll_shannon_dots,method="L-BFGS-B",lower=c(a=0),start=c(a=5));
		
		###
		##normal signals
		ll_norm_dots <- function(a) {
			like <- 0; #negative log likelihood

			for(k in 1:100) {
			  
				#solve for sqrt of precision
				solve_sqrt_prec <- function(sqrt_prec) {
					return(a*sqrt_prec - 2/5*dnorm(0.5*sqrt_prec)*inc_matrix[i,k]);
				}
				sqrt_prec_soln <- nleqslv(2,solve_sqrt_prec)$x;

				#if true state is 38 or 42, then there's one likelihood (because on edges)
				#if true state is 39, 40, or 41, then there's another likelihood (because in interior)

				if(true[i,k] %in% c(38,42)) {
					if(correct[i,k]==1) {
						like <- like - log(pnorm(0.5,sd=1/sqrt_prec_soln));
					} else {
						like <- like - log(pnorm(0.5,sd=1/sqrt_prec_soln,lower.tail=FALSE));
					}
				} else {
					if(correct[i,k]==1) {
						like <- like - log(pnorm(0.5,sd=1/sqrt_prec_soln)-pnorm(-0.5,sd=1/sqrt_prec_soln));
					} else {
						like <- like - log(2*pnorm(-0.5,sd=1/sqrt_prec_soln));
					}
				}
			}
			return(like);		
		}

		#mle step
		norm_est[[i]] <- mle2(ll_norm_dots,start=list(a=20),lower=(a=0),method="L-BFGS-B");
		
		
		###
		##binary
		breaks_reg_noslope[[i]] <- as.list(mat.or.vec(100,1));
		for(k in 1:100) {
			#run regressions and store r squareds
			breaks_reg_noslope[[i]][[k]] <- lm(correct[i,] ~ inc_dummies[k,]);
			breaks_reg_noslope_r2[i,k] <- summary(breaks_reg_noslope[[i]][[k]])$r.squared;
		}
		
		#find optimal r squared
		breaks_reg_noslope_optinc[i] <- which.max(breaks_reg_noslope_r2[i,]);
		#store this regression
		breaks_reg_noslope_opt[[i]] <- breaks_reg_noslope[[i]][[breaks_reg_noslope_optinc[i]]];
		breaks_reg_noslope_opt_sum[[i]] <- summary(breaks_reg_noslope_opt[[i]]);
		
		
		
	  ###
		##tsallis
		ll_tsal_dots <- function(alpha, sigma) {
		  like <- 0; #negative log likelihood
		  
		  for(k in 1:100) {
		    if(abs(sigma-1)<tolerance_sigma) {
		      perf_soln <- exp(inc_matrix[i,k]/alpha)/(4+exp(inc_matrix[i,k]/alpha));
		    } else {
		      
		      #solve for performance
		      solve_perf <- function(perf) {
		        return(
		          alpha*sigma/(sigma-1)*(((1-perf)/4)^(sigma-1)-perf^(sigma-1))+inc_matrix[i,k]
		        );
		      }
		      
		      perf_soln <- try(as.numeric(uniroot(solve_perf,c(0.2,1),extendInt="downX")$root),silent=TRUE);
		      #Looking at the graph, the only way for rootfinding to fail is if the root is greater than 1 (since "roots" smaller than 0.2 will never be chosen). If performance larger than 1 is computed, then this resets it to just below 1 so that a likelihood can actually be computed
		      if("try-error" %in% class(perf_soln)) {
		        perf_soln <- 1-tolerance_perf; 
		      }
		    }
		    
		    like <- like - correct[i,k]*log(perf_soln) - (1-correct[i,k])*log(1-perf_soln);
		  }
		  
		  return(like);		
		}
		
		#mle step
		for(errnum in 1:length(sigma_starts)) {
		  tsal_est[[i]] <- tryCatch(
		    mle2(ll_tsal_dots,start=list(alpha=100,sigma=sigma_starts[errnum]),lower=list(alpha=0,sigma=0),upper=list(alpha=Inf,sigma=Inf),method="L-BFGS-B"),
		    warning=function(w) {
		      return(NA);
		    },
		    error=function(e) {
		      return(NA);
		    }
		  );
		  
		  #break out of loop if success
		  if(!is.na(tsal_est[[i]])) {
		    break;
		  }
		}
		
		#debug, comment out
		print(paste("Fine Tsallis iteration:",i));
		
		######
		##AIC storage
		#1: shannon
		#2: normal
		#3: binary
		#4: tsallis
		aic_mat_shan_shan[i,j] <- AIC(shan_est[[i]]);
		aic_mat_shan_norm[i,j] <- AIC(norm_est[[i]]);
		aic_mat_shan_bin[i,j] <- AIC(breaks_reg_noslope_opt[[i]])+2; #extra penalty for location estimation
		aic_mat_shan_tsal[i,j] <- tryCatch(AIC(tsal_est[[i]]), error = function(e) Inf);
		
		best_model_shan[i,j] <- which.min(c(aic_mat_shan_shan[i,j],aic_mat_shan_norm[i,j],aic_mat_shan_bin[i,j],aic_mat_shan_tsal[i,j]));
	}	
	
		
		
	###########################
	#########COARSE############
	###########################
		
	##generate correctness data
	draw_thresholds_coarse <- exp(inc_matrix_coarse/shan_vals[j])/(4+exp(inc_matrix_coarse/shan_vals[j]));
	random_draws_coarse <- matrix(runif(100*n_subj),nrow=n_subj,ncol=100);
	correct_coarse <- (random_draws_coarse<=draw_thresholds_coarse)+0; #answered correctly?
		
	##generate mistake data
	random_mistake_coarse <- runif(100*n_subj);
	mistake_vector_coarse <- potential_mistakes[cbind(ceiling(4*random_mistake_coarse),subjects_col,task_col)];
	mistake_matrix_coarse <- t(matrix(mistake_vector_coarse,nrow=100,ncol=n_subj));
		
	##generate response data
	answer_coarse <- correct_coarse*true + (1-correct_coarse)*mistake_matrix_coarse;
		
	##conduct estimations##
	##generate required variables
	shan_est_coarse <- as.list(mat.or.vec(n_subj,1));
	norm_est_coarse <- as.list(mat.or.vec(n_subj,1));
		
	breaks_reg_noslope_coarse <- as.list(mat.or.vec(n_subj,1)); #stores regression results for each dummy for each person
	breaks_reg_noslope_r2_coarse <- matrix(nrow=n_subj,ncol=100); #store r squareds for each regression
	breaks_reg_noslope_optinc_coarse <- vector(length=n_subj); #stores optimal incentive level for break
	breaks_reg_noslope_opt_coarse <- as.list(mat.or.vec(n_subj,1)); #stores regression results with the highest r2 for each person
	breaks_reg_noslope_opt_sum_coarse <- as.list(mat.or.vec(n_subj,1)); #stores regression summary with the highest r2 for each person
	
	tsal_est_coarse <- as.list(mat.or.vec(n_subj,1));
		
	for (i in 1:n_subj) {
		###
	  ##shannon
		ll_shannon_dots <- function(a) {
		  -sum(
		     correct[i,]*(
		      inc_matrix[i,]/a
		      -log(
		         4+exp(inc_matrix_coarse[i,]/a)
		      )
		    )+(1-correct[i,])*(
		      log(4)
		      -log(
		         4+exp(inc_matrix_coarse[i,]/a)
		      )
		    )
		  );
		}
		shan_est_coarse[[i]] <- mle2(ll_shannon_dots,method="L-BFGS-B",lower=c(a=0),start=c(a=5));
		  
		###
		##normal signals
		ll_norm_dots <- function(a) {
		  like <- 0; #negative log likelihood
		    
		  for(k in 1:100) {
		      
		    #solve for sqrt of precision
		    solve_sqrt_prec <- function(sqrt_prec) {
		      return(a*sqrt_prec - 2/5*dnorm(0.5*sqrt_prec)*inc_matrix_coarse[i,k]);
		    }
		    sqrt_prec_soln <- nleqslv(2,solve_sqrt_prec)$x;
		      
		    #if true state is 38 or 42, then there's one likelihood (because on edges)
		    #if true state is 39, 40, or 41, then there's another likelihood (because in interior)
		      
		    if(true[i,k] %in% c(38,42)) {
		      if(correct[i,k]==1) {
		        like <- like - log(pnorm(0.5,sd=1/sqrt_prec_soln));
		      } else {
		        like <- like - log(pnorm(0.5,sd=1/sqrt_prec_soln,lower.tail=FALSE));
		      }
		    } else {
		      if(correct[i,k]==1) {
		        like <- like - log(pnorm(0.5,sd=1/sqrt_prec_soln)-pnorm(-0.5,sd=1/sqrt_prec_soln));
		      } else {
		        like <- like - log(2*pnorm(-0.5,sd=1/sqrt_prec_soln));
		      }
		    }
		  }
		  return(like);		
		}
		  
		#mle step
		norm_est_coarse[[i]] <- mle2(ll_norm_dots,start=list(a=20),lower=(a=0),method="L-BFGS-B");
		  
		  
		###
		##binary
		breaks_reg_noslope_coarse[[i]] <- as.list(mat.or.vec(10,1));
		for(k in 1:10) {
		  #run regressions and store r squareds
		  breaks_reg_noslope_coarse[[i]][[k]] <- lm(correct_coarse[i,] ~ inc_dummies_coarse[k,]);
		  breaks_reg_noslope_r2_coarse[i,k] <- summary(breaks_reg_noslope_coarse[[i]][[k]])$r.squared;
		}
		  
		#find optimal r squared
		breaks_reg_noslope_optinc_coarse[i] <- which.max(breaks_reg_noslope_r2_coarse[i,]);
		#store this regression
		breaks_reg_noslope_opt_coarse[[i]] <- breaks_reg_noslope_coarse[[i]][[breaks_reg_noslope_optinc_coarse[i]]];
		breaks_reg_noslope_opt_sum_coarse[[i]] <- summary(breaks_reg_noslope_opt_coarse[[i]]);
		
		###
		##tsallis
		ll_tsal_dots <- function(alpha, sigma) {
		  like <- 0; #negative log likelihood
		  
		  for(k in 1:100) {
		    if(abs(sigma-1)<tolerance_sigma) {
		      perf_soln <- exp(inc_matrix_coarse[i,k]/alpha)/(4+exp(inc_matrix_coarse[i,k]/alpha));
		    } else {
		      
		      #solve for performance
		      solve_perf <- function(perf) {
		        return(
		          alpha*sigma/(sigma-1)*(((1-perf)/4)^(sigma-1)-perf^(sigma-1))+inc_matrix_coarse[i,k]
		        );
		      }
		      
		      perf_soln <- try(as.numeric(uniroot(solve_perf,c(0.2,1),extendInt="downX")$root),silent=TRUE);
		      #Looking at the graph, the only way for rootfinding to fail is if the root is greater than 1 (since "roots" smaller than 0.2 will never be chosen). If performance larger than 1 is computed, then this resets it to just below 1 so that a likelihood can actually be computed
		      if("try-error" %in% class(perf_soln)) {
		        perf_soln <- 1-tolerance_perf; 
		      }
		    }
		    
		    like <- like - correct_coarse[i,k]*log(perf_soln) - (1-correct_coarse[i,k])*log(1-perf_soln);
		  }
		  
		  return(like);		
		}
		
		#mle step
		for(errnum in 1:length(sigma_starts)) {
		  tsal_est_coarse[[i]] <- tryCatch(
		    mle2(ll_tsal_dots,start=list(alpha=100,sigma=sigma_starts[errnum]),lower=list(alpha=0,sigma=0),upper=list(alpha=Inf,sigma=Inf),method="L-BFGS-B"),
		    warning=function(w) {
		      return(NA);
		    },
		    error=function(e) {
		      return(NA);
		    }
		  );
		  
		  #break out of loop if success
		  if(!is.na(tsal_est_coarse[[i]])) {
		    break;
		  }
		}
		
		#debug, comment out
		print(paste("Coarse Tsallis iteration:",i));
		
		
		######
		##AIC storage
		#1: shannon
		#2: normal
		#3: binary
		aic_mat_shan_shan_coarse[i,j] <- AIC(shan_est_coarse[[i]]);
		aic_mat_shan_norm_coarse[i,j] <- AIC(norm_est_coarse[[i]]);
		aic_mat_shan_bin_coarse[i,j] <- AIC(breaks_reg_noslope_opt_coarse[[i]])+2; #extra penalty for location estimation
		aic_mat_shan_tsal_coarse[i,j] <- tryCatch(AIC(tsal_est_coarse[[i]]), error = function(e) Inf);  
		best_model_shan_coarse[i,j] <- which.min(c(aic_mat_shan_shan_coarse[i,j],aic_mat_shan_norm_coarse[i,j],aic_mat_shan_bin_coarse[i,j],aic_mat_shan_tsal_coarse[i,j]));
		  
	}
}
	
#print tables (if out of 100, can be read as percentages)
best_model_shan_table <- apply(best_model_shan,2,table); #fine table
best_model_shan_table_coarse <- apply(best_model_shan_coarse,2,table); #coarse table




##################################
##################################
##################################


###NORMAL AS NULL
##set seed, from as.numeric(Sys.time())
set.seed(1566109977);

norm_vals <- c(1,2,4.5,7,10); #parameter values for model

#aic matrices
#fine
aic_mat_norm_shan <- matrix(nrow=n_subj,ncol=length(norm_vals));
aic_mat_norm_norm <- matrix(nrow=n_subj,ncol=length(norm_vals));
aic_mat_norm_bin <- matrix(nrow=n_subj,ncol=length(norm_vals));
aic_mat_norm_tsal <- matrix(nrow=n_subj,ncol=length(norm_vals));
best_model_norm <- matrix(nrow=n_subj,ncol=length(norm_vals));
#coarse
aic_mat_norm_shan_coarse <- matrix(nrow=n_subj,ncol=length(shan_vals));
aic_mat_norm_norm_coarse <- matrix(nrow=n_subj,ncol=length(shan_vals));
aic_mat_norm_bin_coarse <- matrix(nrow=n_subj,ncol=length(shan_vals));
aic_mat_norm_tsal_coarse <- matrix(nrow=n_subj,ncol=length(norm_vals));
best_model_norm_coarse <- matrix(nrow=n_subj,ncol=length(shan_vals));


normal_base_time <- as.numeric(Sys.time()); #for debugging to calculate time elapsed

for(j in 1:length(norm_vals)) {
  
  print(paste("NORMAL TRUE MODEL -- iteration: j",j));
  print(paste("NORMAL TRUE MODEL -- time elapsed:",as.numeric(Sys.time())-normal_base_time));
  
  #solve for sqrt of precision
	solve_sqrt_prec_inc <- function(incentive) {
		solve_sqrt_prec_simulate <- function(sqrt_prec) {
		return(norm_vals[j]*sqrt_prec - 2/5*dnorm(0.5*sqrt_prec)*incentive);
		}
		sqrt_prec_soln <- nleqslv(2,solve_sqrt_prec_simulate)$x;
	}

	#function that defines probability of answers
	answer_func <- function(drawval,trueval,precval) {
		truenum <- trueval-37
		if(drawval <= pnorm(precval*0.5*(3-2*truenum))) {
			return(38);
		} else if((drawval > pnorm(precval*0.5*(3-2*truenum))) & (drawval <= pnorm(precval*0.5*(5-2*truenum)))) {
			return(39);
		} else if((drawval > pnorm(precval*0.5*(5-2*truenum))) & (drawval <= pnorm(precval*0.5*(7-2*truenum)))) {
			return(40);
		} else if((drawval > pnorm(precval*0.5*(7-2*truenum))) & (drawval <= pnorm(precval*0.5*(9-2*truenum)))) {
			return(41);
		} else {
			return(42);
		}
	}
  
  ###########################
  ##########FINE#############
  ###########################
  
	random_draws <- matrix(runif(100*n_subj),nrow=n_subj,ncol=100);
	
	optimal_simulated_precision <- apply(inc_matrix,c(1,2),solve_sqrt_prec_inc);
	
	answer <- matrix(mapply(answer_func,random_draws,true,optimal_simulated_precision),nrow=n_subj,ncol=100);
	
	correct <- (answer==true)+0;
	
	##conduct estimations##
	##generate required variables
	shan_est <- as.list(mat.or.vec(n_subj,1));
	norm_est <- as.list(mat.or.vec(n_subj,1));
	
	breaks_reg_noslope <- as.list(mat.or.vec(n_subj,1)); #stores regression results for each dummy for each person
	breaks_reg_noslope_r2 <- matrix(nrow=n_subj,ncol=100); #store r squareds for each regression
	breaks_reg_noslope_optinc <- vector(length=n_subj); #stores optimal incentive level for break
	breaks_reg_noslope_opt <- as.list(mat.or.vec(n_subj,1)); #stores regression results with the highest r2 for each person
	breaks_reg_noslope_opt_sum <- as.list(mat.or.vec(n_subj,1)); #stores regression summary with the highest r2 for each person
	
	tsal_est <- as.list(mat.or.vec(n_subj,1));
	
	for (i in 1:n_subj) {
		###
		##shannon
		ll_shannon_dots <- function(a) {
			-sum(
				correct[i,]*(
					inc_matrix[i,]/a
					-log(
						4+exp(inc_matrix[i,]/a)
					)
				)+(1-correct[i,])*(
					log(4)
					-log(
						4+exp(inc_matrix[i,]/a)
					)
				)
			);
		}
		shan_est[[i]] <- mle2(ll_shannon_dots,method="L-BFGS-B",lower=c(a=0),start=c(a=5));
		
		###
		##normal signals
		ll_norm_dots <- function(a) {
			like <- 0; #negative log likelihood

			for(k in 1:100) {
			  
				#solve for sqrt of precision
				solve_sqrt_prec <- function(sqrt_prec) {
					return(a*sqrt_prec - 2/5*dnorm(0.5*sqrt_prec)*inc_matrix[i,k]);
				}
				sqrt_prec_soln <- nleqslv(2,solve_sqrt_prec)$x;

				#if true state is 38 or 42, then there's one likelihood (because on edges)
				#if true state is 39, 40, or 41, then there's another likelihood (because in interior)

				if(true[i,k] %in% c(38,42)) {
					if(correct[i,k]==1) {
						like <- like - log(pnorm(0.5,sd=1/sqrt_prec_soln));
					} else {
						like <- like - log(pnorm(0.5,sd=1/sqrt_prec_soln,lower.tail=FALSE));
					}
				} else {
					if(correct[i,k]==1) {
						like <- like - log(pnorm(0.5,sd=1/sqrt_prec_soln)-pnorm(-0.5,sd=1/sqrt_prec_soln));
					} else {
						like <- like - log(2*pnorm(-0.5,sd=1/sqrt_prec_soln));
					}
				}
			}
			return(like);		
		}

		#mle step
		norm_est[[i]] <- mle2(ll_norm_dots,start=list(a=20),lower=(a=0),method="L-BFGS-B");
		
		
		###
		##binary
		breaks_reg_noslope[[i]] <- as.list(mat.or.vec(100,1));
		for(k in 1:100) {
			#run regressions and store r squareds
			breaks_reg_noslope[[i]][[k]] <- lm(correct[i,] ~ inc_dummies[k,]);
			breaks_reg_noslope_r2[i,k] <- summary(breaks_reg_noslope[[i]][[k]])$r.squared;
		}
		
		#find optimal r squared
		breaks_reg_noslope_optinc[i] <- which.max(breaks_reg_noslope_r2[i,]);
		#store this regression
		breaks_reg_noslope_opt[[i]] <- breaks_reg_noslope[[i]][[breaks_reg_noslope_optinc[i]]];
		breaks_reg_noslope_opt_sum[[i]] <- summary(breaks_reg_noslope_opt[[i]]);
		
		
		###
		##tsallis
		ll_tsal_dots <- function(alpha, sigma) {
		  like <- 0; #negative log likelihood
		  
		  for(k in 1:100) {
		    if(abs(sigma-1)<tolerance_sigma) {
		      perf_soln <- exp(inc_matrix[i,k]/alpha)/(4+exp(inc_matrix[i,k]/alpha));
		    } else {
		      
		      #solve for performance
		      solve_perf <- function(perf) {
		        return(
		          alpha*sigma/(sigma-1)*(((1-perf)/4)^(sigma-1)-perf^(sigma-1))+inc_matrix[i,k]
		        );
		      }
		      
		      perf_soln <- try(as.numeric(uniroot(solve_perf,c(0.2,1),extendInt="downX")$root),silent=TRUE);
		      #Looking at the graph, the only way for rootfinding to fail is if the root is greater than 1 (since "roots" smaller than 0.2 will never be chosen). If performance larger than 1 is computed, then this resets it to just below 1 so that a likelihood can actually be computed
		      if("try-error" %in% class(perf_soln)) {
		        perf_soln <- 1-tolerance_perf; 
		      }
		    }
		    
		    like <- like - correct[i,k]*log(perf_soln) - (1-correct[i,k])*log(1-perf_soln);
		  }
		  
		  return(like);		
		}
		
		#mle step
		for(errnum in 1:length(sigma_starts)) {
		  tsal_est[[i]] <- tryCatch(
		    mle2(ll_tsal_dots,start=list(alpha=100,sigma=sigma_starts[errnum]),lower=list(alpha=0,sigma=0),upper=list(alpha=Inf,sigma=Inf),method="L-BFGS-B"),
		    warning=function(w) {
		      return(NA);
		    },
		    error=function(e) {
		      return(NA);
		    }
		  );
		  
		  #break out of loop if success
		  if(!is.na(tsal_est[[i]])) {
		    break;
		  }
		}
		
		#debug, comment out
		print(paste("Fine Tsallis iteration:",i));
		
		
		
		######
		##AIC storage
		#1: shannon
		#2: normal
		#3: binary
		#4: tsallis
		aic_mat_norm_shan[i,j] <- AIC(shan_est[[i]]);
		aic_mat_norm_norm[i,j] <- AIC(norm_est[[i]]);
		aic_mat_norm_bin[i,j] <- AIC(breaks_reg_noslope_opt[[i]])+2; #extra penalty for location estimation
		aic_mat_norm_tsal[i,j] <- tryCatch(AIC(tsal_est[[i]]), error = function(e) Inf);
		
		best_model_norm[i,j] <- which.min(c(aic_mat_norm_shan[i,j],aic_mat_norm_norm[i,j],aic_mat_norm_bin[i,j],aic_mat_norm_tsal[i,j]));
	}	
	
		
		
	###########################
	#########COARSE############
	###########################
		
	random_draws_coarse <- matrix(runif(100*n_subj),nrow=n_subj,ncol=100);
	
	optimal_simulated_precision_coarse <- apply(inc_matrix_coarse,c(1,2),solve_sqrt_prec_inc);
	
	answer_coarse <- matrix(mapply(answer_func,random_draws_coarse,true,optimal_simulated_precision_coarse),nrow=n_subj,ncol=100);
	
	correct_coarse <- (answer_coarse==true)+0;
		
	##conduct estimations##
	##generate required variables
	shan_est_coarse <- as.list(mat.or.vec(n_subj,1));
	norm_est_coarse <- as.list(mat.or.vec(n_subj,1));
		
	breaks_reg_noslope_coarse <- as.list(mat.or.vec(n_subj,1)); #stores regression results for each dummy for each person
	breaks_reg_noslope_r2_coarse <- matrix(nrow=n_subj,ncol=100); #store r squareds for each regression
	breaks_reg_noslope_optinc_coarse <- vector(length=n_subj); #stores optimal incentive level for break
	breaks_reg_noslope_opt_coarse <- as.list(mat.or.vec(n_subj,1)); #stores regression results with the highest r2 for each person
	breaks_reg_noslope_opt_sum_coarse <- as.list(mat.or.vec(n_subj,1)); #stores regression summary with the highest r2 for each person
	
	tsal_est_coarse <- as.list(mat.or.vec(n_subj,1));
		
	for (i in 1:n_subj) {
		###
	  ##shannon
		ll_shannon_dots <- function(a) {
		  -sum(
		     correct[i,]*(
		      inc_matrix[i,]/a
		      -log(
		         4+exp(inc_matrix_coarse[i,]/a)
		      )
		    )+(1-correct[i,])*(
		      log(4)
		      -log(
		         4+exp(inc_matrix_coarse[i,]/a)
		      )
		    )
		  );
		}
		shan_est_coarse[[i]] <- mle2(ll_shannon_dots,method="L-BFGS-B",lower=c(a=0),start=c(a=5));
		  
		###
		##normal signals
		ll_norm_dots <- function(a) {
		  like <- 0; #negative log likelihood
		    
		  for(k in 1:100) {
		      
		    #solve for sqrt of precision
		    solve_sqrt_prec <- function(sqrt_prec) {
		      return(a*sqrt_prec - 2/5*dnorm(0.5*sqrt_prec)*inc_matrix_coarse[i,k]);
		    }
		    sqrt_prec_soln <- nleqslv(2,solve_sqrt_prec)$x;
		      
		    #if true state is 38 or 42, then there's one likelihood (because on edges)
		    #if true state is 39, 40, or 41, then there's another likelihood (because in interior)
		      
		    if(true[i,k] %in% c(38,42)) {
		      if(correct[i,k]==1) {
		        like <- like - log(pnorm(0.5,sd=1/sqrt_prec_soln));
		      } else {
		        like <- like - log(pnorm(0.5,sd=1/sqrt_prec_soln,lower.tail=FALSE));
		      }
		    } else {
		      if(correct[i,k]==1) {
		        like <- like - log(pnorm(0.5,sd=1/sqrt_prec_soln)-pnorm(-0.5,sd=1/sqrt_prec_soln));
		      } else {
		        like <- like - log(2*pnorm(-0.5,sd=1/sqrt_prec_soln));
		      }
		    }
		  }
		  return(like);		
		}
		  
		#mle step
		norm_est_coarse[[i]] <- mle2(ll_norm_dots,start=list(a=20),lower=(a=0),method="L-BFGS-B");
		  
		  
		###
		##binary
		breaks_reg_noslope_coarse[[i]] <- as.list(mat.or.vec(10,1));
		for(k in 1:10) {
		  #run regressions and store r squareds
		  breaks_reg_noslope_coarse[[i]][[k]] <- lm(correct_coarse[i,] ~ inc_dummies_coarse[k,]);
		  breaks_reg_noslope_r2_coarse[i,k] <- summary(breaks_reg_noslope_coarse[[i]][[k]])$r.squared;
		}
		  
		#find optimal r squared
		breaks_reg_noslope_optinc_coarse[i] <- which.max(breaks_reg_noslope_r2_coarse[i,]);
		#store this regression
		breaks_reg_noslope_opt_coarse[[i]] <- breaks_reg_noslope_coarse[[i]][[breaks_reg_noslope_optinc_coarse[i]]];
		breaks_reg_noslope_opt_sum_coarse[[i]] <- summary(breaks_reg_noslope_opt_coarse[[i]]);
		  
		
		###
		##tsallis
		ll_tsal_dots <- function(alpha, sigma) {
		  like <- 0; #negative log likelihood
		  
		  for(k in 1:100) {
		    if(abs(sigma-1)<tolerance_sigma) {
		      perf_soln <- exp(inc_matrix_coarse[i,k]/alpha)/(4+exp(inc_matrix_coarse[i,k]/alpha));
		    } else {
		      
		      #solve for performance
		      solve_perf <- function(perf) {
		        return(
		          alpha*sigma/(sigma-1)*(((1-perf)/4)^(sigma-1)-perf^(sigma-1))+inc_matrix_coarse[i,k]
		        );
		      }
		      
		      perf_soln <- try(as.numeric(uniroot(solve_perf,c(0.2,1),extendInt="downX")$root),silent=TRUE);
		      #Looking at the graph, the only way for rootfinding to fail is if the root is greater than 1 (since "roots" smaller than 0.2 will never be chosen). If performance larger than 1 is computed, then this resets it to just below 1 so that a likelihood can actually be computed
		      if("try-error" %in% class(perf_soln)) {
		        perf_soln <- 1-tolerance_perf; 
		      }
		    }
		    
		    like <- like - correct_coarse[i,k]*log(perf_soln) - (1-correct_coarse[i,k])*log(1-perf_soln);
		  }
		  
		  return(like);		
		}
		
		#mle step
		for(errnum in 1:length(sigma_starts)) {
		  tsal_est_coarse[[i]] <- tryCatch(
		    mle2(ll_tsal_dots,start=list(alpha=100,sigma=sigma_starts[errnum]),lower=list(alpha=0,sigma=0),upper=list(alpha=Inf,sigma=Inf),method="L-BFGS-B"),
		    warning=function(w) {
		      return(NA);
		    },
		    error=function(e) {
		      return(NA);
		    }
		  );
		  
		  #break out of loop if success
		  if(!is.na(tsal_est_coarse[[i]])) {
		    break;
		  }
		}
		
		#debug, comment out
		print(paste("Coarse Tsallis iteration:",i));
		
		  
		######
		##AIC storage
		#1: shannon
		#2: normal
		#3: binary
		#4: tsallis
		aic_mat_norm_shan_coarse[i,j] <- AIC(shan_est_coarse[[i]]);
		aic_mat_norm_norm_coarse[i,j] <- AIC(norm_est_coarse[[i]]);
		aic_mat_norm_bin_coarse[i,j] <- AIC(breaks_reg_noslope_opt_coarse[[i]])+2; #extra penalty for location estimation
		aic_mat_norm_tsal_coarse[i,j] <- tryCatch(AIC(tsal_est_coarse[[i]]), error = function(e) Inf); 
		
		best_model_norm_coarse[i,j] <- which.min(c(aic_mat_norm_shan_coarse[i,j],aic_mat_norm_norm_coarse[i,j],aic_mat_norm_bin_coarse[i,j],aic_mat_norm_tsal_coarse[i,j]));
		  
	}
}
	
#save tables (if out of 100, can be read as percentages)
best_model_norm_table <- apply(best_model_norm,2,table); #fine table
best_model_norm_table_coarse <- apply(best_model_norm_coarse,2,table); #coarse table



##################################
##################################
##################################

###BINARY AS NULL
##set seed, from as.numeric(Sys.time())
set.seed(1566110026);

###Testing parameters
##Start at 0.2 and jump up by break size
##Break locations (actually centre of slope for logistic)
break_locations <- c(25,50,75);
n_loc <- length(break_locations);
##Break sizes 
break_sizes<-c(0.2,0.4,0.6,0.8);
n_size <- length(break_sizes);

#aic matrices
#fine
aic_array_bin_shan <- array(dim=c(n_subj,n_loc,n_size));
aic_array_bin_norm <- array(dim=c(n_subj,n_loc,n_size));
aic_array_bin_bin <- array(dim=c(n_subj,n_loc,n_size));
aic_array_bin_tsal <- array(dim=c(n_subj,n_loc,n_size));
best_model_bin <- array(dim=c(n_subj,n_loc,n_size));
#coarse
aic_array_bin_shan_coarse <- array(dim=c(n_subj,n_loc,n_size));
aic_array_bin_norm_coarse <- array(dim=c(n_subj,n_loc,n_size));
aic_array_bin_bin_coarse <- array(dim=c(n_subj,n_loc,n_size));
aic_array_bin_tsal_coarse <- array(dim=c(n_subj,n_loc,n_size));
best_model_bin_coarse <- array(dim=c(n_subj,n_loc,n_size));


binary_base_time <- as.numeric(Sys.time()); #for debugging to calculate time elapsed

for(j in 1:n_loc) {
for(jj in 1:n_size) {
  
  print(paste("BINARY TRUE MODEL -- iteration: j",j,"jj",jj));
  print(paste("BINARY TRUE MODEL -- time elapsed:",as.numeric(Sys.time())-binary_base_time));
  
  
  ###########################
  ##########FINE#############
  ###########################
  
	##generate correctness data
	draw_thresholds <- 0.2+break_sizes[jj]*((inc_matrix>break_locations[j])+0);
	random_draws <- matrix(runif(100*n_subj),nrow=n_subj,ncol=100);
	correct <- (random_draws<=draw_thresholds)+0; #answered correctly?
	
	##generate mistake data
	random_mistake <- runif(100*n_subj);
	mistake_vector <- potential_mistakes[cbind(ceiling(4*random_mistake),subjects_col,task_col)];
	mistake_matrix <- t(matrix(mistake_vector,nrow=100,ncol=n_subj));
	
	##generate response data
	answer <- correct*true + (1-correct)*mistake_matrix;
	
	##conduct estimations##
	##generate required variables
	shan_est <- as.list(mat.or.vec(n_subj,1));
	norm_est <- as.list(mat.or.vec(n_subj,1));
	
	breaks_reg_noslope <- as.list(mat.or.vec(n_subj,1)); #stores regression results for each dummy for each person
	breaks_reg_noslope_r2 <- matrix(nrow=n_subj,ncol=100); #store r squareds for each regression
	breaks_reg_noslope_optinc <- vector(length=n_subj); #stores optimal incentive level for break
	breaks_reg_noslope_opt <- as.list(mat.or.vec(n_subj,1)); #stores regression results with the highest r2 for each person
	breaks_reg_noslope_opt_sum <- as.list(mat.or.vec(n_subj,1)); #stores regression summary with the highest r2 for each person
	
	tsal_est <- as.list(mat.or.vec(n_subj,1));
	
	for (i in 1:n_subj) {
		###
		##shannon
		ll_shannon_dots <- function(a) {
			-sum(
				correct[i,]*(
					inc_matrix[i,]/a
					-log(
						4+exp(inc_matrix[i,]/a)
					)
				)+(1-correct[i,])*(
					log(4)
					-log(
						4+exp(inc_matrix[i,]/a)
					)
				)
			);
		}
		shan_est[[i]] <- mle2(ll_shannon_dots,method="L-BFGS-B",lower=c(a=0),start=c(a=5));
		
		###
		##normal signals
		ll_norm_dots <- function(a) {
			like <- 0; #negative log likelihood

			for(k in 1:100) {
			  
				#solve for sqrt of precision
				solve_sqrt_prec <- function(sqrt_prec) {
					return(a*sqrt_prec - 2/5*dnorm(0.5*sqrt_prec)*inc_matrix[i,k]);
				}
				sqrt_prec_soln <- nleqslv(2,solve_sqrt_prec)$x;

				#if true state is 38 or 42, then there's one likelihood (because on edges)
				#if true state is 39, 40, or 41, then there's another likelihood (because in interior)

				if(true[i,k] %in% c(38,42)) {
					if(correct[i,k]==1) {
						like <- like - log(pnorm(0.5,sd=1/sqrt_prec_soln));
					} else {
						like <- like - log(pnorm(0.5,sd=1/sqrt_prec_soln,lower.tail=FALSE));
					}
				} else {
					if(correct[i,k]==1) {
						like <- like - log(pnorm(0.5,sd=1/sqrt_prec_soln)-pnorm(-0.5,sd=1/sqrt_prec_soln));
					} else {
						like <- like - log(2*pnorm(-0.5,sd=1/sqrt_prec_soln));
					}
				}
			}
			return(like);		
		}

		#mle step
		norm_est[[i]] <- mle2(ll_norm_dots,start=list(a=20),lower=(a=0),method="L-BFGS-B");
		
		
		###
		##binary
		breaks_reg_noslope[[i]] <- as.list(mat.or.vec(100,1));
		for(k in 1:100) {
			#run regressions and store r squareds
			breaks_reg_noslope[[i]][[k]] <- lm(correct[i,] ~ inc_dummies[k,]);
			breaks_reg_noslope_r2[i,k] <- summary(breaks_reg_noslope[[i]][[k]])$r.squared;
		}
		
		#find optimal r squared
		breaks_reg_noslope_optinc[i] <- which.max(breaks_reg_noslope_r2[i,]);
		#store this regression
		breaks_reg_noslope_opt[[i]] <- breaks_reg_noslope[[i]][[breaks_reg_noslope_optinc[i]]];
		breaks_reg_noslope_opt_sum[[i]] <- summary(breaks_reg_noslope_opt[[i]]);
		
		
		###
		##tsallis
		ll_tsal_dots <- function(alpha, sigma) {
		  like <- 0; #negative log likelihood
		  
		  for(k in 1:100) {
		    if(abs(sigma-1)<tolerance_sigma) {
		      perf_soln <- exp(inc_matrix[i,k]/alpha)/(4+exp(inc_matrix[i,k]/alpha));
		    } else {
		      
		      #solve for performance
		      solve_perf <- function(perf) {
		        return(
		          alpha*sigma/(sigma-1)*(((1-perf)/4)^(sigma-1)-perf^(sigma-1))+inc_matrix[i,k]
		        );
		      }
		      
		      perf_soln <- try(as.numeric(uniroot(solve_perf,c(0.2,1),extendInt="downX")$root),silent=TRUE);
		      #Looking at the graph, the only way for rootfinding to fail is if the root is greater than 1 (since "roots" smaller than 0.2 will never be chosen). If performance larger than 1 is computed, then this resets it to just below 1 so that a likelihood can actually be computed
		      if("try-error" %in% class(perf_soln)) {
		        perf_soln <- 1-tolerance_perf; 
		      }
		    }
		    
		    like <- like - correct[i,k]*log(perf_soln) - (1-correct[i,k])*log(1-perf_soln);
		  }
		  
		  return(like);		
		}
		
		#mle step
		for(errnum in 1:length(sigma_starts)) {
		  tsal_est[[i]] <- tryCatch(
		    mle2(ll_tsal_dots,start=list(alpha=100,sigma=sigma_starts[errnum]),lower=list(alpha=0,sigma=0),upper=list(alpha=Inf,sigma=Inf),method="L-BFGS-B"),
		    warning=function(w) {
		      return(NA);
		    },
		    error=function(e) {
		      return(NA);
		    }
		  );
		  
		  #break out of loop if success
		  if(!is.na(tsal_est[[i]])) {
		    break;
		  }
		}
		
		#debug, comment out
		print(paste("Fine Tsallis iteration:",i));
		
		
		
		######
		##AIC storage
		#1: shannon
		#2: normal
		#3: binary
		#4: tsallis
		aic_array_bin_shan[i,j,jj] <- AIC(shan_est[[i]]);
		aic_array_bin_norm[i,j,jj] <- AIC(norm_est[[i]]);
		aic_array_bin_bin[i,j,jj] <- AIC(breaks_reg_noslope_opt[[i]])+2; #extra penalty for location estimation
		aic_array_bin_tsal[i,j,jj] <- tryCatch(AIC(tsal_est[[i]]), error = function(e) Inf);
		
		best_model_bin[i,j,jj] <- which.min(c(aic_array_bin_shan[i,j,jj],aic_array_bin_norm[i,j,jj],aic_array_bin_bin[i,j,jj],aic_array_bin_tsal[i,j,jj]));

  }
		
		
	###########################
	#########COARSE############
	###########################
		
	##generate correctness data
	draw_thresholds_coarse <- 0.2+break_sizes[jj]*((inc_matrix_coarse>break_locations[j])+0);
	random_draws_coarse <- matrix(runif(100*n_subj),nrow=n_subj,ncol=100);
	correct_coarse <- (random_draws_coarse<=draw_thresholds_coarse)+0; #answered correctly?
	
	##generate mistake data
	random_mistake_coarse <- runif(100*n_subj);
	mistake_vector_coarse <- potential_mistakes[cbind(ceiling(4*random_mistake_coarse),subjects_col,task_col)];
	mistake_matrix_coarse <- t(matrix(mistake_vector_coarse,nrow=100,ncol=n_subj));
	
	##generate response data
	answer_coarse <- correct_coarse*true + (1-correct_coarse)*mistake_matrix_coarse;
		
	##conduct estimations##
	##generate required variables
	shan_est_coarse <- as.list(mat.or.vec(n_subj,1));
	norm_est_coarse <- as.list(mat.or.vec(n_subj,1));
		
	breaks_reg_noslope_coarse <- as.list(mat.or.vec(n_subj,1)); #stores regression results for each dummy for each person
	breaks_reg_noslope_r2_coarse <- matrix(nrow=n_subj,ncol=100); #store r squareds for each regression
	breaks_reg_noslope_optinc_coarse <- vector(length=n_subj); #stores optimal incentive level for break
	breaks_reg_noslope_opt_coarse <- as.list(mat.or.vec(n_subj,1)); #stores regression results with the highest r2 for each person
	breaks_reg_noslope_opt_sum_coarse <- as.list(mat.or.vec(n_subj,1)); #stores regression summary with the highest r2 for each person
	
	tsal_est_coarse <- as.list(mat.or.vec(n_subj,1));
		
	for (i in 1:n_subj) {
		###
	  ##shannon
		ll_shannon_dots <- function(a) {
		  -sum(
		     correct[i,]*(
		      inc_matrix[i,]/a
		      -log(
		         4+exp(inc_matrix_coarse[i,]/a)
		      )
		    )+(1-correct[i,])*(
		      log(4)
		      -log(
		         4+exp(inc_matrix_coarse[i,]/a)
		      )
		    )
		  );
		}
		shan_est_coarse[[i]] <- mle2(ll_shannon_dots,method="L-BFGS-B",lower=c(a=0),start=c(a=5));
		  
		###
		##normal signals
		ll_norm_dots <- function(a) {
		  like <- 0; #negative log likelihood
		    
		  for(k in 1:100) {
		      
		    #solve for sqrt of precision
		    solve_sqrt_prec <- function(sqrt_prec) {
		      return(a*sqrt_prec - 2/5*dnorm(0.5*sqrt_prec)*inc_matrix_coarse[i,k]);
		    }
		    sqrt_prec_soln <- nleqslv(2,solve_sqrt_prec)$x;
		      
		    #if true state is 38 or 42, then there's one likelihood (because on edges)
		    #if true state is 39, 40, or 41, then there's another likelihood (because in interior)
		      
		    if(true[i,k] %in% c(38,42)) {
		      if(correct[i,k]==1) {
		        like <- like - log(pnorm(0.5,sd=1/sqrt_prec_soln));
		      } else {
		        like <- like - log(pnorm(0.5,sd=1/sqrt_prec_soln,lower.tail=FALSE));
		      }
		    } else {
		      if(correct[i,k]==1) {
		        like <- like - log(pnorm(0.5,sd=1/sqrt_prec_soln)-pnorm(-0.5,sd=1/sqrt_prec_soln));
		      } else {
		        like <- like - log(2*pnorm(-0.5,sd=1/sqrt_prec_soln));
		      }
		    }
		  }
		  return(like);		
		}
		  
		#mle step
		norm_est_coarse[[i]] <- mle2(ll_norm_dots,start=list(a=20),lower=(a=0),method="L-BFGS-B");
		  
		  
		###
		##binary
		breaks_reg_noslope_coarse[[i]] <- as.list(mat.or.vec(10,1));
		for(k in 1:10) {
		  #run regressions and store r squareds
		  breaks_reg_noslope_coarse[[i]][[k]] <- lm(correct_coarse[i,] ~ inc_dummies_coarse[k,]);
		  breaks_reg_noslope_r2_coarse[i,k] <- summary(breaks_reg_noslope_coarse[[i]][[k]])$r.squared;
		}
		  
		#find optimal r squared
		breaks_reg_noslope_optinc_coarse[i] <- which.max(breaks_reg_noslope_r2_coarse[i,]);
		#store this regression
		breaks_reg_noslope_opt_coarse[[i]] <- breaks_reg_noslope_coarse[[i]][[breaks_reg_noslope_optinc_coarse[i]]];
		breaks_reg_noslope_opt_sum_coarse[[i]] <- summary(breaks_reg_noslope_opt_coarse[[i]]);
		
		
		###
		##tsallis
		ll_tsal_dots <- function(alpha, sigma) {
		  like <- 0; #negative log likelihood
		  
		  for(k in 1:100) {
		    if(abs(sigma-1)<tolerance_sigma) {
		      perf_soln <- exp(inc_matrix_coarse[i,k]/alpha)/(4+exp(inc_matrix_coarse[i,k]/alpha));
		    } else {
		      
		      #solve for performance
		      solve_perf <- function(perf) {
		        return(
		          alpha*sigma/(sigma-1)*(((1-perf)/4)^(sigma-1)-perf^(sigma-1))+inc_matrix_coarse[i,k]
		        );
		      }
		      
		      perf_soln <- try(as.numeric(uniroot(solve_perf,c(0.2,1),extendInt="downX")$root),silent=TRUE);
		      #Looking at the graph, the only way for rootfinding to fail is if the root is greater than 1 (since "roots" smaller than 0.2 will never be chosen). If performance larger than 1 is computed, then this resets it to just below 1 so that a likelihood can actually be computed
		      if("try-error" %in% class(perf_soln)) {
		        perf_soln <- 1-tolerance_perf; 
		      }
		    }
		    
		    like <- like - correct_coarse[i,k]*log(perf_soln) - (1-correct_coarse[i,k])*log(1-perf_soln);
		  }
		  
		  return(like);		
		}
		
		#mle step
		for(errnum in 1:length(sigma_starts)) {
		  tsal_est_coarse[[i]] <- tryCatch(
		    mle2(ll_tsal_dots,start=list(alpha=100,sigma=sigma_starts[errnum]),lower=list(alpha=0,sigma=0),upper=list(alpha=Inf,sigma=Inf),method="L-BFGS-B"),
		    warning=function(w) {
		      return(NA);
		    },
		    error=function(e) {
		      return(NA);
		    }
		  );
		  
		  #break out of loop if success
		  if(!is.na(tsal_est_coarse[[i]])) {
		    break;
		  }
		}
		
		#debug, comment out
		print(paste("Coarse Tsallis iteration:",i));
		  
		  
		  
		######
		##AIC storage
		#1: shannon
		#2: normal
		#3: binary
		#4: tsallis
		aic_array_bin_shan_coarse[i,j,jj] <- AIC(shan_est_coarse[[i]]);
		aic_array_bin_norm_coarse[i,j,jj] <- AIC(norm_est_coarse[[i]]);
		aic_array_bin_bin_coarse[i,j,jj] <- AIC(breaks_reg_noslope_opt_coarse[[i]])+2; #extra penalty for location estimation
		aic_array_bin_tsal_coarse[i,j,jj] <- tryCatch(AIC(tsal_est_coarse[[i]]), error = function(e) Inf);
		
		best_model_bin_coarse[i,j,jj] <- which.min(c(aic_array_bin_shan_coarse[i,j,jj],aic_array_bin_norm_coarse[i,j,jj],aic_array_bin_bin_coarse[i,j,jj],aic_array_bin_tsal_coarse[i,j,jj]));
	}
}
}

	
#save tables (if out of 100, can be read as percentages)
best_model_bin_table <- apply(best_model_bin,c(2,3),table); #fine table
best_model_bin_table_coarse <- apply(best_model_bin_coarse,c(2,3),table); #coarse table

#print these tables line by line
best_model_bin_table[1,];
best_model_bin_table[2,];
best_model_bin_table[3,];
best_model_bin_table_coarse[1,];
best_model_bin_table_coarse[2,];
best_model_bin_table_coarse[3,];
#end of binary simulation




##################################
##################################
##################################

###TSALLIS AS NULL
##set seed, from as.numeric(Sys.time())
set.seed(1566110051);

###Testing parameters
tsal_slopes <- c(75,250,2000);
n_slope <- length(tsal_slopes);
##Curvature parameters 
tsal_curves<-c(0.5,1.5,2,2.5,4);
n_curve <- length(tsal_curves);

#aic matrices
#fine
aic_array_tsal_shan <- array(dim=c(n_subj,n_slope,n_curve));
aic_array_tsal_norm <- array(dim=c(n_subj,n_slope,n_curve));
aic_array_tsal_bin <- array(dim=c(n_subj,n_slope,n_curve));
aic_array_tsal_tsal <- array(dim=c(n_subj,n_slope,n_curve));
best_model_tsal <- array(dim=c(n_subj,n_slope,n_curve));
#coarse
aic_array_tsal_shan_coarse <- array(dim=c(n_subj,n_slope,n_curve));
aic_array_tsal_norm_coarse <- array(dim=c(n_subj,n_slope,n_curve));
aic_array_tsal_bin_coarse <- array(dim=c(n_subj,n_slope,n_curve));
aic_array_tsal_tsal_coarse <- array(dim=c(n_subj,n_slope,n_curve));
best_model_tsal_coarse <- array(dim=c(n_subj,n_slope,n_curve));


tsallis_base_time <- as.numeric(Sys.time()); #for debugging to calculate time elapsed

for(j in 1:n_slope) {
  for(jj in 1:n_curve) {
    
    print(paste("TSALLIS TRUE MODEL -- iteration: j",j,"jj",jj));
    print(paste("TSALLIS TRUE MODEL -- time elapsed:",as.numeric(Sys.time())-tsallis_base_time));
    
    #functions for solving for threshold, since performance function is implicit
    solve_threshold <- function(incentive) {
      implicit_perf <- function(perf) {
        return(
          tsal_slopes[j]*tsal_curves[jj]/(tsal_curves[jj]-1)*(((1-perf)/4)^(tsal_curves[jj]-1)-perf^(tsal_curves[jj]-1))+incentive
        );
      }
      threshold_solve <- nleqslv(0.7,implicit_perf)$x;
      return(threshold_solve);
    }
    
    ###########################
    ##########FINE#############
    ###########################
    
    ##generate correctness data
    draw_thresholds <- apply(inc_matrix,c(1,2),solve_threshold); #doesn't matter if solution exceeds 1; will be marked as correct anyway
    random_draws <- matrix(runif(100*n_subj),nrow=n_subj,ncol=100);
    correct <- (random_draws<=draw_thresholds)+0; #answered correctly?
    
    ##generate mistake data
    random_mistake <- runif(100*n_subj);
    mistake_vector <- potential_mistakes[cbind(ceiling(4*random_mistake),subjects_col,task_col)];
    mistake_matrix <- t(matrix(mistake_vector,nrow=100,ncol=n_subj));
    
    ##generate response data
    answer <- correct*true + (1-correct)*mistake_matrix;
    
    ##conduct estimations##
    ##generate required variables
    shan_est <- as.list(mat.or.vec(n_subj,1));
    norm_est <- as.list(mat.or.vec(n_subj,1));
    
    breaks_reg_noslope <- as.list(mat.or.vec(n_subj,1)); #stores regression results for each dummy for each person
    breaks_reg_noslope_r2 <- matrix(nrow=n_subj,ncol=100); #store r squareds for each regression
    breaks_reg_noslope_optinc <- vector(length=n_subj); #stores optimal incentive level for break
    breaks_reg_noslope_opt <- as.list(mat.or.vec(n_subj,1)); #stores regression results with the highest r2 for each person
    breaks_reg_noslope_opt_sum <- as.list(mat.or.vec(n_subj,1)); #stores regression summary with the highest r2 for each person
    
    tsal_est <- as.list(mat.or.vec(n_subj,1));
    
    for (i in 1:n_subj) {
      ###
      ##shannon
      ll_shannon_dots <- function(a) {
        -sum(
          correct[i,]*(
            inc_matrix[i,]/a
            -log(
              4+exp(inc_matrix[i,]/a)
            )
          )+(1-correct[i,])*(
            log(4)
            -log(
              4+exp(inc_matrix[i,]/a)
            )
          )
        );
      }
      shan_est[[i]] <- mle2(ll_shannon_dots,method="L-BFGS-B",lower=c(a=0),start=c(a=5));
      
      ###
      ##normal signals
      ll_norm_dots <- function(a) {
        like <- 0; #negative log likelihood
        
        for(k in 1:100) {
          
          #solve for sqrt of precision
          solve_sqrt_prec <- function(sqrt_prec) {
            return(a*sqrt_prec - 2/5*dnorm(0.5*sqrt_prec)*inc_matrix[i,k]);
          }
          sqrt_prec_soln <- nleqslv(2,solve_sqrt_prec)$x;
          
          #if true state is 38 or 42, then there's one likelihood (because on edges)
          #if true state is 39, 40, or 41, then there's another likelihood (because in interior)
          
          if(true[i,k] %in% c(38,42)) {
            if(correct[i,k]==1) {
              like <- like - log(pnorm(0.5,sd=1/sqrt_prec_soln));
            } else {
              like <- like - log(pnorm(0.5,sd=1/sqrt_prec_soln,lower.tail=FALSE));
            }
          } else {
            if(correct[i,k]==1) {
              like <- like - log(pnorm(0.5,sd=1/sqrt_prec_soln)-pnorm(-0.5,sd=1/sqrt_prec_soln));
            } else {
              like <- like - log(2*pnorm(-0.5,sd=1/sqrt_prec_soln));
            }
          }
        }
        return(like);		
      }
      
      #mle step
      norm_est[[i]] <- mle2(ll_norm_dots,start=list(a=20),lower=(a=0),method="L-BFGS-B");
      
      
      ###
      ##binary
      breaks_reg_noslope[[i]] <- as.list(mat.or.vec(100,1));
      for(k in 1:100) {
        #run regressions and store r squareds
        breaks_reg_noslope[[i]][[k]] <- lm(correct[i,] ~ inc_dummies[k,]);
        breaks_reg_noslope_r2[i,k] <- summary(breaks_reg_noslope[[i]][[k]])$r.squared;
      }
      
      #find optimal r squared
      breaks_reg_noslope_optinc[i] <- which.max(breaks_reg_noslope_r2[i,]);
      #store this regression
      breaks_reg_noslope_opt[[i]] <- breaks_reg_noslope[[i]][[breaks_reg_noslope_optinc[i]]];
      breaks_reg_noslope_opt_sum[[i]] <- summary(breaks_reg_noslope_opt[[i]]);
      
      
      ###
      ##tsallis
      ll_tsal_dots <- function(alpha, sigma) {
        like <- 0; #negative log likelihood
        
        for(k in 1:100) {
          if(abs(sigma-1)<tolerance_sigma) {
            perf_soln <- exp(inc_matrix[i,k]/alpha)/(4+exp(inc_matrix[i,k]/alpha));
          } else {
            
            #solve for performance
            solve_perf <- function(perf) {
              return(
                alpha*sigma/(sigma-1)*(((1-perf)/4)^(sigma-1)-perf^(sigma-1))+inc_matrix[i,k]
              );
            }
            
            perf_soln <- try(as.numeric(uniroot(solve_perf,c(0.2,1),extendInt="downX")$root),silent=TRUE);
            #Looking at the graph, the only way for rootfinding to fail is if the root is greater than 1 (since "roots" smaller than 0.2 will never be chosen). If performance larger than 1 is computed, then this resets it to just below 1 so that a likelihood can actually be computed
            if("try-error" %in% class(perf_soln)) {
              perf_soln <- 1-tolerance_perf; 
            }
          }
          
          like <- like - correct[i,k]*log(perf_soln) - (1-correct[i,k])*log(1-perf_soln);
        }
        
        return(like);		
      }
      
      #mle step
      for(errnum in 1:length(sigma_starts)) {
        tsal_est[[i]] <- tryCatch(
          mle2(ll_tsal_dots,start=list(alpha=100,sigma=sigma_starts[errnum]),lower=list(alpha=0,sigma=0),upper=list(alpha=Inf,sigma=Inf),method="L-BFGS-B"),
          warning=function(w) {
            return(NA);
          },
          error=function(e) {
            return(NA);
          }
        );
        
        #break out of loop if success
        if(!is.na(tsal_est[[i]])) {
          break;
        }
      }
      
      #debug, comment out
      print(paste("Fine Tsallis iteration:",i));
      
      
      
      ######
      ##AIC storage
      #1: shannon
      #2: normal
      #3: binary
      #4: tsallis
      aic_array_tsal_shan[i,j,jj] <- AIC(shan_est[[i]]);
      aic_array_tsal_norm[i,j,jj] <- AIC(norm_est[[i]]);
      aic_array_tsal_bin[i,j,jj] <- AIC(breaks_reg_noslope_opt[[i]])+2; #extra penalty for location estimation
      aic_array_tsal_tsal[i,j,jj] <- tryCatch(AIC(tsal_est[[i]]), error = function(e) Inf);
      
      best_model_tsal[i,j,jj] <- which.min(c(aic_array_tsal_shan[i,j,jj],aic_array_tsal_norm[i,j,jj],aic_array_tsal_bin[i,j,jj],aic_array_tsal_tsal[i,j,jj]));
      
    }
    
    
    ###########################
    #########COARSE############
    ###########################
    
    ##generate correctness data
    draw_thresholds_coarse <- apply(inc_matrix_coarse,c(1,2),solve_threshold); #doesn't matter if solution exceeds 1; will be marked as correct anyway
    random_draws_coarse <- matrix(runif(100*n_subj),nrow=n_subj,ncol=100);
    correct_coarse <- (random_draws_coarse<=draw_thresholds_coarse)+0; #answered correctly?
    
    ##generate mistake data
    random_mistake_coarse <- runif(100*n_subj);
    mistake_vector_coarse <- potential_mistakes[cbind(ceiling(4*random_mistake_coarse),subjects_col,task_col)];
    mistake_matrix_coarse <- t(matrix(mistake_vector_coarse,nrow=100,ncol=n_subj));
    
    ##generate response data
    answer_coarse <- correct_coarse*true + (1-correct_coarse)*mistake_matrix_coarse;
    
    ##conduct estimations##
    ##generate required variables
    shan_est_coarse <- as.list(mat.or.vec(n_subj,1));
    norm_est_coarse <- as.list(mat.or.vec(n_subj,1));
    
    breaks_reg_noslope_coarse <- as.list(mat.or.vec(n_subj,1)); #stores regression results for each dummy for each person
    breaks_reg_noslope_r2_coarse <- matrix(nrow=n_subj,ncol=100); #store r squareds for each regression
    breaks_reg_noslope_optinc_coarse <- vector(length=n_subj); #stores optimal incentive level for break
    breaks_reg_noslope_opt_coarse <- as.list(mat.or.vec(n_subj,1)); #stores regression results with the highest r2 for each person
    breaks_reg_noslope_opt_sum_coarse <- as.list(mat.or.vec(n_subj,1)); #stores regression summary with the highest r2 for each person
    
    tsal_est_coarse <- as.list(mat.or.vec(n_subj,1));
    
    for (i in 1:n_subj) {
      ###
      ##shannon
      ll_shannon_dots <- function(a) {
        -sum(
          correct[i,]*(
            inc_matrix[i,]/a
            -log(
              4+exp(inc_matrix_coarse[i,]/a)
            )
          )+(1-correct[i,])*(
            log(4)
            -log(
              4+exp(inc_matrix_coarse[i,]/a)
            )
          )
        );
      }
      shan_est_coarse[[i]] <- mle2(ll_shannon_dots,method="L-BFGS-B",lower=c(a=0),start=c(a=5));
      
      ###
      ##normal signals
      ll_norm_dots <- function(a) {
        like <- 0; #negative log likelihood
        
        for(k in 1:100) {
          
          #solve for sqrt of precision
          solve_sqrt_prec <- function(sqrt_prec) {
            return(a*sqrt_prec - 2/5*dnorm(0.5*sqrt_prec)*inc_matrix_coarse[i,k]);
          }
          sqrt_prec_soln <- nleqslv(2,solve_sqrt_prec)$x;
          
          #if true state is 38 or 42, then there's one likelihood (because on edges)
          #if true state is 39, 40, or 41, then there's another likelihood (because in interior)
          
          if(true[i,k] %in% c(38,42)) {
            if(correct[i,k]==1) {
              like <- like - log(pnorm(0.5,sd=1/sqrt_prec_soln));
            } else {
              like <- like - log(pnorm(0.5,sd=1/sqrt_prec_soln,lower.tail=FALSE));
            }
          } else {
            if(correct[i,k]==1) {
              like <- like - log(pnorm(0.5,sd=1/sqrt_prec_soln)-pnorm(-0.5,sd=1/sqrt_prec_soln));
            } else {
              like <- like - log(2*pnorm(-0.5,sd=1/sqrt_prec_soln));
            }
          }
        }
        return(like);		
      }
      
      #mle step
      norm_est_coarse[[i]] <- mle2(ll_norm_dots,start=list(a=20),lower=(a=0),method="L-BFGS-B");
      
      
      ###
      ##binary
      breaks_reg_noslope_coarse[[i]] <- as.list(mat.or.vec(10,1));
      for(k in 1:10) {
        #run regressions and store r squareds
        breaks_reg_noslope_coarse[[i]][[k]] <- lm(correct_coarse[i,] ~ inc_dummies_coarse[k,]);
        breaks_reg_noslope_r2_coarse[i,k] <- summary(breaks_reg_noslope_coarse[[i]][[k]])$r.squared;
      }
      
      #find optimal r squared
      breaks_reg_noslope_optinc_coarse[i] <- which.max(breaks_reg_noslope_r2_coarse[i,]);
      #store this regression
      breaks_reg_noslope_opt_coarse[[i]] <- breaks_reg_noslope_coarse[[i]][[breaks_reg_noslope_optinc_coarse[i]]];
      breaks_reg_noslope_opt_sum_coarse[[i]] <- summary(breaks_reg_noslope_opt_coarse[[i]]);
      
      
      ###
      ##tsallis
      ll_tsal_dots <- function(alpha, sigma) {
        like <- 0; #negative log likelihood
        
        for(k in 1:100) {
          if(abs(sigma-1)<tolerance_sigma) {
            perf_soln <- exp(inc_matrix_coarse[i,k]/alpha)/(4+exp(inc_matrix_coarse[i,k]/alpha));
          } else {
            
            #solve for performance
            solve_perf <- function(perf) {
              return(
                alpha*sigma/(sigma-1)*(((1-perf)/4)^(sigma-1)-perf^(sigma-1))+inc_matrix_coarse[i,k]
              );
            }
            
            perf_soln <- try(as.numeric(uniroot(solve_perf,c(0.2,1),extendInt="downX")$root),silent=TRUE);
            #Looking at the graph, the only way for rootfinding to fail is if the root is greater than 1 (since "roots" smaller than 0.2 will never be chosen). If performance larger than 1 is computed, then this resets it to just below 1 so that a likelihood can actually be computed
            if("try-error" %in% class(perf_soln)) {
              perf_soln <- 1-tolerance_perf; 
            }
          }
          
          like <- like - correct_coarse[i,k]*log(perf_soln) - (1-correct_coarse[i,k])*log(1-perf_soln);
        }
        
        return(like);		
      }
      
      #mle step
      for(errnum in 1:length(sigma_starts)) {
        tsal_est_coarse[[i]] <- tryCatch(
          mle2(ll_tsal_dots,start=list(alpha=100,sigma=sigma_starts[errnum]),lower=list(alpha=0,sigma=0),upper=list(alpha=Inf,sigma=Inf),method="L-BFGS-B"),
          warning=function(w) {
            return(NA);
          },
          error=function(e) {
            return(NA);
          }
        );
        
        #break out of loop if success
        if(!is.na(tsal_est_coarse[[i]])) {
          break;
        }
      }
      
      #debug, comment out
      print(paste("Coarse Tsallis iteration:",i));
      
      
      
      ######
      ##AIC storage
      #1: shannon
      #2: normal
      #3: binary
      #4: tsallis
      aic_array_tsal_shan_coarse[i,j,jj] <- AIC(shan_est_coarse[[i]]);
      aic_array_tsal_norm_coarse[i,j,jj] <- AIC(norm_est_coarse[[i]]);
      aic_array_tsal_bin_coarse[i,j,jj] <- AIC(breaks_reg_noslope_opt_coarse[[i]])+2; #extra penalty for location estimation
      aic_array_tsal_tsal_coarse[i,j,jj] <- tryCatch(AIC(tsal_est_coarse[[i]]), error = function(e) Inf);
      
      best_model_tsal_coarse[i,j,jj] <- which.min(c(aic_array_tsal_shan_coarse[i,j,jj],aic_array_tsal_norm_coarse[i,j,jj],aic_array_tsal_bin_coarse[i,j,jj],aic_array_tsal_tsal_coarse[i,j,jj]));
    }
  }
}


#save tables (if out of 100, can be read as percentages)
best_model_tsal_table <- apply(best_model_tsal,c(2,3),table); #fine table
best_model_tsal_table_coarse <- apply(best_model_tsal_coarse,c(2,3),table); #coarse table

#print these tables line by line
best_model_tsal_table[1,];
best_model_tsal_table[2,];
best_model_tsal_table[3,];
best_model_tsal_table_coarse[1,];
best_model_tsal_table_coarse[2,];
best_model_tsal_table_coarse[3,];
#end of tsallis simulation


