####This script compares full-sample analysis to half-sample analyses.

#Consolidate relevant variables
#Ensure that relevant data files are in working directory before running

##########################
###############FULL SAMPLE

##First, run analysis for entire sample
##Save data, comment out if already saved
#save.image("variables_full.RData");

##Load data, comment out if already loaded
load("variables_full.RData");

#Variables that need to be saved:
#-mon_subj - subjects who satisfy niac
#-nias_subj - subjects who satisfy nias
#-rat_subj - subjects who satisfy both
#-str_mon_subj - responsive subjects
#-well_behaved_subj - well-behaved subjects

#-aic_matrix_dots
#-best_model_dots

#create list that stores all these data
class_fullsample <- list(mon_subj,nias_subj,rat_subj,str_mon_subj,well_behaved_subj,aic_matrix_dots,best_model_dots);
names(class_fullsample) <- c("niac","nias","ratl","resp","well","aicm","best"); #assign names

#remove unnecessary variables
rm(list=setdiff(ls(),"class_fullsample"));


################################
###############FIRST HALF SAMPLE

##First, run analysis for entire sample
##Save data, comment out if already saved
#save.image("variables_first50.RData");

##Load data, comment out if already loaded
load("variables_first50.RData");

#Variables that need to be saved:
#-mon_subj - subjects who satisfy niac
#-nias_subj - subjects who satisfy nias
#-rat_subj - subjects who satisfy both
#-str_mon_subj - responsive subjects
#-well_behaved_subj - well-behaved subjects

#-aic_matrix_dots
#-best_model_dots

#create list that stores all these data
class_first50 <- list(mon_subj,nias_subj,rat_subj,str_mon_subj,well_behaved_subj,aic_matrix_dots,best_model_dots);
names(class_first50) <- c("niac","nias","ratl","resp","well","aicm","best"); #assign names

#remove unnecessary variables
rm(list=setdiff(ls(),c("class_fullsample","class_first50")));


#################################
###############SECOND HALF SAMPLE

##First, run analysis for entire sample
##Save data, comment out if already saved
#save.image("variables_last50.RData");

##Load data, comment out if already loaded
load("variables_last50.RData");

#Variables that need to be saved:
#-mon_subj - subjects who satisfy niac
#-nias_subj - subjects who satisfy nias
#-rat_subj - subjects who satisfy both
#-str_mon_subj - responsive subjects
#-well_behaved_subj - well-behaved subjects

#-aic_matrix_dots
#-best_model_dots

#create list that stores all these data
class_last50 <- list(mon_subj,nias_subj,rat_subj,str_mon_subj,well_behaved_subj,aic_matrix_dots,best_model_dots);
names(class_last50) <- c("niac","nias","ratl","resp","well","aicm","best"); #assign names

#remove unnecessary variables
rm(list=setdiff(ls(),c("class_fullsample","class_first50","class_last50")));


####################
####################
####################
####################


#CONSISTENCY OF CLASSIFICATIONS

#niac
niac_allfirst <- intersect(class_fullsample$niac,class_first50$niac);
niac_alllast <- intersect(class_fullsample$niac,class_last50$niac);
niac_firstlast <- intersect(class_first50$niac,class_last50$niac);
niac_3 <- intersect(class_fullsample$niac,niac_firstlast);

#nias
nias_allfirst <- intersect(class_fullsample$nias,class_first50$nias);
nias_alllast <- intersect(class_fullsample$nias,class_last50$nias);
nias_firstlast <- intersect(class_first50$nias,class_last50$nias);
nias_3 <- intersect(class_fullsample$nias,nias_firstlast);

#rational
ratl_allfirst <- intersect(class_fullsample$ratl,class_first50$ratl);
ratl_alllast <- intersect(class_fullsample$ratl,class_last50$ratl);
ratl_firstlast <- intersect(class_first50$ratl,class_last50$ratl);
ratl_3 <- intersect(class_fullsample$ratl,ratl_firstlast);

#responsiveness
resp_allfirst <- intersect(class_fullsample$resp,class_first50$resp);
resp_alllast <- intersect(class_fullsample$resp,class_last50$resp);
resp_firstlast <- intersect(class_first50$resp,class_last50$resp);
resp_3 <- intersect(class_fullsample$resp,resp_firstlast);
resp_one <- union(class_fullsample$resp,union(class_first50$resp,class_last50$resp));

#well-behaved
well_allfirst <- intersect(class_fullsample$well,class_first50$well);
well_alllast <- intersect(class_fullsample$well,class_last50$well);
well_firstlast <- intersect(class_first50$well,class_last50$well);
well_3 <- intersect(class_fullsample$well,well_firstlast);

#correlations of AICs
#responsive in at least one sample
#(can't do every subject; no tsallis computation for some non-responsive subjects)
corr_all_allfirst <- diag(cor(class_fullsample$aicm[resp_one,],class_first50$aicm[resp_one,]));
corr_all_alllast <- diag(cor(class_fullsample$aicm[resp_one,],class_last50$aicm[resp_one,]));
corr_all_firstlast <- diag(cor(class_first50$aicm[resp_one,],class_last50$aicm[resp_one,]));
#responsive in full sample
corr_respf_allfirst <- diag(cor(class_fullsample$aicm[class_fullsample$resp,],class_first50$aicm[class_fullsample$resp,]));
corr_respf_alllast <- diag(cor(class_fullsample$aicm[class_fullsample$resp,],class_last50$aicm[class_fullsample$resp,]));
corr_respf_firstlast <- diag(cor(class_first50$aicm[class_fullsample$resp,],class_last50$aicm[class_fullsample$resp,]));
#responsive in all 3 samples
corr_resp3_allfirst <- diag(cor(class_fullsample$aicm[resp_3,],class_first50$aicm[resp_3,]));
corr_resp3_alllast <- diag(cor(class_fullsample$aicm[resp_3,],class_last50$aicm[resp_3,]));
corr_resp3_firstlast <- diag(cor(class_first50$aicm[resp_3,],class_last50$aicm[resp_3,]));


#############

#for best models, look at two-way tables

#full and odd
t(table(class_fullsample$best[resp_allfirst],class_first50$best[resp_allfirst]));

#full and even
t(table(class_fullsample$best[resp_alllast],class_last50$best[resp_alllast]));

#odd and even
t(table(class_first50$best[resp_firstlast],class_last50$best[resp_firstlast]));