############
############

###POWER TESTS FOR DISCONTINUITY TEST

###install relevant packages
##stargazer [for latex output]
##bbmle [for better mle]

install.packages("stargazer");
install.packages("bbmle");

library(stargazer);
library(bbmle);


###Set random seed
##Obtained from as.numeric(Sys.time())
set.seed(1505346186);

###Set number of "subjects" for power test (should be 100)
n_subj <- 100;

###Set number of tasks (should be 100)
num_tasks <- 100;

###Testing parameters --- variable names reflect that the code was originally written with discontinuity as alternative, not null
##Start at 0.2 and curve up to upper limit
##Break locations (actually centre of slope for logistic)
break_locations <- c(25,50,75);
n_loc <- length(break_locations);
##Break sizes 
break_sizes<-c(0.2,0.4,0.6,0.8);
n_size <- length(break_sizes);
##Slope
slopes <- c(0.05, 0.1, 0.3, 1, 4);
n_slope <- length(slopes);

###Define function that generates probabilities
prob_gen <- function(incentive,size,location,slope) {
	prob <- 0.2 + size/(1+exp(-slope*(incentive-location)));
	return(prob);
}



#######################
###FINE INCENTIVES
initial_time <- as.numeric(Sys.time());

###Generate random incentive orders
incentives_unrand <- matrix(1:num_tasks,nrow=num_tasks,ncol=n_subj);
incentives <- t(apply(incentives_unrand,2,sample));

###Generate blank draw and correctness arrays
draw <- array(dim=c(n_subj,n_size,n_loc,n_slope,num_tasks)); #random number from which correctness determined (keep this array around for bugtesting)
correct <- array(dim=c(n_subj,n_size,n_loc,n_slope,num_tasks));


###Create necessary vectors; fill in as regressions are run
inc_dummies_dots <- array(dim=c(n_subj,num_tasks,num_tasks)); #creates vectors of dummies for whether or not the incentive level is at least the dummy level
breaks_reg_noslope_dots <- as.list(mat.or.vec(n_subj,1)); #stores regression results for each dummy for each person
breaks_reg_noslope_r2_dots <- array(dim=c(n_subj,n_size,n_loc,n_slope,num_tasks)); #store r squareds for each regression
breaks_reg_noslope_optinc_dots <- array(dim=c(n_subj,n_size,n_loc,n_slope)); #stores optimal incentive level for break
breaks_reg_noslope_opt_dots <- as.list(mat.or.vec(n_subj,1)); #stores regression results with the highest r2 for each person
breaks_reg_noslope_opt_dots_sum <- as.list(mat.or.vec(n_subj,1)); #stores regression summary with the highest r2 for each person
mle_logistic <- as.list(mat.or.vec(n_subj,1)); #stores mle results
ll_test_pval <- array(dim=c(n_subj,n_size,n_loc,n_slope)); #stores p-values for each "subject" for each break size and location

###Populate the vectors with empty lists for each location/size
for(i in 1:n_subj) {
	#for debugging; comment out as necessary
	print(paste("i",i));

	breaks_reg_noslope_dots[[i]] <- as.list(mat.or.vec(n_size,1));
	breaks_reg_noslope_opt_dots[[i]] <- as.list(mat.or.vec(n_size,1)); 
	breaks_reg_noslope_opt_dots_sum[[i]] <- as.list(mat.or.vec(n_size,1)); 
	mle_logistic[[i]] <- as.list(mat.or.vec(n_size,1));
	
	##Populate each size list with an empty location list
	for(j in 1:n_size) {
		#for debugging; comment out as necessary
		print(paste("i",i,"j",j));
		
		breaks_reg_noslope_dots[[i]][[j]] <- as.list(mat.or.vec(n_loc,1));
		breaks_reg_noslope_opt_dots[[i]][[j]] <- as.list(mat.or.vec(n_loc,1)); 
		breaks_reg_noslope_opt_dots_sum[[i]][[j]] <- as.list(mat.or.vec(n_loc,1)); 
		mle_logistic[[i]][[j]] <- as.list(mat.or.vec(n_loc,1));

		for(k in 1:n_loc) {
			#for debugging; comment out as necessary
			print(paste("i",i,"j",j,"k",k));
			
			##Populate each size/location list with an empty slope list
			breaks_reg_noslope_dots[[i]][[j]][[k]] <- as.list(mat.or.vec(n_slope,1));
			breaks_reg_noslope_opt_dots[[i]][[j]][[k]] <- as.list(mat.or.vec(n_slope,1)); 
			breaks_reg_noslope_opt_dots_sum[[i]][[j]][[k]] <- as.list(mat.or.vec(n_slope,1)); 
			mle_logistic[[i]][[j]][[k]] <- as.list(mat.or.vec(n_slope,1));
				
			for(l in 1:n_slope) {
				#for debugging; comment out as necessary
				print(paste("i",i,"j",j,"k",k,"l",l));
				print(paste("Time elapsed:",(as.numeric(Sys.time())-initial_time)/60,"minutes"));
				
				##Empty list for each incentive level
				breaks_reg_noslope_dots[[i]][[j]][[k]][[l]] <- as.list(mat.or.vec(num_tasks,1));

				##Generate random data
				draw[i,j,k,l,] <- runif(num_tasks); #random draw
				probs <- apply(as.matrix(incentives[i,]),1,prob_gen,break_sizes[j],break_locations[k],slopes[l]);

				correct[i,j,k,l,] <- (draw[i,j,k,l,]<=probs)+0; #if random number falls below prob, then mark correct

				##Perform estimations
				#Break, no slope
				for(m in 1:num_tasks) {
					#populate inc_dummies_dots for subject i
					inc_dummies_dots[i,m,] <- (incentives[i,] >= m)+0;
					#run regressions and store r squareds
					breaks_reg_noslope_dots[[i]][[j]][[k]][[l]][[m]] <- lm(correct[i,j,k,l,] ~ inc_dummies_dots[i,m,]);
					breaks_reg_noslope_r2_dots[i,j,k,l,m] <- summary(breaks_reg_noslope_dots[[i]][[j]][[k]][[l]][[m]])$r.squared;
				}
				#find optimal r squared
				breaks_reg_noslope_optinc_dots[i,j,k,l] <- which.max(breaks_reg_noslope_r2_dots[i,j,k,l,]);
				#store this regression
				breaks_reg_noslope_opt_dots[[i]][[j]][[k]][[l]] <- breaks_reg_noslope_dots[[i]][[j]][[k]][[l]][[breaks_reg_noslope_optinc_dots[i,j,k,l]]];
				breaks_reg_noslope_opt_dots_sum[[i]][[j]][[k]][[l]] <- summary(breaks_reg_noslope_opt_dots[[i]][[j]][[k]][[l]]);

				#Flexible MLE
				##Set MLE function
				ll_logistic <- function(a,b,c,d) {
					log_likelihood <- 0;
					for(m in 1:num_tasks) {
						log_likelihood <- log_likelihood+log((a/(1+exp(-b*(incentives[i,m]-c)))+d))*(correct[i,j,k,l,m])+log((1-a/(1+exp(-b*(incentives[i,m]-c)))-d))*(1-correct[i,j,k,l,m]);
					}
					return(-log_likelihood);
				}

				mle_logistic[[i]][[j]][[k]][[l]] <- mle2(ll_logistic,method="SANN",start=list(a=0.5,b=1,c=50,d=0.2));

				discts_ll <- logLik(breaks_reg_noslope_opt_dots[[i]][[j]][[k]][[l]]);
				cts_ll <- logLik(mle_logistic[[i]][[j]][[k]][[l]]);
				ll_stat <- -2*(discts_ll-cts_ll);
				ll_test_pval[i,j,k,l] <- pchisq(ll_stat,1,lower.tail=FALSE);
			
			}
		}
	}
}

###Power calculations
poorly_behaved <- (ll_test_pval > 0.05)+0; #mark "subjects" for whom discontinuity cannot be rejected

power_discty <- 1-apply(poorly_behaved,c(2,3,4),mean); #power array: shows simulated test power for each size/location/slope

dimnames(power_discty)[[1]] <- break_sizes;
dimnames(power_discty)[[2]] <- break_locations;
dimnames(power_discty)[[3]] <- slopes;

#####
##remove some variables to save RAM
##comment out if you have LOTS of RAM (16-32 GB)
rm("inc_dummies_dots","breaks_reg_noslope_dots","breaks_reg_noslope_r2_dots","breaks_reg_noslope_optinc_dots","breaks_reg_noslope_opt_dots","breaks_reg_noslope_opt_dots_sum","mle_logistic");


##################

###COARSE INCENTIVE POWER TEST

initial_time_coarse <- as.numeric(Sys.time());

###Generate matrix of incentives
incentives_coarse_slice <- cbind(rep(10,n_subj),rep(20,n_subj),rep(30,n_subj),rep(40,n_subj),rep(50,n_subj),rep(60,n_subj),rep(70,n_subj),rep(80,n_subj),rep(90,n_subj),rep(100,n_subj));
incentives_coarse <- matrix(rep(incentives_coarse_slice,10),nrow=100,ncol=100);


###Generate blank draw and correctness arrays
draw_coarse <- array(dim=c(n_subj,n_size,n_loc,n_slope,num_tasks)); #random number from which correctness determined (keep this array around for bugtesting)
correct_coarse <- array(dim=c(n_subj,n_size,n_loc,n_slope,num_tasks));


###Create necessary vectors; fill in as regressions are run
inc_dummies_dots_coarse <- array(dim=c(n_subj,num_tasks,num_tasks)); #creates vectors of dummies for whether or not the incentive level is at least the dummy level
breaks_reg_noslope_dots_coarse <- as.list(mat.or.vec(n_subj,1)); #stores regression results for each dummy for each person
breaks_reg_noslope_r2_dots_coarse <- array(dim=c(n_subj,n_size,n_loc,n_slope,num_tasks)); #store r squareds for each regression
breaks_reg_noslope_optinc_dots_coarse <- array(dim=c(n_subj,n_size,n_loc,n_slope)); #stores optimal incentive level for break
breaks_reg_noslope_opt_dots_coarse <- as.list(mat.or.vec(n_subj,1)); #stores regression results with the highest r2 for each person
breaks_reg_noslope_opt_dots_sum_coarse <- as.list(mat.or.vec(n_subj,1)); #stores regression summary with the highest r2 for each person
mle_logistic_coarse <- as.list(mat.or.vec(n_subj,1)); #stores mle results
ll_test_pval_coarse <- array(dim=c(n_subj,n_size,n_loc,n_slope)); #stores p-values for each "subject" for each break size and location

###Populate the vectors with empty lists for each location/size
for(i in 1:n_subj) {
	#for debugging; comment out as necessary
	print(paste("i",i));

	breaks_reg_noslope_dots_coarse[[i]] <- as.list(mat.or.vec(n_size,1));
	breaks_reg_noslope_opt_dots_coarse[[i]] <- as.list(mat.or.vec(n_size,1)); 
	breaks_reg_noslope_opt_dots_sum_coarse[[i]] <- as.list(mat.or.vec(n_size,1)); 
	mle_logistic_coarse[[i]] <- as.list(mat.or.vec(n_size,1));
	
	##Populate each size list with an empty location list
	for(j in 1:n_size) {
		#for debugging; comment out as necessary
		print(paste("i",i,"j",j));
		
		breaks_reg_noslope_dots_coarse[[i]][[j]] <- as.list(mat.or.vec(n_loc,1));
		breaks_reg_noslope_opt_dots_coarse[[i]][[j]] <- as.list(mat.or.vec(n_loc,1)); 
		breaks_reg_noslope_opt_dots_sum_coarse[[i]][[j]] <- as.list(mat.or.vec(n_loc,1)); 
		mle_logistic_coarse[[i]][[j]] <- as.list(mat.or.vec(n_loc,1));

		for(k in 1:n_loc) {
			#for debugging; comment out as necessary
			print(paste("i",i,"j",j,"k",k));
			
			##Populate each size/location list with an empty slope list
			breaks_reg_noslope_dots_coarse[[i]][[j]][[k]] <- as.list(mat.or.vec(n_slope,1));
			breaks_reg_noslope_opt_dots_coarse[[i]][[j]][[k]] <- as.list(mat.or.vec(n_slope,1)); 
			breaks_reg_noslope_opt_dots_sum_coarse[[i]][[j]][[k]] <- as.list(mat.or.vec(n_slope,1)); 
			mle_logistic_coarse[[i]][[j]][[k]] <- as.list(mat.or.vec(n_slope,1));
				
			for(l in 1:n_slope) {
				#for debugging; comment out as necessary
				print(paste("i",i,"j",j,"k",k,"l",l));
				print(paste("COARSE Time elapsed:",(as.numeric(Sys.time())-initial_time_coarse)/60,"minutes"));
				
				##Empty list for each incentive level
				breaks_reg_noslope_dots_coarse[[i]][[j]][[k]][[l]] <- as.list(mat.or.vec(num_tasks,1));

				##Generate random data
				draw_coarse[i,j,k,l,] <- runif(num_tasks); #random draw
				probs <- apply(as.matrix(incentives_coarse[i,]),1,prob_gen,break_sizes[j],break_locations[k],slopes[l]);

				correct_coarse[i,j,k,l,] <- (draw_coarse[i,j,k,l,]<=probs)+0; #if random number falls below prob, then mark correct

				##Perform estimations
				#Break, no slope
				for(m in 1:num_tasks) {
					#populate inc_dummies_dots_coarse for subject i
					inc_dummies_dots_coarse[i,m,] <- (incentives_coarse[i,] >= m)+0;
					#run regressions and store r squareds
					breaks_reg_noslope_dots_coarse[[i]][[j]][[k]][[l]][[m]] <- lm(correct_coarse[i,j,k,l,] ~ inc_dummies_dots_coarse[i,m,]);
					breaks_reg_noslope_r2_dots_coarse[i,j,k,l,m] <- summary(breaks_reg_noslope_dots_coarse[[i]][[j]][[k]][[l]][[m]])$r.squared;
				}
				#find optimal r squared
				breaks_reg_noslope_optinc_dots_coarse[i,j,k,l] <- which.max(breaks_reg_noslope_r2_dots_coarse[i,j,k,l,]);
				#store this regression
				breaks_reg_noslope_opt_dots_coarse[[i]][[j]][[k]][[l]] <- breaks_reg_noslope_dots_coarse[[i]][[j]][[k]][[l]][[breaks_reg_noslope_optinc_dots_coarse[i,j,k,l]]];
				breaks_reg_noslope_opt_dots_sum_coarse[[i]][[j]][[k]][[l]] <- summary(breaks_reg_noslope_opt_dots_coarse[[i]][[j]][[k]][[l]]);

				#Flexible MLE
				##Set MLE function
				ll_logistic_coarse <- function(a,b,c,d) {
					log_likelihood <- 0;
					for(m in 1:num_tasks) {
						log_likelihood <- log_likelihood+log((a/(1+exp(-b*(incentives_coarse[i,m]-c)))+d))*(correct_coarse[i,j,k,l,m])+log((1-a/(1+exp(-b*(incentives_coarse[i,m]-c)))-d))*(1-correct_coarse[i,j,k,l,m]);
					}
					return(-log_likelihood);
				}

				mle_logistic_coarse[[i]][[j]][[k]][[l]] <- mle2(ll_logistic_coarse,method="SANN",start=list(a=0.5,b=1,c=50,d=0.2));

				discts_ll_coarse <- logLik(breaks_reg_noslope_opt_dots_coarse[[i]][[j]][[k]][[l]]);
				cts_ll_coarse <- logLik(mle_logistic_coarse[[i]][[j]][[k]][[l]]);
				ll_stat_coarse <- -2*(discts_ll_coarse-cts_ll_coarse);
				ll_test_pval_coarse[i,j,k,l] <- pchisq(ll_stat_coarse,1,lower.tail=FALSE);
			
			}
		}
	}
}

###Power calculations
poorly_behaved_coarse <- (ll_test_pval_coarse > 0.05)+0; #mark "subjects" for whom discontinuity cannot be rejected

power_discty_coarse <- 1-apply(poorly_behaved_coarse,c(2,3,4),mean); #power array: shows simulated test power for each size/location/slope

dimnames(power_discty_coarse)[[1]] <- break_sizes;
dimnames(power_discty_coarse)[[2]] <- break_locations;
dimnames(power_discty_coarse)[[3]] <- slopes;


