##DOVEH-SHAPIRO-FEIGIN METHOD
##run analysis up to end of preamble code before running this

##make sure nonlinear equation solver (nleqslv) package is there
install.packages("nleqslv");
library(nleqslv);


unres_monotone <- rep(NA,n_subj); #monotone even when unrestricted
out_int <- rep(NA,n_subj); #roots outside of (1,100)
continue1 <- rep(NA,n_subj); #root 1 in interval
continue2 <- rep(NA,n_subj); #root 2 in interval
reject_test <- rep(NA,n_subj); #1 if not monotone

pval_dsf <- rep(1,n_subj);
Tbysigma_dsf <- rep(NA,n_subj);

exclude <- c(3000);
#change the exclude if it fails, remove [-exclude] if nothing to be excluded
#exclude <- c(1,6,7,9,10,13,14,17,19,20,24,25,26,27,29,30); #excluded in first round
#exclude <- c(35); #lab experiments, all
#exclude <- c(42,57); #lab experiments, last 50
exclude <- c(65); #lab experiments, first 50
#exclude <- c(93); #web experiments

for(i in (1:n_subj)[-exclude]) {
	#set data
	x <- inc_dots[i,];
	y <- correct_dots[i,];

	##STEP 1: compute A
	A <- cbind(1,x,x^2,x^3);

	##STEP 2: compute unrestricted regression
	poly_reg <- lm(y ~ x + I(x^2) + I(x^3));
	theta_tilde <- poly_reg$coefficients;

	##STEP 3: compute test statistic
	#function to minimize
	restr_reg <- function(theta) {
		theta1 <- theta[1];
		theta2 <- theta[2];
		theta3 <- theta[3];
		theta4 <- theta[4];
		theta_min <- rbind(theta1,theta2,theta3,theta4);
		return((t(y-A%*%theta_min)%*%(y-A%*%theta_min))[1,1]);
	}

	#gradient function
	restr_reg_grad <- function(theta) {
		theta1 <- theta[1];
		theta2 <- theta[2];
		theta3 <- theta[3];
		theta4 <- theta[4];
		theta_min <- rbind(theta1,theta2,theta3,theta4);
		return(-2*t(A)%*%(y-A%*%theta_min));
	}

	#constraint matrix
	constr_mat <- cbind(0,1,2*x,3*x^2);

	constr_reg <- constrOptim(rep(0.01,4),restr_reg,restr_reg_grad,ui=constr_mat,ci=rep(0,num_tasks));
	T <- constr_reg$value - sum(summary(poly_reg)$residuals^2);

	#STEP 4: Compute an estimate of Omega
	Omega <- 1/length(x)*(t(A)%*%A);


	#STEP 5: Compute an estimate of sigma squared
	sigma2 <- summary(poly_reg)$sigma^2;

	#STEP 6: Find roots of derivative

	#first check discriminant
	if(theta_tilde[3]^2-3*theta_tilde[2]*theta_tilde[4]<0) {
		#mark as monotone everywhere even when unrestricted
		unres_monotone[i] <- 1;
		reject_test[i] <- 0;
		continue1[i] <- 0;
		continue2[i] <- 0;
	} else {
		unres_monotone[i] <- 0;
		
		root1 <- (-theta_tilde[3] - sqrt(theta_tilde[3]^2-3*theta_tilde[2]*theta_tilde[4]))/theta_tilde[4];
		root2 <- (-theta_tilde[3] + sqrt(theta_tilde[3]^2-3*theta_tilde[2]*theta_tilde[4]))/theta_tilde[4];
	
		#check if at least one of the roots falls in the interval
		if(((root1 >= 1)&(root1 <=100))|((root2 >= 1)&(root2 <=100))) {
			out_int[i] <- 0;
			continue1[i] <- 1;
			continue2[i] <- 1;

			if(!((root1 >= 1)&(root1 <=100))) {
				continue1[i] <- 0;
			}
			if(!((root2 >= 1)&(root2 <=100))) {
				continue2[i] <- 0;
			}

		} else {
			#mark roots as being outside interval
			continue1[i] <- 0;
			continue2[i] <- 0;
			out_int[i] <- 1;
		}
	}

	##STEPS 7 TO 10: calculate distribution of test stat
	w_chibar <- c(0); #indices 1,2,3 correspond to 0,1,2 in paper
	#continue on only if roots exist and fall in interval
	if(continue1[i]==1|continue2[i]==1) {
		##STEP 7: Compute B
		b1 <- c(0,1,2*root1,3*root1^2);
		b2 <- c(0,1,2*root2,3*root2^2);
	
		if(continue1[i]==1|continue2[i]!=1) {
			B <- b1;
		} else if(continue1[i]!=1|continue2[i]==1) {
			B <- b2;
		} else {
			B <- cbind(b1,b2);
		}

		##STEP 8: Compute weights for chi-bar squared distribution
		cov_matrix <- t(B) %*% solve(Omega) %*% B;

		if(continue1[i]*continue2[i]==1) {
			w_chibar[1] <- (pi-acos(cov_matrix[1,2]))/(2*pi);
			w_chibar[2] <- 1/2;
			w_chibar[3] <- 1/2-weight[1];
		} else {
			w_chibar[1] <- 1/2;
			w_chibar[2] <- 1/2;
		}

		#STEP 9: Find critical values
		#set significance level
		sig_lev <- 0.05;
		chibarsq <- function(c) {
			cbsq <- -sig_lev;
			for(j in 2:length(w_chibar)) {
				cbsq <- cbsq + w_chibar[j]*pchisq(c,j,lower.tail=FALSE);
			}
			return(cbsq);
			#sum(w_chibar*pchisq(c,(2:length(w_chibar)),lower.tail=FALSE))-sig_lev;
		}
		crit_soln <- nleqslv(c(2),chibarsq);
		crit_val <- crit_soln$x;
		pval_dsf[i] <- chibarsq(T/sigma2)+sig_lev;
		Tbysigma_dsf[i] <- T/sigma2;

		#STEP 10: Compare test statistic
		if(T >= sigma2*crit_val) {
			reject_test[i] <- 1;
		} else {
			reject_test[i] <- 0;
		}
	} else {
		reject_test[i] <- 0;
	}
}

mon_subj <- which(reject_test %in% c(0,NA));


###histogram of p-values
hist(pval_dsf, breaks=(0:20)/20, main="Histogram of p-values for Monotonicity Test", xlab="p-value", ylim=c(0,40));
