#mTurk analysis

#list of required packages:
#no need to run install after first time
##stringr [for string-related functions, esp. substrings]
##strucchange [for structural change]
##bbmle [for better mle]
##dgof [for discrete ks tests]
##stargazer [for latex output]
##nnet [for multinomial logit]
##nleqslv [for implicit functions]

install.packages("stringr");
install.packages("strucchange");
install.packages("bbmle");
install.packages("dgof");
install.packages("stargazer");
install.packages("nnet");
install.packages("nleqslv");

library(stringr);
library(strucchange);
library(bbmle);
library(dgof);
library(stargazer);
library(nnet);
library(nleqslv);



##INCENTIVE THREE
#read and import data
data_all_3 <- read.delim("mturk_3dots.txt"); #change file name as needed
data_3 <- data_all_3[-1,];

#read dots data
true_dots1_3 <- data_3["trueDots1"];
inc_dots1_3 <- data_3["incDots1"];
chose_dots1_3 <- data_3["choseDots1"];
true_dots2_3 <- data_3["trueDots2"];
inc_dots2_3 <- data_3["incDots2"];
chose_dots2_3 <- data_3["choseDots2"];

#put in list form
true_dots1_list_3 <- apply(true_dots1_3,1,strsplit,split=".",fixed=TRUE);
true_dots2_list_3 <- apply(true_dots2_3,1,strsplit,split=".",fixed=TRUE);
chose_dots1_list_3 <- apply(chose_dots1_3,1,strsplit,split=".",fixed=TRUE);
chose_dots2_list_3 <- apply(chose_dots2_3,1,strsplit,split=".",fixed=TRUE);
inc_dots1_list_3 <- apply(inc_dots1_3,1,strsplit,split=".",fixed=TRUE);
inc_dots2_list_3 <- apply(inc_dots2_3,1,strsplit,split=".",fixed=TRUE);

#write to numeric list
true_dots1_num_3 <- vector("list",nrow(data_3));
true_dots2_num_3 <- vector("list",nrow(data_3));
chose_dots1_num_3 <- vector("list",nrow(data_3));
chose_dots2_num_3 <- vector("list",nrow(data_3));
inc_dots1_num_3 <- vector("list",nrow(data_3));
inc_dots2_num_3 <- vector("list",nrow(data_3));
for(i in 1:nrow(data_3)) {
	#convert to numbers and remove first extra NA
	true_dots1_num_3[[i]] <- as.numeric(true_dots1_list_3[[i]][[1]][-1]);
	true_dots2_num_3[[i]] <- as.numeric(true_dots2_list_3[[i]][[1]][-1]);
	chose_dots1_num_3[[i]] <- as.numeric(chose_dots1_list_3[[i]][[1]][-1]);
	chose_dots2_num_3[[i]] <- as.numeric(chose_dots2_list_3[[i]][[1]][-1]);
	inc_dots1_num_3[[i]] <- as.numeric(inc_dots1_list_3[[i]][[1]][-1]);
	inc_dots2_num_3[[i]] <- as.numeric(inc_dots2_list_3[[i]][[1]][-1]);
}

#get lengths of recorded data
true_dots1_lengths_3 <- unlist(lapply(true_dots1_num_3,length));
true_dots2_lengths_3 <- unlist(lapply(true_dots2_num_3,length));
chose_dots1_lengths_3 <- unlist(lapply(chose_dots1_num_3,length));
chose_dots2_lengths_3 <- unlist(lapply(chose_dots2_num_3,length));
inc_dots1_lengths_3 <- unlist(lapply(inc_dots1_num_3,length));
inc_dots2_lengths_3 <- unlist(lapply(inc_dots2_num_3,length));

#get qualtrics-recorded chosen values to check
chose_dots1_mat_3 <- data_3[which(str_sub(names(data_3),-4) == "QID1")];
chose_dots2_mat_3 <- data_3[which(str_sub(names(data_3),-5) == "QID56")];
chose_dots_mat_3 <- cbind(chose_dots1_mat_3,chose_dots2_mat_3);

#make data frames with incentive and choice data in order and compare
inc_choice1_3 <- vector("list",nrow(data_3));
inc_choice2_3 <- vector("list",nrow(data_3));
compare_choice1_3 <- vector("list",nrow(data_3));
compare_choice2_3 <- vector("list",nrow(data_3));
mismatch1_3 <- mat.or.vec(nrow(data_3),1);
mismatch2_3 <- mat.or.vec(nrow(data_3),1);
empty1_3 <- mat.or.vec(nrow(data_3),1);
empty2_3 <- mat.or.vec(nrow(data_3),1);
for(i in 1:nrow(data_3)) {
	#check to see chosen dots is non-empty, and if it is, replace with zero
	if(length(chose_dots1_num_3[[i]])==0) {
		chose_dots1_num_3[[i]] <- 0;
		empty1_3[i] <- 1;
	}
	if(length(chose_dots2_num_3[[i]])==0) {
		chose_dots2_num_3[[i]] <- 0;
		empty2_3[i] <- 1;
	}

	#put data in frames
	temp_frame1_3 <- data.frame(inc_dots1_num_3[[i]][1:chose_dots1_lengths_3[i]],chose_dots1_num_3[[i]]);
	temp_frame2_3 <- data.frame(inc_dots2_num_3[[i]][1:chose_dots2_lengths_3[i]],chose_dots2_num_3[[i]]);
	colnames(temp_frame1_3) <- c("inc","chose");
	colnames(temp_frame2_3) <- c("inc","chose");

	#reorder by incentive
	attach(temp_frame1_3);
	inc_choice1_3[[i]] <- temp_frame1_3[order(inc),];
	detach(temp_frame1_3);

	attach(temp_frame2_3);
	inc_choice2_3[[i]] <- temp_frame2_3[order(inc),];
	detach(temp_frame2_3);
	
	#make comparisons
	compare_choice1_3[[i]] <- (unlist(inc_choice1_3[[i]]["chose"])==chose_dots1_mat_3[i,unlist(inc_choice1_3[[i]]["inc"])])+0;
	compare_choice2_3[[i]] <- (unlist(inc_choice2_3[[i]]["chose"])==chose_dots2_mat_3[i,unlist(inc_choice2_3[[i]]["inc"])])+0;
	
	#find mismatches and empty ones
	mismatch1_3[i] <- ((0 %in% compare_choice1_3[[i]])|(empty1_3[i]==1))+0;
	mismatch2_3[i] <- ((0 %in% compare_choice2_3[[i]])|(empty2_3[i]==1))+0;
}

#find which subjects have wrong lengths
wrong_num_3 <- 1-(true_dots1_lengths_3 == 100)*(true_dots2_lengths_3 == 100)*(inc_dots1_lengths_3 == 100)*(inc_dots2_lengths_3 == 100);
low_num_3 <- 1-(true_dots1_lengths_3 >= 100)*(true_dots2_lengths_3 >= 100)*(inc_dots1_lengths_3 >= 100)*(inc_dots2_lengths_3 >= 100);

#find which subjects have any mismatch whatsoever
mismatch_3 <- 1-(1-mismatch1_3)*(1-mismatch2_3);

#find subjects that should be kept
keep_3 <- ((wrong_num_3==0)&(mismatch_3==0))+0;
subjects_3 <- which(keep_3==1);


##INCENTIVE FIVE
#read and import data
data_all_5 <- read.delim("mturk_5dots.txt"); #change file name as needed
data_5 <- data_all_5[-1,];

#read dots data
true_dots1_5 <- data_5["trueDots1"];
inc_dots1_5 <- data_5["incDots1"];
chose_dots1_5 <- data_5["choseDots1"];
true_dots2_5 <- data_5["trueDots2"];
inc_dots2_5 <- data_5["incDots2"];
chose_dots2_5 <- data_5["choseDots2"];

#put in list form
true_dots1_list_5 <- apply(true_dots1_5,1,strsplit,split=".",fixed=TRUE);
true_dots2_list_5 <- apply(true_dots2_5,1,strsplit,split=".",fixed=TRUE);
chose_dots1_list_5 <- apply(chose_dots1_5,1,strsplit,split=".",fixed=TRUE);
chose_dots2_list_5 <- apply(chose_dots2_5,1,strsplit,split=".",fixed=TRUE);
inc_dots1_list_5 <- apply(inc_dots1_5,1,strsplit,split=".",fixed=TRUE);
inc_dots2_list_5 <- apply(inc_dots2_5,1,strsplit,split=".",fixed=TRUE);

#write to numeric list
true_dots1_num_5 <- vector("list",nrow(data_5));
true_dots2_num_5 <- vector("list",nrow(data_5));
chose_dots1_num_5 <- vector("list",nrow(data_5));
chose_dots2_num_5 <- vector("list",nrow(data_5));
inc_dots1_num_5 <- vector("list",nrow(data_5));
inc_dots2_num_5 <- vector("list",nrow(data_5));
for(i in 1:nrow(data_5)) {
	#convert to numbers and remove first extra NA
	true_dots1_num_5[[i]] <- as.numeric(true_dots1_list_5[[i]][[1]][-1]);
	true_dots2_num_5[[i]] <- as.numeric(true_dots2_list_5[[i]][[1]][-1]);
	chose_dots1_num_5[[i]] <- as.numeric(chose_dots1_list_5[[i]][[1]][-1]);
	chose_dots2_num_5[[i]] <- as.numeric(chose_dots2_list_5[[i]][[1]][-1]);
	inc_dots1_num_5[[i]] <- as.numeric(inc_dots1_list_5[[i]][[1]][-1]);
	inc_dots2_num_5[[i]] <- as.numeric(inc_dots2_list_5[[i]][[1]][-1]);
}

#get lengths of recorded data
true_dots1_lengths_5 <- unlist(lapply(true_dots1_num_5,length));
true_dots2_lengths_5 <- unlist(lapply(true_dots2_num_5,length));
chose_dots1_lengths_5 <- unlist(lapply(chose_dots1_num_5,length));
chose_dots2_lengths_5 <- unlist(lapply(chose_dots2_num_5,length));
inc_dots1_lengths_5 <- unlist(lapply(inc_dots1_num_5,length));
inc_dots2_lengths_5 <- unlist(lapply(inc_dots2_num_5,length));

#get qualtrics-recorded chosen values to check
chose_dots1_mat_5 <- data_5[which(str_sub(names(data_5),-4) == "QID1")];
chose_dots2_mat_5 <- data_5[which(str_sub(names(data_5),-5) == "QID56")];
chose_dots_mat_5 <- cbind(chose_dots1_mat_5,chose_dots2_mat_5);

#make data frames with incentive and choice data in order and compare
inc_choice1_5 <- vector("list",nrow(data_5));
inc_choice2_5 <- vector("list",nrow(data_5));
compare_choice1_5 <- vector("list",nrow(data_5));
compare_choice2_5 <- vector("list",nrow(data_5));
mismatch1_5 <- mat.or.vec(nrow(data_5),1);
mismatch2_5 <- mat.or.vec(nrow(data_5),1);
empty1_5 <- mat.or.vec(nrow(data_5),1);
empty2_5 <- mat.or.vec(nrow(data_5),1);
for(i in 1:nrow(data_5)) {
	#check to see chosen dots is non-empty, and if it is, replace with zero
	if(length(chose_dots1_num_5[[i]])==0) {
		chose_dots1_num_5[[i]] <- 0;
		empty1_5[i] <- 1;
	}
	if(length(chose_dots2_num_5[[i]])==0) {
		chose_dots2_num_5[[i]] <- 0;
		empty2_5[i] <- 1;
	}

	#put data in frames
	temp_frame1_5 <- data.frame(inc_dots1_num_5[[i]][1:chose_dots1_lengths_5[i]],chose_dots1_num_5[[i]]);
	temp_frame2_5 <- data.frame(inc_dots2_num_5[[i]][1:chose_dots2_lengths_5[i]],chose_dots2_num_5[[i]]);
	colnames(temp_frame1_5) <- c("inc","chose");
	colnames(temp_frame2_5) <- c("inc","chose");

	#reorder by incentive
	attach(temp_frame1_5);
	inc_choice1_5[[i]] <- temp_frame1_5[order(inc),];
	detach(temp_frame1_5);

	attach(temp_frame2_5);
	inc_choice2_5[[i]] <- temp_frame2_5[order(inc),];
	detach(temp_frame2_5);
	
	#make comparisons
	compare_choice1_5[[i]] <- (unlist(inc_choice1_5[[i]]["chose"])==chose_dots1_mat_5[i,unlist(inc_choice1_5[[i]]["inc"])])+0;
	compare_choice2_5[[i]] <- (unlist(inc_choice2_5[[i]]["chose"])==chose_dots2_mat_5[i,unlist(inc_choice2_5[[i]]["inc"])])+0;
	
	#find mismatches and empty ones
	mismatch1_5[i] <- ((0 %in% compare_choice1_5[[i]])|(empty1_5[i]==1))+0;
	mismatch2_5[i] <- ((0 %in% compare_choice2_5[[i]])|(empty2_5[i]==1))+0;
}

#find which subjects have wrong lengths
wrong_num_5 <- 1-(true_dots1_lengths_5 == 100)*(true_dots2_lengths_5 == 100)*(inc_dots1_lengths_5 == 100)*(inc_dots2_lengths_5 == 100);
low_num_5 <- 1-(true_dots1_lengths_5 >= 100)*(true_dots2_lengths_5 >= 100)*(inc_dots1_lengths_5 >= 100)*(inc_dots2_lengths_5 >= 100);

#find which subjects have any mismatch whatsoever
mismatch_5 <- 1-(1-mismatch1_5)*(1-mismatch2_5);

#find subjects that should be kept
keep_5 <- ((wrong_num_5==0)&(mismatch_5==0))+0;
subjects_5 <- which(keep_5==1);


#########
#########

#NOW, EXTRACT RELEVANT DATA
#set matrices that hold data
inc_dots <- matrix(0,nrow=length(subjects_3)+length(subjects_5),ncol=200);
true_dots <- matrix(0,nrow=length(subjects_3)+length(subjects_5),ncol=200);
chose_dots <- matrix(0,nrow=length(subjects_3)+length(subjects_5),ncol=200);

for(i in 1:length(subjects_3)) {
	inc_dots[i,1:100] <- inc_dots1_num_3[[subjects_3[i]]];
	inc_dots[i,101:200] <- inc_dots2_num_3[[subjects_3[i]]];
	true_dots[i,1:100] <- true_dots1_num_3[[subjects_3[i]]];
	true_dots[i,101:200] <- true_dots2_num_3[[subjects_3[i]]];
	chose_dots[i,1:100] <- chose_dots1_num_3[[subjects_3[i]]];
	chose_dots[i,101:200] <- chose_dots2_num_3[[subjects_3[i]]];
}

for(i in 1:length(subjects_5)) {
	inc_dots[i+length(subjects_3),1:100] <- inc_dots1_num_5[[subjects_5[i]]];
	inc_dots[i+length(subjects_3),101:200] <- inc_dots2_num_5[[subjects_5[i]]];
	true_dots[i+length(subjects_3),1:100] <- true_dots1_num_5[[subjects_5[i]]];
	true_dots[i+length(subjects_3),101:200] <- true_dots2_num_5[[subjects_5[i]]];
	chose_dots[i+length(subjects_3),1:100] <- chose_dots1_num_5[[subjects_5[i]]];
	chose_dots[i+length(subjects_3),101:200] <- chose_dots2_num_5[[subjects_5[i]]];
}

#correct or not
correct_dots <- (true_dots==chose_dots)+0;

#distance
dist_dots <- chose_dots-true_dots;

#indicator for if people have higher incentive
is5 <- c(rep(0,length(subjects_3)),rep(1,length(subjects_5)));

#number of subjects
n_subj <- length(subjects_3)+length(subjects_5);

#number of tasks
num_tasks <- 200;

#set lpm weighting function
#0.001 and 0.999 are ad hoc choices
lpm_weights <- function(x) {
	if(x <= 0) {
		fit <- 0.001;
	} else if (x >= 1) {
		fit <- 0.999;
	} else {
		fit <- x;
	}
	return(sqrt(fit*(1-fit)));
}


########
##DEMOGRAPHICS

##AGE
age_3_all <- as.numeric(as.vector(unlist(data_3["QID9_TEXT"])));
age_5_all <- as.numeric(as.vector(unlist(data_5["QID9_TEXT"])));
age <- c(age_3_all[subjects_3],age_5_all[subjects_5]);

##GENDER
#coded as 1 for female, 0 for male
#no one reported non-binary/other, so this is fine
gender_3_all <- as.vector(unlist(data_3["QID39"]));
gender_5_all <- as.vector(unlist(data_5["QID39"]));
gender_text <- c(gender_3_all[subjects_3],gender_5_all[subjects_5]);
gender <- (gender_text=="Female")+0;

##LEVEL OF EDUCATION
lvl_edu_3_all <- as.vector(unlist(data_3["QID40"]));
lvl_edu_5_all <- as.vector(unlist(data_5["QID40"]));
lvl_edu <- c(lvl_edu_3_all[subjects_3],lvl_edu_5_all[subjects_5]);


########
########
#END OF PREAMBLE CODE
#RUN UP TO HERE BEFORE RUNNING OTHER SCRIPTS
########
########


########
########
#MODEL FITS

######
#Constant regressions (means with associated likelihoods)
mean_reg_dots <- as.list(mat.or.vec(n_subj,1));
mean_reg_sum_dots <- as.list(mat.or.vec(n_subj,1));
for(i in 1:n_subj) {
	mean_reg_dots[[i]] <- lm(correct_dots[i,] ~ 1);
	mean_reg_sum_dots[[i]] <- summary(mean_reg_dots[[i]]);
}

######
#LPMs
#standard errors with and without heteroskedasticity correction
mean_dots <- mat.or.vec(n_subj,1);
reg_dots <- as.list(mat.or.vec(n_subj,1)); #no het correction
reg_sum_dots <- as.list(mat.or.vec(n_subj,1));
lpm_weights_dots <-  mat.or.vec(n_subj,num_tasks);
lpm_reg_dots <- as.list(mat.or.vec(n_subj,1)); #het correction
lpm_reg_sum_dots <- as.list(mat.or.vec(n_subj,1));
for(i in 1:n_subj) {
	#x11();
	#plot(correct_dots[i,] ~ inc_dots[i,], main=paste("Dots: subject ",i),xlab="Incentive Level",ylab="Correct",pch=20);
	mean_dots[i] <- mean(correct_dots[i,]);
	reg_dots[[i]] <- lm(correct_dots[i,] ~ inc_dots[i,]);
	reg_sum_dots[[i]] <- summary(reg_dots[[i]]);
	lpm_weights_dots[i,] <- apply(as.matrix(fitted(reg_dots[[i]])),1,lpm_weights);
	lpm_reg_dots[[i]] <- lm(correct_dots[i,] ~ inc_dots[i,], weights=lpm_weights_dots[i,]);
	lpm_reg_sum_dots[[i]] <- summary(lpm_reg_dots[[i]]);
}

######
#POLYNOMIAL LPMs (2nd and 3rd)
#standard errors with and without heteroskedasticity correction
reg2_dots <- as.list(mat.or.vec(n_subj,1));
lpm2_weights_dots <-  mat.or.vec(n_subj,num_tasks);
lpm2_reg_dots <- as.list(mat.or.vec(n_subj,1)); #het correction
lpm2_reg_sum_dots <- as.list(mat.or.vec(n_subj,1));
reg3_dots <- as.list(mat.or.vec(n_subj,1));
lpm3_weights_dots <-  mat.or.vec(n_subj,num_tasks);
lpm3_reg_dots <- as.list(mat.or.vec(n_subj,1)); #het correction
lpm3_reg_sum_dots <- as.list(mat.or.vec(n_subj,1));
for(i in 1:n_subj) {
	reg2_dots[[i]] <- lm(correct_dots[i,] ~ inc_dots[i,] + I(inc_dots[i,]^2));
	lpm2_weights_dots[i,] <- apply(as.matrix(fitted(reg2_dots[[i]])),1,lpm_weights);
	lpm2_reg_dots[[i]] <- lm(correct_dots[i,] ~ inc_dots[i,] + I(inc_dots[i,]^2), weights=lpm2_weights_dots[i,]);
	lpm2_reg_sum_dots[[i]] <- summary(lpm2_reg_dots[[i]]);

	reg3_dots[[i]] <- lm(correct_dots[i,] ~ inc_dots[i,] + I(inc_dots[i,]^2) + I(inc_dots[i,]^3));
	lpm3_weights_dots[i,] <- apply(as.matrix(fitted(reg3_dots[[i]])),1,lpm_weights);
	lpm3_reg_dots[[i]] <- lm(correct_dots[i,] ~ inc_dots[i,] + I(inc_dots[i,]^2) + I(inc_dots[i,]^3), weights=lpm3_weights_dots[i,]);
	lpm3_reg_sum_dots[[i]] <- summary(lpm3_reg_dots[[i]]);
}

######
#BREAKPOINT ANALYSIS (allowing for slope)
breaks_dots <- as.list(mat.or.vec(n_subj,1)); #has breakpoints in the list element $breakpoints
exceeds_break_dots <- matrix(nrow=n_subj,ncol=num_tasks); #does incentive exceed break
exceeds_break_inc_dots <- matrix(nrow=n_subj,ncol=num_tasks); #incentive if exceeds break, else 0
breaks_reg_dots <- as.list(mat.or.vec(n_subj,1)); #linear regression results with breaks, if they exist
breaks_reg_dots_sum <- as.list(mat.or.vec(n_subj,1)); #linear regression summary with breaks, if they exist
for(i in 1:n_subj) {
	breaks_dots[[i]] <- breakpoints(correct_dots[i,] ~ inc_dots[i,],breaks=1);
	if(is.numeric(breaks_dots[[i]]$breakpoints)) {
		exceeds_break_dots[i,] <- (inc_dots[i,] >= breaks_dots[[i]]$breakpoints)+0;
		exceeds_break_inc_dots[i,] <- exceeds_break_dots[i,]*inc_dots[i,];
		breaks_reg_dots[[i]] <- lm(correct_dots[i,] ~ inc_dots[i,] + exceeds_break_dots[i,] + exceeds_break_inc_dots[i,]);
	}
	else {
		breaks_reg_dots[[i]] <- lm(correct_dots[i,] ~ inc_dots[i,]);
	}
	breaks_reg_dots_sum[[i]] <- summary(breaks_reg_dots[[i]]);
}

######
#BREAKPOINT ANALYSIS (no slope)
inc_dummies_dots <- array(dim=c(n_subj,num_tasks,num_tasks)); #creates vectors of dummies for whether or not the incentive level is at least the dummy level
breaks_reg_noslope_dots <- as.list(mat.or.vec(n_subj,1)); #stores regression results for each dummy for each person
breaks_reg_noslope_r2_dots <- matrix(nrow=n_subj,ncol=num_tasks); #store r squareds for each regression
breaks_reg_noslope_optinc_dots <- vector(length=n_subj); #stores optimal incentive level for break
breaks_reg_noslope_opt_dots <- as.list(mat.or.vec(n_subj,1)); #stores regression results with the highest r2 for each person
breaks_reg_noslope_opt_dots_sum <- as.list(mat.or.vec(n_subj,1)); #stores regression summary with the highest r2 for each person
for(i in 1:n_subj) {
	breaks_reg_noslope_dots[[i]] <- as.list(mat.or.vec(num_tasks,1));
	for(j in 1:num_tasks) {
		#populate inc_dummies_dots for subject i
		inc_dummies_dots[i,,j] <- (inc_dots[i,] >= j)+0;
		#run regressions and store r squareds
		breaks_reg_noslope_dots[[i]][[j]] <- lm(correct_dots[i,] ~ inc_dummies_dots[i,,j]);
		breaks_reg_noslope_r2_dots[i,j] <- summary(breaks_reg_noslope_dots[[i]][[j]])$r.squared;
	}
	#find optimal r squared
	breaks_reg_noslope_optinc_dots[i] <- which.max(breaks_reg_noslope_r2_dots[i,]);
	#store this regression
	breaks_reg_noslope_opt_dots[[i]] <- breaks_reg_noslope_dots[[i]][[breaks_reg_noslope_optinc_dots[i]]];
	breaks_reg_noslope_opt_dots_sum[[i]] <- summary(breaks_reg_noslope_opt_dots[[i]])
}

######
#BREAKPOINT ANALYSIS (constant slope)
inc_dummies_dots <- array(dim=c(n_subj,num_tasks,num_tasks)); #creates vectors of dummies for whether or not the incentive level is at least the dummy level
breaks_reg_cslope_dots <- as.list(mat.or.vec(n_subj,1)); #stores regression results for each dummy for each person
breaks_reg_cslope_r2_dots <- matrix(nrow=n_subj,ncol=num_tasks); #store r squareds for each regression
breaks_reg_cslope_optinc_dots <- vector(length=n_subj); #stores optimal incentive level for break
breaks_reg_cslope_opt_dots <- as.list(mat.or.vec(n_subj,1)); #stores regression results with the highest r2 for each person
breaks_reg_cslope_opt_dots_sum <- as.list(mat.or.vec(n_subj,1)); #stores regression summary with the highest r2 for each person
for(i in 1:n_subj) {
	breaks_reg_cslope_dots[[i]] <- as.list(mat.or.vec(num_tasks,1));
	for(j in 1:num_tasks) {
		#populate inc_dummies_dots for subject i
		inc_dummies_dots[i,,j] <- (inc_dots[i,] >= j)+0;
		#run regressions and store r squareds
		breaks_reg_cslope_dots[[i]][[j]] <- lm(correct_dots[i,] ~ inc_dots[i,] + inc_dummies_dots[i,,j]);
		breaks_reg_cslope_r2_dots[i,j] <- summary(breaks_reg_cslope_dots[[i]][[j]])$r.squared;
	}
	#find optimal r squared
	breaks_reg_cslope_optinc_dots[i] <- which.max(breaks_reg_cslope_r2_dots[i,]);
	#store this regression
	breaks_reg_cslope_opt_dots[[i]] <- breaks_reg_cslope_dots[[i]][[breaks_reg_cslope_optinc_dots[i]]];
	breaks_reg_cslope_opt_dots_sum[[i]] <- summary(breaks_reg_cslope_opt_dots[[i]])
}


######
#MLE (Shannon)
#starting parameter is set arbitrarily
mle_dots <- as.list(mat.or.vec(n_subj,1));
for(i in 1:n_subj) {
	ll_shannon_dots <- function(a) {
		-sum(
			correct_dots[i,]*(
				inc_dots[i,]/a
				-log(
					4+exp(inc_dots[i,]/a)
				)
			)+(1-correct_dots[i,])*(
				log(4)
				-log(
					4+exp(inc_dots[i,]/a)
				)
			)
		);
	}
	mle_dots[[i]] <- mle2(ll_shannon_dots,method="L-BFGS-B",lower=c(a=0),start=c(a=5));
}


######
#TSALLIS
######
######
tsallis_dots <- as.list(mat.or.vec(n_subj,1));
tolerance_perf <- 0.00001; #sets distance performance can be from 0 or 1
tolerance_sigma <- 0.00001; #ensures there's no "blow-up" in the vicinity of sigma=1
#set sigma initialization values according to excel doc
sigma_init <- rep(1,n_subj);
sigma_init[c(34,45,54,79)] <- 0.1;
sigma_init[c(68,69)] <- 0.001;

#run only on responsive subjects to save on computation time
for (i in c(68,69)) {
	ll_tsal_dots <- function(alpha, sigma) {
		like <- 0; #negative log likelihood

		for(j in 1:num_tasks) {
			if(abs(sigma-1)<tolerance_sigma) {
				perf_soln <- exp(inc_dots[i,j]/alpha)/(4+exp(inc_dots[i,j]/alpha));
			} else {
			
				#solve for performance
				solve_perf <- function(perf) {
					return(
						alpha*sigma/(sigma-1)*(((1-perf)/4)^(sigma-1)-perf^(sigma-1))+inc_dots[i,j]
					);
				}
				
				perf_soln <- try(as.numeric(uniroot(solve_perf,c(0.2,1),extendInt="downX")$root));
				#Looking at the graph, the only way for rootfinding to fail is if the root is greater than 1 (since "roots" smaller than 0.2 will never be chosen). If performance larger than 1 is computed, then this resets it to just below 1 so that a likelihood can actually be computed
				if("try-error" %in% class(perf_soln)) {
					perf_soln <- 1-tolerance_perf; 
				}
			}
			
			like <- like - correct_dots[i,j]*log(perf_soln) - (1-correct_dots[i,j])*log(1-perf_soln);
		}
			
		return(like);		
	}

	#mle step
	try(tsallis_dots[[i]] <- mle2(ll_tsal_dots,start=list(alpha=20,sigma=sigma_init[i]),lower=list(alpha=0,sigma=0),upper=list(alpha=Inf,sigma=Inf),method="L-BFGS-B"));
	#debug, comment out
	print(i);
}


######
#NORMAL SIGNALS (assuming costs linear in precision w/ parameter a)
######
######
#dots
normsig_dots <- as.list(mat.or.vec(n_subj,1));

for (i in 1:n_subj) {
	ll_norm_dots <- function(a) {
		like <- 0; #negative log likelihood

		for(j in 1:num_tasks) {
			#solve for sqrt of precision
			solve_sqrt_prec <- function(sqrt_prec) {
				return(a*sqrt_prec - 2/5*dnorm(0.5*sqrt_prec)*inc_dots[i,j]);
			}
			sqrt_prec_soln <- nleqslv(2,solve_sqrt_prec)$x;

			#if true state is 38 or 42, then there's one likelihood (because on edges)
			#if true state is 39, 40, or 41, then there's another likelihood (because in interior)

			if(true_dots[i,j] %in% c(38,42)) {
				if(correct_dots[i,j]==1) {
					like <- like - log(pnorm(0.5,sd=1/sqrt_prec_soln));
				} else {
					like <- like - log(pnorm(0.5,sd=1/sqrt_prec_soln,lower.tail=FALSE));
				}
			} else {
				if(correct_dots[i,j]==1) {
					like <- like - log(pnorm(0.5,sd=1/sqrt_prec_soln)-pnorm(-0.5,sd=1/sqrt_prec_soln));
				} else {
					like <- like - log(2*pnorm(-0.5,sd=1/sqrt_prec_soln));
				}
			}
		}
		return(like);		
	}

	#mle step
	normsig_dots[[i]] <- mle2(ll_norm_dots,start=list(a=5),lower=(a=0),method="L-BFGS-B");
}



######
######
######
#This section requires the running of previous sections as listed below.

#1. Constant performance
#2. Binary performance
#3. Linear model
#4. Linear with break
#5. Polynomial (deg 2)
#6. Polynomial (deg 3)
#7. Shannon MLE
#8. Tsallis entropy costs
#9. Normal Signals, cost linear in precision

num_models <- 9;

#"noprobit" variables are actually no normal

#dots
aic_matrix_dots <- mat.or.vec(n_subj,num_models);
best_model_dots <- mat.or.vec(n_subj,1);
best_model_nopoly_dots <- mat.or.vec(n_subj,1);
best_model_noprobit_dots <- mat.or.vec(n_subj,1);
rank_models_noprobit_dots <- mat.or.vec(n_subj,num_models-1);
rank_models_dots <- mat.or.vec(n_subj,num_models);

#add bic calculations
bic_matrix_dots <- mat.or.vec(n_subj,num_models);
best_model_bic_dots <- mat.or.vec(n_subj,1);
rank_models_bic_dots <- mat.or.vec(n_subj,num_models);

#add AICc calculations
aicc_matrix_dots <- mat.or.vec(n_subj,num_models);
best_model_aicc_dots <- mat.or.vec(n_subj,1);
rank_models_aicc_dots <- mat.or.vec(n_subj,num_models);

aicc_correction <- function(k,n) {
	return(2*k*(k+1)/(n-k-1));
}

aicc_penalty <- c(aicc_correction(1,num_tasks),aicc_correction(3,num_tasks),aicc_correction(2,num_tasks),aicc_correction(5,num_tasks),aicc_correction(3,num_tasks),aicc_correction(4,num_tasks),aicc_correction(1,num_tasks),aicc_correction(1,num_tasks),aicc_correction(2,num_tasks)); #calculates AICc penalty for the models we use


for(i in str_mon_subj) {
	aic_matrix_dots[i,1] <- AIC(mean_reg_dots[[i]]);
	aic_matrix_dots[i,2] <- AIC(breaks_reg_noslope_opt_dots[[i]])+2; #need extra penalty for estimating break location
	aic_matrix_dots[i,3] <- AIC(reg_dots[[i]]);
	aic_matrix_dots[i,4] <- AIC(breaks_reg_dots[[i]])+2; #need extra penalty for estimating break location
	aic_matrix_dots[i,5] <- AIC(lpm2_reg_dots[[i]]);
	aic_matrix_dots[i,6] <- AIC(lpm3_reg_dots[[i]]);
	aic_matrix_dots[i,7] <- AIC(mle_dots[[i]]);
	aic_matrix_dots[i,8] <- AIC(tsallis_dots[[i]]);
	aic_matrix_dots[i,9] <- AIC(normsig_dots[[i]]);
	
	best_model_dots[i] <- which.min(aic_matrix_dots[i,]);
	bnp <- which.min(aic_matrix_dots[i,c(1:4,7:9)]);
	best_model_nopoly_dots[i] <- bnp + 2*round(bnp/9); #turns 5s and 6s into 7s and 8s
	
	best_model_noprobit_dots[i] <- which.min(aic_matrix_dots[i,1:8]);
	rank_models_noprobit_dots[i,] <- rank(aic_matrix_dots[i,1:8]);
	rank_models_dots[i,] <- rank(aic_matrix_dots[i,]);
	
	bic_matrix_dots[i,1] <- BIC(mean_reg_dots[[i]]);
	bic_matrix_dots[i,2] <- BIC(breaks_reg_noslope_opt_dots[[i]])+log(num_tasks); #need extra penalty for estimating break location
	bic_matrix_dots[i,3] <- BIC(reg_dots[[i]]);
	bic_matrix_dots[i,4] <- BIC(breaks_reg_dots[[i]])+log(num_tasks); #need extra penalty for estimating break location
	bic_matrix_dots[i,5] <- BIC(lpm2_reg_dots[[i]]);
	bic_matrix_dots[i,6] <- BIC(lpm3_reg_dots[[i]]);
	bic_matrix_dots[i,7] <- 2*mle_dots[[i]]@min+log(num_tasks);
	bic_matrix_dots[i,8] <- 2*normsig_dots[[i]]@min+log(num_tasks);
	bic_matrix_dots[i,9] <- 2*tsallis_dots[[i]]@min + 2*log(num_tasks);

	best_model_bic_dots[i] <- which.min(bic_matrix_dots[i,]);
	rank_models_bic_dots[i,] <- rank(bic_matrix_dots[i,]);
	
	aicc_matrix_dots[i,] <- aic_matrix_dots[i,]+aicc_penalty;
	best_model_aicc_dots[i] <- which.min(aicc_matrix_dots[i,]);
	rank_models_aicc_dots[i,] <- rank(aicc_matrix_dots[i,]);
}




###############
###############
#WHAT FOLLOWS IS DEPRECATED
###############
###############

######
#Graph constant, binary, Shannon, probit (because these are the only ones that minimized AIC)
#Highlight the best fit by AIC

for(i in 1:n_subj) {
	color_cons <- "black";
	color_bin <- "black";
	color_shan <- "black";
	color_prob <- "black";

	if(best_model_noprobit_dots[i] == 1) {
		color_cons <- "red";
		best_model_text_dots <- "Constant";
	} else if(best_model_noprobit_dots[i] == 2) {
		color_bin <- "red";
		best_model_text_dots <- "Binary";
	} else if(best_model_noprobit_dots[i] == 7) {
		color_shan <- "red";
		best_model_text_dots <- "Shannon";
	} 

	#x11(); #comment out not to have individual windows
	plot(correct_dots[i,] ~ inc_dots[i,], main=paste("Dots: subject ",i," --- ",best_model_text_dots," in red"),xlab="Incentive Level",ylab="Correct",pch=20);


	#constant
	#curve(0*x+mean_reg_dots[[i]]$coefficients[1],1,100,add=TRUE,col=color_cons);

	#binary
	#lower line
	curve(0*x+breaks_reg_noslope_opt_dots[[i]]$coefficients[[1]],1,breaks_reg_noslope_optinc_dots[i],add=TRUE,col=color_bin);
	#upper line
	curve(0*x+breaks_reg_noslope_opt_dots[[i]]$coefficients[[1]]+breaks_reg_noslope_opt_dots[[i]]$coefficients[[2]],breaks_reg_noslope_optinc_dots[i],100,add=TRUE,col=color_bin);

	#shannon
	curve(exp(x/(coef(mle_dots[[i]])))/
		(4+exp(x/(coef(mle_dots[[i]])))),1,100,add=TRUE,col=color_shan);

	#pnorm
	#curve(pnorm(probit_dots[[i]]$coefficients[1]+probit_dots[[i]]$coefficients[2]*x),1,100,add=TRUE,col=color_prob)

	#save graphs - comment out next two lines to avoid saving
	#dev.copy(pdf,paste("mturk_graph_dots_",i,".pdf",sep=""));
	#dev.off();
}

######
#Multinomial logit of model fits
best_model_rearrange <- rep(NA,n_subj);
for(i in 1:n_subj) {
	if(best_model_dots[i] == 7) {
		best_model_rearrange[i] <- 0;
	} else {
		best_model_rearrange[i] <- best_model_dots[i];
	}
}
demo_model_choice <- multinom(best_model_rearrange ~ age + gender);


#Multinomial logit of model fits, no probit
best_model_np_rearrange <- rep(NA,n_subj);
for(i in 1:n_subj) {
	if(best_model_noprobit_dots[i] == 7) {
		best_model_np_rearrange[i] <- 0;
	} else {
		best_model_np_rearrange[i] <- best_model_noprobit_dots[i];
	}
}
demo_model_np_choice <- multinom(best_model_np_rearrange ~ age + gender);
z_np_choice <- summary(demo_model_np_choice)$coefficients/summary(demo_model_np_choice)$standard.errors;
p_np_choice <- 2*(1-pnorm(abs(z_np_choice),0,1));


######
######

######
######
#KOLMOGOROV-SMIRNOV TESTS (CONOVER CORRECTION)
dot_counts = c(38,39,40,41,42);
dist_totals_dots <- matrix(nrow=n_subj,ncol=2*length(dot_counts)-1); #actual counts of (directed) distances
truth_totals_dots <- matrix(nrow=n_subj,ncol=length(dot_counts)); #totals in each state
error_shannon_dots <- matrix(nrow=n_subj,ncol=2*length(dot_counts)-1); #error distance frequencies for each subject under Shannon assumptions, empirical distribution of states
cum_error_shannon_dots <- matrix(nrow=n_subj,ncol=2*length(dot_counts)-1); #cumulative of previous line
for(i in 1:n_subj) {
	for(j in -(length(dot_counts)-1):(length(dot_counts)-1)) {
		dist_totals_dots[i,j+length(dot_counts)] <- sum(dist_dots[i,]==j);
	}	
 
	for(k in 1:length(dot_counts)) {
		truth_totals_dots[i,k] <- sum(true_dots[i,]==dot_counts[k]);
	}
	
	for(j in -(length(dot_counts)-1):(length(dot_counts)-1)) {
		if(j == 0) {
			error_shannon_dots[i,j+length(dot_counts)] <- mean(correct_dots[i,]);
		}
		else {
			error_shannon_dots[i,j+length(dot_counts)] <- 1/num_tasks*1/(length(dot_counts)-1)*(1-mean(correct_dots[i,]))*sum(truth_totals_dots[i,max(1,1-j):min(5,5-j)]);
		}

		cum_error_shannon_dots[i,j+length(dot_counts)] <- sum(error_shannon_dots[i,1:(j+length(dot_counts))]);
	}
}
#add 0 to start of cumulative error distribution
cum_error_shannon_dots <- cbind(0,cum_error_shannon_dots);

#create cdfs for comparison and test
shannon_cdf_dots <- as.list(mat.or.vec(n_subj,1));
ks_test_dots <- as.list(mat.or.vec(n_subj,1));
ks_stat_dots <- mat.or.vec(n_subj,1);
ks_pval_dots <- mat.or.vec(n_subj,1);
for(i in 1:n_subj) {
	shannon_cdf_dots[[i]] <- stepfun(-4:4, cum_error_shannon_dots[i,]);
	ks_test_dots[[i]] <- ks.test(dist_dots[i,],shannon_cdf_dots[[i]],simulate.p.value=TRUE);	
	ks_stat_dots[i] <- ks_test_dots[[i]]$statistic;
	ks_pval_dots[i] <- ks_test_dots[[i]]$p.value;
}

#check for significance
ks_pval_signif <- (ks_pval_dots<=0.05)+0;


######
######
#print out tables
aic_tables <- cbind(1:118, aic_matrix_dots, best_model_dots, best_model_noprobit_dots);
stargazer(aic_tables);
