#grab coefficients and get summary statistics for responsive subjects

#MODEL 1 (constant)
model1_alpha <- as.numeric(lapply(mean_reg_dots,coef));
model1_alpha_resp <- model1_alpha[str_mon_subj];
model1_alpha_resp_mean <- mean(model1_alpha_resp);
model1_alpha_resp_stdev <- sd(model1_alpha_resp);
model1_alpha_resp_med <- median(model1_alpha_resp);


grab_coef <- function(x) {
	return(as.vector(coef(x)));
}

#MODEL 2 (break model)
#beta are baseline and jump; delta is break location
model2_beta <- t(sapply(lapply(breaks_reg_noslope_opt_dots,grab_coef),as.numeric));
model2_beta_resp <- model2_beta[str_mon_subj,];
model2_beta_resp_mean <- apply(model2_beta_resp,2,mean);
model2_beta_resp_stdev <- apply(model2_beta_resp,2,sd);
model2_beta_resp_median <- apply(model2_beta_resp,2,median);

model2_delta <- breaks_reg_noslope_optinc_dots;
model2_delta_resp <- model2_delta[str_mon_subj];
model2_delta_resp_mean <- mean(model2_delta_resp);
model2_delta_resp_stdev <- sd(model2_delta_resp);
model2_delta_resp_med <- median(model2_delta_resp);


#MODEL 3 (breaks with slope)
#beta are baseline, jump, and slopes; delta is break location
library(strucchange);
model3_delta <- mat.or.vec(n_subj,1);
model3_beta0 <- mat.or.vec(n_subj,1);
model3_beta1 <- mat.or.vec(n_subj,1);
model3_beta2 <- mat.or.vec(n_subj,1);
model3_beta3 <- mat.or.vec(n_subj,1);

for(i in 1:n_subj) {
	model3_delta[i] <- breaks_dots[[i]]$breakpoints;
	model3_beta0[i] <- breaks_reg_dots_sum[[i]]$coefficients[1,1];
	model3_beta1[i] <- breaks_reg_dots_sum[[i]]$coefficients[2,1];
	#grab break coefficients for subjects estimated to have breaks
	if(!is.na(model3_delta[i])) {
		model3_beta2[i] <- breaks_reg_dots_sum[[i]]$coefficients[3,1];
		model3_beta3[i] <- breaks_reg_dots_sum[[i]]$coefficients[4,1];
	} else {
		model3_beta2[i] <- NA;
		model3_beta3[i] <- NA;
	}
}

model3_delta_resp <- model3_delta[str_mon_subj];
model3_delta_resp_mean <- mean(model3_delta_resp,na.rm=TRUE);
model3_delta_resp_stdev <- sd(model3_delta_resp,na.rm=TRUE);
model3_delta_resp_med <- median(model3_delta_resp,na.rm=TRUE);

model3_beta0_resp <- model3_beta0[str_mon_subj];
model3_beta0_resp_mean <- mean(model3_beta0_resp,na.rm=TRUE);
model3_beta0_resp_stdev <- sd(model3_beta0_resp,na.rm=TRUE);
model3_beta0_resp_med <- median(model3_beta0_resp,na.rm=TRUE);

model3_beta1_resp <- model3_beta1[str_mon_subj];
model3_beta1_resp_mean <- mean(model3_beta1_resp,na.rm=TRUE);
model3_beta1_resp_stdev <- sd(model3_beta1_resp,na.rm=TRUE);
model3_beta1_resp_med <- median(model3_beta1_resp,na.rm=TRUE);

model3_beta2_resp <- model3_beta2[str_mon_subj];
model3_beta2_resp_mean <- mean(model3_beta2_resp,na.rm=TRUE);
model3_beta2_resp_stdev <- sd(model3_beta2_resp,na.rm=TRUE);
model3_beta2_resp_med <- median(model3_beta2_resp,na.rm=TRUE);

model3_beta3_resp <- model3_beta3[str_mon_subj];
model3_beta3_resp_mean <- mean(model3_beta3_resp,na.rm=TRUE);
model3_beta3_resp_stdev <- sd(model3_beta3_resp,na.rm=TRUE);
model3_beta3_resp_med <- median(model3_beta3_resp,na.rm=TRUE);


#MODEL 4 (linear)
model4_beta <- t(sapply(lapply(reg_dots,grab_coef),as.numeric));
model4_beta_resp <- model4_beta[str_mon_subj,];
model4_beta_resp_mean <- apply(model4_beta_resp,2,mean);
model4_beta_resp_stdev <- apply(model4_beta_resp,2,sd);
model4_beta_resp_median <- apply(model4_beta_resp,2,median);


#MODEL 5 (quadratic)
model5_beta <- t(sapply(lapply(lpm2_reg_dots,grab_coef),as.numeric));
model5_beta_resp <- model5_beta[str_mon_subj,];
model5_beta_resp_mean <- apply(model5_beta_resp,2,mean);
model5_beta_resp_stdev <- apply(model5_beta_resp,2,sd);
model5_beta_resp_median <- apply(model5_beta_resp,2,median);


#MODEL 6 (cubic)
model6_beta <- t(sapply(lapply(lpm3_reg_dots,grab_coef),as.numeric));
model6_beta_resp <- model6_beta[str_mon_subj,];
model6_beta_resp_mean <- apply(model6_beta_resp,2,mean);
model6_beta_resp_stdev <- apply(model6_beta_resp,2,sd);
model6_beta_resp_median <- apply(model6_beta_resp,2,median);


#MODEL 7 (Shannon)
library(bbmle);
library(stats4);
model7_alpha <- as.numeric(lapply(mle_dots,coef));
model7_alpha_resp <- model7_alpha[str_mon_subj];
model7_alpha_resp_mean <- mean(model7_alpha_resp);
model7_alpha_resp_stdev <- sd(model7_alpha_resp);
model7_alpha_resp_med <- median(model7_alpha_resp);


#MODEL 8 (Normal)
model8_alpha <- as.numeric(lapply(normsig_dots,coef));
model8_alpha_resp <- model8_alpha[str_mon_subj];
model8_alpha_resp_mean <- mean(model8_alpha_resp);
model8_alpha_resp_stdev <- sd(model8_alpha_resp);
model8_alpha_resp_med <- median(model8_alpha_resp);


#MODEL 9 (Tsallis)
model9_alphasigma <- t(sapply(lapply(tsallis_dots,grab_coef),as.numeric));
model9_alphasigma_resp <- model9_alphasigma[str_mon_subj,];
model9_alphasigma_resp_mean <- apply(model9_alphasigma_resp,2,mean);
model9_alphasigma_resp_stdev <- apply(model9_alphasigma_resp,2,sd);
model9_alphasigma_resp_median <- apply(model9_alphasigma_resp,2,median);