###PROBABILITY WEIGHTED ESTIMATIONS OF MODELS

#load relevant libraries in case they haven't already been loaded
library(stats4);
library(bbmle);
library(nleqslv);


##set weighting functions
weight_pr <- function(x,b) {
	exp(-(-log(x))^b);
}

weight_risk <- function(x,b) {
	x^b;
}


#set indicator function
indic <- function(threshold,arg) {
	if (arg>=threshold) {
		return(1);
	} else {
		return(0);
	}
}

#ensure that incentives can be interpreted as probabilities
inc_dots_div <- 1/100*inc_dots;

##SHANNON
#PRELEC WEIGHTS
mle_dots_pr <- as.list(mat.or.vec(n_subj,1));
for(i in str_mon_subj) {
	ll_shannon_dots_pr <- function(a,g) {
		-sum(
			correct_dots[i,]*(
				(inc_dots_div[i,]^g)/((inc_dots_div[i,]^g+(1-inc_dots_div[i,])^g)^(1/g))/a
				-log(						length(dot_counts)-1+exp((inc_dots_div[i,]^g)/((inc_dots_div[i,]^g+(1-inc_dots_div[i,])^g)^(1/g))/a)
				)
			)+(1-correct_dots[i,])*(
				log(length(dot_counts)-1)
				-log(					length(dot_counts)-1+exp((inc_dots_div[i,]^g)/((inc_dots_div[i,]^g+(1-inc_dots_div[i,])^g)^(1/g))/a)
				)
			)
		);
	}
	try(mle_dots_pr[[i]] <- mle2(ll_shannon_dots_pr,method="SANN",start=list(a=0.9,g=0.7)));
	#for box contraints: method="L-BFGS-B",lower=list(a=0,g=0),upper=list(a=Inf,g=1) but will need to adjust starting points for each individual
	
	#debug, comment out if you want
	print(i);
}



#RISK WEIGHTS
mle_dots_rs <- as.list(mat.or.vec(n_subj,1));
for(i in str_mon_subj) {
	ll_shannon_dots_rs <- function(a,g) {
		-sum(
			correct_dots[i,]*(
				(inc_dots_div[i,]^g)/a
				-log(
					length(dot_counts)-1+exp((inc_dots_div[i,]^g)/a)
				)
			)+(1-correct_dots[i,])*(
				log(length(dot_counts)-1)
				-log(
					length(dot_counts)-1+exp((inc_dots_div[i,]^g)/a)
				)
			)
		);
	}
	try(mle_dots_rs[[i]] <- mle2(ll_shannon_dots_rs,method="SANN",start=list(a=0.99,g=0.9)));
	#for constraints replace with L-BFGS-B, lower=list(a=0,g=0), start=list(a=0.99,g=0.9)
}


######
#check what's going on for strictly rational (responsive) subjects

#pr
coeff_table_pr <- mat.or.vec(length(str_mon_subj),2);
for (i in 1:length(str_mon_subj)) {
	coeff_table_pr[i,] <- as.vector(coef(mle_dots_pr[[str_mon_subj[i]]]));
}
coeff_table_pr <- cbind(str_mon_subj,coeff_table_pr);

#RISK
coeff_table_rs <- mat.or.vec(length(str_mon_subj),2);
for (i in 1:length(str_mon_subj)) {
	coeff_table_rs[i,] <- as.vector(coef(mle_dots_rs[[str_mon_subj[i]]]));
}
coeff_table_rs <- cbind(str_mon_subj,coeff_table_rs);



##################
###QUADRATIC COSTS
quad_dots_pr <- as.list(mat.or.vec(n_subj,1));
for(i in str_mon_subj) {
	ll_quad_dots_pr <- function(a,b,g) {
		-sum(
			correct_dots[i,]*log(
				a+b*(inc_dots_div[i,]^g)/((inc_dots_div[i,]^g+(1-inc_dots_div[i,])^g)^(1/g))
			)+(1-correct_dots[i,])*log(
				1-a-b*(inc_dots_div[i,]^g)/((inc_dots_div[i,]^g+(1-inc_dots_div[i,])^g)^(1/g))
			)
		);
	}
	try(quad_dots_pr[[i]] <- mle2(ll_quad_dots_pr,method="SANN",start=list(a=0.2,b=0.5,g=0.7)));
}

quad_dots_rs <- as.list(mat.or.vec(n_subj,1));
for(i in str_mon_subj) {
	ll_quad_dots_rs <- function(a,b,g) {
		-sum(
			correct_dots[i,]*log(
				a+b*(inc_dots_div[i,]^g)
			)+(1-correct_dots[i,])*log(
				1-a-b*(inc_dots_div[i,]^g)
			)
		);
	}
	try(quad_dots_rs[[i]] <- mle2(ll_quad_dots_rs,method="SANN",start=list(a=0.2,b=0.5,g=0.7)));
}

#add to coefficient table
#pr
coeff_quad_table_pr <- mat.or.vec(length(str_mon_subj),3);
for (i in 1:length(str_mon_subj)) {
	coeff_quad_table_pr[i,] <- as.vector(coef(quad_dots_pr[[str_mon_subj[i]]]));
}
coeff_quad_table_pr <- cbind(str_mon_subj,coeff_quad_table_pr);

#RISK
coeff_quad_table_rs <- mat.or.vec(length(str_mon_subj),3);
for (i in 1:length(str_mon_subj)) {
	coeff_quad_table_rs[i,] <- as.vector(coef(quad_dots_rs[[str_mon_subj[i]]]));
}
coeff_quad_table_rs <- cbind(str_mon_subj,coeff_quad_table_rs);


##################
###2ND DEGREE POLYNOMIAL PERFORMANCE
poly2_dots_pr <- as.list(mat.or.vec(n_subj,1));
for(i in str_mon_subj) {
	ll_poly2_dots_pr <- function(a,b,c,g) {
		-sum(
			correct_dots[i,]*log(
				a+b*(inc_dots_div[i,]^g)/((inc_dots_div[i,]^g+(1-inc_dots_div[i,])^g)^(1/g))+c*(inc_dots_div[i,]^g)/((inc_dots_div[i,]^g+(1-inc_dots_div[i,])^g)^(1/g))^2
			)+(1-correct_dots[i,])*log(
				1-a-b*(inc_dots_div[i,]^g)/((inc_dots_div[i,]^g+(1-inc_dots_div[i,])^g)^(1/g))-c*(inc_dots_div[i,]^g)/((inc_dots_div[i,]^g+(1-inc_dots_div[i,])^g)^(1/g))^2
			)
		);
	}
	try(poly2_dots_pr[[i]] <- mle2(ll_poly2_dots_pr,method="SANN",start=list(a=0.2,b=0.2,c=0.1,g=0.7)));
}

poly2_dots_rs <- as.list(mat.or.vec(n_subj,1));
for(i in str_mon_subj) {
	ll_poly2_dots_rs <- function(a,b,c,g) {
		-sum(
			correct_dots[i,]*log(
				a+b*(inc_dots_div[i,]^g)+c*(inc_dots_div[i,]^g)^2
			)+(1-correct_dots[i,])*log(
				1-a-b*(inc_dots_div[i,]^g)-c*(inc_dots_div[i,]^g)^2
			)
		);
	}
	try(poly2_dots_rs[[i]] <- mle2(ll_poly2_dots_rs,method="SANN",start=list(a=0.2,b=0.5,c=0.1,g=0.7)));
}

#add to coefficient table
#pr
coeff_poly2_table_pr <- mat.or.vec(length(str_mon_subj),4);
for (i in 1:length(str_mon_subj)) {
	coeff_poly2_table_pr[i,] <- as.vector(coef(poly2_dots_pr[[str_mon_subj[i]]]));
}
coeff_poly2_table_pr <- cbind(str_mon_subj,coeff_poly2_table_pr);

#RISK
coeff_poly2_table_rs <- mat.or.vec(length(str_mon_subj),4);
for (i in 1:length(str_mon_subj)) {
	coeff_poly2_table_rs[i,] <- as.vector(coef(poly2_dots_rs[[str_mon_subj[i]]]));
}
coeff_poly2_table_rs <- cbind(str_mon_subj,coeff_poly2_table_rs);




##################
###3RD DEGREE POLYNOMIAL PERFORMANCE
poly3_dots_pr <- as.list(mat.or.vec(n_subj,1));
for(i in str_mon_subj) {
	ll_poly3_dots_pr <- function(a,b,c,d,g) {
		-sum(
			correct_dots[i,]*log(
				a+b*(inc_dots_div[i,]^g)/((inc_dots_div[i,]^g+(1-inc_dots_div[i,])^g)^(1/g))+c*(inc_dots_div[i,]^g)/((inc_dots_div[i,]^g+(1-inc_dots_div[i,])^g)^(1/g))^2++d*(inc_dots_div[i,]^g)/((inc_dots_div[i,]^g+(1-inc_dots_div[i,])^g)^(1/g))^3
			)+(1-correct_dots[i,])*log(
				1-a-b*(inc_dots_div[i,]^g)/((inc_dots_div[i,]^g+(1-inc_dots_div[i,])^g)^(1/g))-c*(inc_dots_div[i,]^g)/((inc_dots_div[i,]^g+(1-inc_dots_div[i,])^g)^(1/g))^2-d*(inc_dots_div[i,]^g)/((inc_dots_div[i,]^g+(1-inc_dots_div[i,])^g)^(1/g))^3
			)
		);
	}
	try(poly3_dots_pr[[i]] <- mle2(ll_poly3_dots_pr,method="SANN",start=list(a=0.2,b=0.2,c=0.1,d=0.05,g=0.7)));
}

poly3_dots_rs <- as.list(mat.or.vec(n_subj,1));
for(i in str_mon_subj) {
	ll_poly3_dots_rs <- function(a,b,c,d,g) {
		-sum(
			correct_dots[i,]*log(
				a+b*(inc_dots_div[i,]^g)+c*(inc_dots_div[i,]^g)^2+d*(inc_dots_div[i,]^g)^3
			)+(1-correct_dots[i,])*log(
				1-a-b*(inc_dots_div[i,]^g)-c*(inc_dots_div[i,]^g)^2-d*(inc_dots_div[i,]^g)^3
			)
		);
	}
	try(poly3_dots_rs[[i]] <- mle2(ll_poly3_dots_rs,method="SANN",start=list(a=0.2,b=0.5,c=0.1,d=0.05,g=0.7)));
}

#add to coefficient table
#pr
coeff_poly3_table_pr <- mat.or.vec(length(str_mon_subj),5);
for (i in 1:length(str_mon_subj)) {
	coeff_poly3_table_pr[i,] <- as.vector(coef(poly3_dots_pr[[str_mon_subj[i]]]));
}
coeff_poly3_table_pr <- cbind(str_mon_subj,coeff_poly3_table_pr);

#RISK
coeff_poly3_table_rs <- mat.or.vec(length(str_mon_subj),5);
for (i in 1:length(str_mon_subj)) {
	coeff_poly3_table_rs[i,] <- as.vector(coef(poly3_dots_rs[[str_mon_subj[i]]]));
}
coeff_poly3_table_rs <- cbind(str_mon_subj,coeff_poly3_table_rs);




##################
###MODEL WITH BREAK
break_dots_pr <- as.list(mat.or.vec(n_subj,1));
for(i in str_mon_subj) {
	ll_break_dots_pr <- function(a,b,c,d,e,g) {
		likelihood_object <- 0;
		for(j in 1:100) {
			likelihood_object <- likelihood_object - correct_dots[i,j]*log(a+b*weight_pr(inc_dots_div[i,j],g)+c*indic(e,weight_pr(inc_dots_div[i,j],g))+d*indic(e,weight_pr(inc_dots_div[i,j],g))*weight_pr(inc_dots_div[i,j],g)) - (1-correct_dots[i,j])*log(1-a-b*weight_pr(inc_dots_div[i,j],g)-c*indic(e,weight_pr(inc_dots_div[i,j],g))-d*indic(e,weight_pr(inc_dots_div[i,j],g))*weight_pr(inc_dots_div[i,j],g))
		}
		return(likelihood_object);
	}
	try(break_dots_pr[[i]] <- mle2(ll_break_dots_pr,method="SANN",start=list(a=0.2,b=0.5,c=0.1,d=0,e=0.5,g=0.7)));
	print(i); #debug, comment out as necessary
}

break_dots_rs <- as.list(mat.or.vec(n_subj,1));
for(i in str_mon_subj) {
	ll_break_dots_rs <- function(a,b,c,d,e,g) {
		likelihood_object <- 0;
		for(j in 1:100) {
			likelihood_object <- likelihood_object - correct_dots[i,j]*log(a+b*inc_dots_div[i,j]^g+c*indic(e,inc_dots_div[i,j]^g)+d*indic(e,inc_dots_div[i,j]^g)*inc_dots_div[i,j]^g) - (1-correct_dots[i,j])*log(1-a-b*inc_dots_div[i,j]^g-c*indic(e,inc_dots_div[i,j]^g)-d*indic(e,inc_dots_div[i,j]^g)*inc_dots_div[i,j]^g)
		}
		return(likelihood_object);
	}
	try(break_dots_rs[[i]] <- mle2(ll_break_dots_rs,method="SANN",start=list(a=0.2,b=0.5,c=0.1,d=0,e=0.5,g=0.7)));
	print(i); #debug, comment out as necessary
}

#add to coefficient table
#pr
coeff_break_table_pr <- mat.or.vec(length(str_mon_subj),6);
for (i in 1:length(str_mon_subj)) {
	coeff_break_table_pr[i,] <- as.vector(coef(break_dots_pr[[str_mon_subj[i]]]));
}
coeff_break_table_pr <- cbind(str_mon_subj,coeff_break_table_pr);

#RISK
coeff_break_table_rs <- mat.or.vec(length(str_mon_subj),6);
for (i in 1:length(str_mon_subj)) {
	coeff_break_table_rs[i,] <- as.vector(coef(break_dots_rs[[str_mon_subj[i]]]));
}
coeff_break_table_rs <- cbind(str_mon_subj,coeff_break_table_rs);


##################
###TSALLIS
tolerance_perf <- 0.00001; #sets distance performance can be from 0 or 1
tolerance_sigma <- 0.00001; #ensures there's no "blow-up" in the vicinity of sigma=1

#Prelec
tsallis_dots_pr <- as.list(mat.or.vec(n_subj,1));
for (i in str_mon_subj) {
	ll_tsal_dots_pr <- function(alpha, sigma, g) {
		like <- 0; #negative log likelihood

		for(j in 1:num_tasks) {
			if(abs(sigma-1)<tolerance_sigma) {
				perf_soln <- exp(weight_pr(inc_dots_div[i,j],g)/alpha)/(4+exp(weight_pr(inc_dots_div[i,j],g)/alpha));
			} else {
			
				#solve for performance
				solve_perf <- function(perf) {
					return(
						alpha*sigma/(sigma-1)*(((1-perf)/4)^(sigma-1)-perf^(sigma-1))+weight_pr(inc_dots_div[i,j],g)
					);
				}
				
				perf_soln <- try(nleqslv(0.5,solve_perf)$x);
				#Looking at the graph, the only way for rootfinding to fail is if the root is greater than 1 (since "roots" smaller than 0.2 will never be chosen). If performance larger than 1 is computed, then this resets it to just below 1 so that a likelihood can actually be computed
				if("try-error" %in% class(perf_soln) | perf_soln >= 1) {
					perf_soln <- 1-tolerance_perf; 
				}
			}
			
			like <- like - correct_dots[i,j]*log(perf_soln) - (1-correct_dots[i,j])*log(1-perf_soln);
		}
			
		return(like);		
	}

	#mle step
	try(tsallis_dots_pr[[i]] <- mle2(ll_tsal_dots_pr,start=list(alpha=1,sigma=0.8,g=0.7),method="SANN"));
	#debug, comment out
	print(i);
}


#risk
tsallis_dots_rs <- as.list(mat.or.vec(n_subj,1));
for (i in str_mon_subj) {
	ll_tsal_dots_rs <- function(alpha, sigma, g) {
		like <- 0; #negative log likelihood

		for(j in 1:num_tasks) {
			if(abs(sigma-1)<tolerance_sigma) {
				perf_soln <- exp(weight_risk(inc_dots_div[i,j],g)/alpha)/(4+exp(weight_risk(inc_dots_div[i,j],g)/alpha));
			} else {
			
				#solve for performance
				solve_perf <- function(perf) {
					return(
						alpha*sigma/(sigma-1)*(((1-perf)/4)^(sigma-1)-perf^(sigma-1))+weight_risk(inc_dots_div[i,j],g)
					);
				}
				
				perf_soln <- try(nleqslv(0.5,solve_perf)$x);
				#Looking at the graph, the only way for rootfinding to fail is if the root is greater than 1 (since "roots" smaller than 0.2 will never be chosen). If performance larger than 1 is computed, then this resets it to just below 1 so that a likelihood can actually be computed
				if("try-error" %in% class(perf_soln) | perf_soln >= 1) {
					perf_soln <- 1-tolerance_perf; 
				}
			}
			
			like <- like - correct_dots[i,j]*log(perf_soln) - (1-correct_dots[i,j])*log(1-perf_soln);
		}
			
		return(like);		
	}

	#mle step
	try(tsallis_dots_rs[[i]] <- mle2(ll_tsal_dots_rs,start=list(alpha=1,sigma=0.8,g=0.7),method="SANN"));
	#debug, comment out
	print(i);
}

#add to coefficient table
#pr
coeff_tsal_table_pr <- mat.or.vec(length(str_mon_subj),3);
for (i in 1:length(str_mon_subj)) {
	coeff_tsal_table_pr[i,] <- as.vector(coef(tsallis_dots_pr[[str_mon_subj[i]]]));
}
coeff_tsal_table_pr <- cbind(str_mon_subj,coeff_tsal_table_pr);

#RISK
coeff_tsal_table_rs <- mat.or.vec(length(str_mon_subj),3);
for (i in 1:length(str_mon_subj)) {
	coeff_tsal_table_rs[i,] <- as.vector(coef(tsallis_dots_rs[[str_mon_subj[i]]]));
}
coeff_tsal_table_rs <- cbind(str_mon_subj,coeff_tsal_table_rs);



##################
###NORMAL SIGNALS
norm_dots_pr <- as.list(mat.or.vec(n_subj,1));
for(i in str_mon_subj) {
	ll_norm_dots_pr <- function(a,g) {
		like <- 0; #negative log likelihood

		for(j in 1:num_tasks) {
			#solve for sqrt of precision
			solve_sqrt_prec <- function(sqrt_prec) {
				return(a*sqrt_prec - 2/5*dnorm(0.5*sqrt_prec)*weight_pr(inc_dots_div[i,j],g));
			}
			sqrt_prec_soln <- nleqslv(0.1,solve_sqrt_prec)$x;

			#if true state is 38 or 42, then there's one likelihood (because on edges)
			#if true state is 39, 40, or 41, then there's another likelihood (because in interior)

			if(true_dots[i,j] %in% c(38,42)) {
				if(correct_dots[i,j]==1) {
					like <- like - log(pnorm(0.5,sd=1/sqrt_prec_soln));
				} else {
					like <- like - log(pnorm(0.5,sd=1/sqrt_prec_soln,lower.tail=FALSE));
				}
			} else {
				if(correct_dots[i,j]==1) {
					like <- like - log(pnorm(0.5,sd=1/sqrt_prec_soln)-pnorm(-0.5,sd=1/sqrt_prec_soln));
				} else {
					like <- like - log(2*pnorm(-0.5,sd=1/sqrt_prec_soln));
				}
			}
		}
		return(like);		
	}

	#mle step
	try(norm_dots_pr[[i]] <- mle2(ll_norm_dots_pr,method="SANN",start=list(a=0.01,g=12)));
	print(i); #debug, comment out as necessary
}

norm_dots_rs <- as.list(mat.or.vec(n_subj,1));
for(i in str_mon_subj) {
	ll_norm_dots_rs <- function(a,g) {
		like <- 0; #negative log likelihood

		for(j in 1:num_tasks) {
			#solve for sqrt of precision
			solve_sqrt_prec <- function(sqrt_prec) {
				return(a*sqrt_prec - 2/5*dnorm(0.5*sqrt_prec)*weight_risk(inc_dots_div[i,j],g));
			}
			sqrt_prec_soln <- nleqslv(2,solve_sqrt_prec)$x;

			#if true state is 38 or 42, then there's one likelihood (because on edges)
			#if true state is 39, 40, or 41, then there's another likelihood (because in interior)

			if(true_dots[i,j] %in% c(38,42)) {
				if(correct_dots[i,j]==1) {
					like <- like - log(pnorm(0.5,sd=1/sqrt_prec_soln));
				} else {
					like <- like - log(pnorm(0.5,sd=1/sqrt_prec_soln,lower.tail=FALSE));
				}
			} else {
				if(correct_dots[i,j]==1) {
					like <- like - log(pnorm(0.5,sd=1/sqrt_prec_soln)-pnorm(-0.5,sd=1/sqrt_prec_soln));
				} else {
					like <- like - log(2*pnorm(-0.5,sd=1/sqrt_prec_soln));
				}
			}
		}
		return(like);		
	}

	#mle step
	try(norm_dots_rs[[i]] <- mle2(ll_norm_dots_rs,method="SANN",start=list(a=0.05,g=0.7)));
	print(i); #debug, comment out as necessary
}

#add to coefficient table
#pr
coeff_norm_table_pr <- mat.or.vec(length(str_mon_subj),2);
for (i in 1:length(str_mon_subj)) {
	coeff_norm_table_pr[i,] <- as.vector(coef(norm_dots_pr[[str_mon_subj[i]]]));
}
coeff_norm_table_pr <- cbind(str_mon_subj,coeff_norm_table_pr);

#RISK
coeff_norm_table_rs <- mat.or.vec(length(str_mon_subj),2);
for (i in 1:length(str_mon_subj)) {
	coeff_norm_table_rs[i,] <- as.vector(coef(norm_dots_rs[[str_mon_subj[i]]]));
}
coeff_norm_table_rs <- cbind(str_mon_subj,coeff_norm_table_rs);





#########TO EDIT

#AIC and classification calculations
num_models_pw <- 7;

#3. quadratic costs
#4. general break model
#5. 2nd degree poly perf
#6. 3rd degree poly perf
#7. shannon
#8. tsallis
#9. normal signals

aic_matrix_dots_pr <- mat.or.vec(n_subj,num_models_pw);
aic_matrix_dots_rs <- mat.or.vec(n_subj,num_models_pw);
best_model_dots_pr <- mat.or.vec(n_subj,1);
best_model_dots_rs <- mat.or.vec(n_subj,1);
rank_models_dots_pr <- mat.or.vec(n_subj,num_models_pw);
rank_models_dots_rs <- mat.or.vec(n_subj,num_models_pw);

best_model_dots_all <- mat.or.vec(n_subj,1);

for(i in 1:n_subj) {
	try(aic_matrix_dots_pr[i,1] <- AIC(quad_dots_pr[[i]]));
	try(aic_matrix_dots_rs[i,1] <- AIC(quad_dots_rs[[i]]));
	
	try(aic_matrix_dots_pr[i,2] <- AIC(break_dots_pr[[i]]));
	try(aic_matrix_dots_rs[i,2] <- AIC(break_dots_rs[[i]]));
	
	try(aic_matrix_dots_pr[i,3] <- AIC(poly2_dots_pr[[i]]));
	try(aic_matrix_dots_rs[i,3] <- AIC(poly2_dots_rs[[i]]));
	
	try(aic_matrix_dots_pr[i,4] <- AIC(poly3_dots_pr[[i]]));
	try(aic_matrix_dots_rs[i,4] <- AIC(poly3_dots_rs[[i]]));
	
	try(aic_matrix_dots_pr[i,5] <- AIC(mle_dots_pr[[i]]));
	try(aic_matrix_dots_rs[i,5] <- AIC(mle_dots_rs[[i]]));
	
	try(aic_matrix_dots_pr[i,6] <- AIC(tsallis_dots_pr[[i]]));
	try(aic_matrix_dots_rs[i,6] <- AIC(tsallis_dots_rs[[i]]));
	
	try(aic_matrix_dots_pr[i,7] <- AIC(norm_dots_pr[[i]]));
	try(aic_matrix_dots_rs[i,7] <- AIC(norm_dots_rs[[i]]));
	
	#best models within each type of probability weighting
	#add 2 to for easy comparison to earlier analysis
	try(best_model_dots_pr[i] <- 2+which.min(setdiff(aic_matrix_dots_pr[i,],c(0))));
	try(best_model_dots_rs[i] <- 2+which.min(setdiff(aic_matrix_dots_rs[i,],c(0))));
	
	#rankings for each type of pw
	#WARNING: AS WRITTEN, WILL RANK pr WITH 0 AIC (THE NON-CONVERGERS) AS HAVING NORMAL SIGNALS OR TSALLIS AS BEST FIT
	try(rank_models_dots_pr[i,] <- rank(aic_matrix_dots_pr[i,]));
	try(rank_models_dots_rs[i,] <- rank(aic_matrix_dots_rs[i,]));
	
	#no pw, rs, pr
	best_model_dots_all[i] <- which.min(setdiff(c(aic_matrix_dots[i,],aic_matrix_dots_rs[i,],aic_matrix_dots_pr[i,]),c(0)));
}

#which performance function is it
best_perf_fun <- best_model_dots_all*(best_model_dots_all<=9) + ((best_model_dots_all %% 9)+2)*((best_model_dots_all>9) & (best_model_dots_all<=16)) + (((best_model_dots_all+2) %% 9)+2)*(best_model_dots_all>16);

best_model_table <- cbind(best_model_dots,best_model_dots_pr,best_model_dots_rs)[str_mon_subj,];
